/*
*                            Copyright (c) 2001
*                        <B>Politecnico di Torino</B>
*                            All Rights Reserved
* <p>
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that: (1) source code distributions
* retain the above copyright notice and this paragraph in its entirety,(2)
* distributions including binary code include the above copyright notice and
* this paragraph in its entirety in the documentation or other materials
* provided with the distribution, and (3) all advertising materials mentioning
* features or use of this software display the following acknowledgement:
* ``This product includes software developed by Politecnico
* di Torino, and its contributors.'' (4) Neither the name of
* the University nor the names of its contributors may be used to endorse
* or promote products derived from this software without specific prior
* written permission.
* THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
* This product includes code licensed from RSA Data Security (Java Secure Socket Extension)
*
* Sun grants you ("Licensee") a non-exclusive, royalty free, license to use, and redistribute
* this software graphics artwork, as individual graphics or as a collection, as part of
* software code or programs that you develop, provided that
* i) this copyright notice and license accompany the software graphics artwork; and
* ii) you do not utilize the software graphics artwork in a manner which is disparaging to Sun.
* Unless enforcement is prohibited by applicable law, you may not modify the graphics,
* and must use them true to color and unmodified in every way.
*
*<p>
*/
package Interfaccia;

import Utility.BusinessModelComponent;
import Utility.ProjectType;

import java.awt.Canvas;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.awt.Point;
import java.awt.geom.Ellipse2D;
import java.awt.Image;
import java.awt.Dimension;
import java.awt.Color;

import java.util.Vector;
import java.util.Enumeration;

import javax.swing.JComponent;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseMotionAdapter;

/**
 * Definisce l'immagine del Business Model
 */
class BusinessModelImage extends JComponent {
	/**
	 * Contiene tutti i componenti del Business Model
	 */
	private Vector businessModelComponents=new Vector();
	/**
	 * Contiene l'immagine del Business Model utile per la tecnica del double buffering
	 */
	private Image offScreenImg= null;
	/**
	 * 	Definisce la larghezza di dell'immagine del Business Model
	 */
	final static int width = 655;
	/**
	 * Definisce l'altezza dell'immagine del Business Model
	 */
	final static int height = 375;
	/**
	 * Definisce la traslazione lungo X che l'immagine deve subire per essere centrata
	 * all'interno dello spazio di visualizzazione
	 */
	private static int traslX;
	/**
	 * Definisce la traslazione lungo Y che l'immagine deve subire per essere centrata
	 * all'interno dello spazio di visualizzazione
	 */
	private static int traslY;
	/**
	 * Ascoltatore dei click del mouse sull'immagine del Busines Model
	 */
	protected MouseAdapter mouseListener=null;
	/**
	 * Ascoltatore dei movimenti del mouse sull'immagine del Busines Model
	 */
	protected MouseMotionAdapter mouseMotionListener=null;
	/**
	 * Riferimento al progetto a cui l'immagine  associata
	 */
	 private ProjectViewing currentPrj=null;

	/**
	 * Costruttore
	 */
	public BusinessModelImage(Image offScreenImg,ProjectViewing currentPrj) {
		super();

		this.currentPrj=currentPrj;
		this.setDoubleBuffered(true);

		// imposto la dimensione pari a quella del diagramma mio business model
		// tenendo conto anche del bordo attorno ad esso
		Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
		this.traslY=10;
		this.traslX=java.lang.Math.abs((int)((screenSize.getWidth()-30-this.width)/2));
		this.setPreferredSize(new Dimension(this.width+2*this.traslX,this.height+2*this.traslY));

		// carico in memoria il Business Model generico
		this.fillDefaultBusinessModelComponents();

		// disegno su schermo il Business Model generico
		this.offScreenImg=offScreenImg;
		this.createImage();

		this.setVisible(false);
    }

	/**
	 * Override del metodo repaint di JComponent
	 */
	 public void repaint()  {
//		System.out.println("ridisegno");
		this.offScreenImg=null;
		super.repaint();
	 }

  /**
   * Ridisegna l'immagine del Business Model sullo schermo
   */
  public void paintComponent(Graphics g) {
	if(this.offScreenImg==null)  {
		this.offScreenImg= this.createImage(this.width,this.height);
		this.createImage();
	}
	try {
		g.drawImage(offScreenImg,this.traslX,this.traslY,this);
	}catch(NullPointerException e)  {	}
  }

	/**
	 * Crea l'immagine disegnandola in offScreenImg
	 */
	private void createImage()  {
		Graphics2D offScreen=(Graphics2D) this.offScreenImg.getGraphics();
		offScreen.setColor(Color.white);
	    offScreen.fillRect(0,0,(int)this.getPreferredSize().getWidth(),(int)this.getPreferredSize().getHeight());

		 // disegno tutti i componenti
		 BusinessModelComponent businessComponent=null;
		 java.util.Enumeration componentList= this.businessModelComponents.elements();
		 while(componentList.hasMoreElements())  {
			businessComponent= (BusinessModelComponent)componentList.nextElement();
			businessComponent.paint(offScreen);
		 }
	}

	/**
	 * Carica tutti i componenti del Business Model all'interno di businessModelComponents
	 */
	private void fillDefaultBusinessModelComponents()   {
		// in questo modo l'immagine  posto esattamente contro i bordi sup e inf dell'interfaccia
		int xTx= 45;
		int yTx=76;

		Vector businessElement= new Vector();;
		GeneralPath path =null;
		short[] busModCompIDLinks =null;

		// business role
		businessElement.add(new Ellipse2D.Float(45-xTx, 229-yTx, 117, 54));
		businessElement.add(new String("Consumer"));
		businessElement.add(new java.awt.Point(55-xTx, 263-yTx));
		busModCompIDLinks = new short[5];
		busModCompIDLinks[0]=ProjectType.BUSINESS_MODEL_RELATIONSHIP_BROKERAGE_CONSUMER;
		busModCompIDLinks[1]=ProjectType.BUSINESS_MODEL_RELATIONSHIP_SERVICE_USAGE_SERV_CONT_PROV;
		busModCompIDLinks[2]=ProjectType.BUSINESS_MODEL_RELATIONSHIP_ACCESS_MANAGEMENT;
		busModCompIDLinks[3]=ProjectType.BUSINESS_MODEL_RELATIONSHIP_USAGE_NET_SERVICES_CONSUMER;
		busModCompIDLinks[4]=ProjectType.BUSINESS_MODEL_RELATIONSHIP_SERVICE_USAGE_CONNECTIVITY_PROV;
		this.businessModelComponents.add(new BusinessModelComponent(ProjectType.BUSINESS_MODEL_ROLE_CONSUMER,false,true,busModCompIDLinks,businessElement,null));

		businessElement.removeAllElements();
		businessElement.add(new Ellipse2D.Float(383-xTx, 228-yTx, 168, 54));
		businessElement.add(new String("Service/Content"));
		businessElement.add(new java.awt.Point(395-xTx, 257-yTx));
		businessElement.add(new String("Provider"));
		businessElement.add(new java.awt.Point(429-xTx, 274-yTx));
		busModCompIDLinks = new short[5];
		busModCompIDLinks[0]=ProjectType.BUSINESS_MODEL_RELATIONSHIP_BROKERAGE_SERV_CONT_PROVIDER;
		busModCompIDLinks[1]=ProjectType.BUSINESS_MODEL_RELATIONSHIP_SERVICE_USAGE_SERV_CONT_PROV;
		busModCompIDLinks[2]=ProjectType.BUSINESS_MODEL_RELATIONSHIP_ACCESS_MANAGEMENT;
		busModCompIDLinks[3]=ProjectType.BUSINESS_MODEL_RELATIONSHIP_3rdPARTY_SERVICE_PROVISIONING;
		busModCompIDLinks[4]=ProjectType.BUSINESS_MODEL_RELATIONSHIP_USAGE_NET_SERVICES_SERV_CONT_PROVIDER;
		this.businessModelComponents.add(new BusinessModelComponent(ProjectType.BUSINESS_MODEL_ROLE_SERVICE_CONTENT_PROVIDER,false,true,busModCompIDLinks,businessElement,null));

		businessElement.removeAllElements();
		businessElement.add(new Ellipse2D.Float(544-xTx, 76-yTx, 117, 54));
		businessElement.add(new String("Broker"));
		businessElement.add(new java.awt.Point(573-xTx, 110-yTx));
		busModCompIDLinks = new short[5];
		busModCompIDLinks[0]=ProjectType.BUSINESS_MODEL_RELATIONSHIP_BROKERAGE_CONSUMER;
		busModCompIDLinks[1]=ProjectType.BUSINESS_MODEL_RELATIONSHIP_BROKERAGE_SERV_CONT_PROVIDER;
		busModCompIDLinks[2]=ProjectType.BUSINESS_MODEL_RELATIONSHIP_BROKERAGE_CONNECTIVITY_PROVIDER;
		busModCompIDLinks[3]=ProjectType.BUSINESS_MODEL_RELATIONSHIP_BROKERAGE_BROKER;
		busModCompIDLinks[4]=ProjectType.BUSINESS_MODEL_RELATIONSHIP_BROKERAGE_BROKER_BIS;
		this.businessModelComponents.add(new BusinessModelComponent(ProjectType.BUSINESS_MODEL_ROLE_BROKER,false,true,busModCompIDLinks,businessElement,null));

		businessElement.removeAllElements();
		businessElement.add(new Ellipse2D.Float(546-xTx, 366-yTx, 117, 54));
		businessElement.add(new String("Connectiv."));
		businessElement.add(new java.awt.Point(557-xTx, 393-yTx));
		businessElement.add(new String("Provider"));
		businessElement.add(new java.awt.Point(566-xTx, 409-yTx));
		busModCompIDLinks = new short[7];
		busModCompIDLinks[0]=ProjectType.BUSINESS_MODEL_RELATIONSHIP_BROKERAGE_CONNECTIVITY_PROVIDER;
		busModCompIDLinks[1]=ProjectType.BUSINESS_MODEL_RELATIONSHIP_3rdPARTY_SERVICE_PROVISIONING;
		busModCompIDLinks[2]=ProjectType.BUSINESS_MODEL_RELATIONSHIP_USAGE_NET_SERVICES_SERV_CONT_PROVIDER;
		busModCompIDLinks[3]=ProjectType.BUSINESS_MODEL_RELATIONSHIP_USAGE_NET_SERVICES_CONSUMER;
		busModCompIDLinks[4]=ProjectType.BUSINESS_MODEL_RELATIONSHIP_SERVICE_USAGE_CONNECTIVITY_PROV;
		busModCompIDLinks[5]=ProjectType.BUSINESS_MODEL_RELATIONSHIP_USAGE_NET_SERVICES_CONNECTIVITY_PROVIDER;
		busModCompIDLinks[6]=ProjectType.BUSINESS_MODEL_RELATIONSHIP_USAGE_NET_SERVICES_CONNECTIVITY_PROVIDER_BIS;
		this.businessModelComponents.add(new BusinessModelComponent(ProjectType.BUSINESS_MODEL_ROLE_CONNECTIVITY_PROVIDER,false,true,busModCompIDLinks,businessElement,null));

		// business relationship
		businessElement.removeAllElements();
		path = new GeneralPath(new Line2D.Double(108-xTx, 105-yTx, 537-xTx, 105-yTx));
	    path.append(new Line2D.Double(108-xTx, 106-yTx, 537-xTx, 106-yTx),true);
	    path.append(new Line2D.Double(108-xTx, 105-yTx, 108-xTx, 222-yTx),true);
	    path.append(new Line2D.Double(109-xTx, 105-yTx, 109-xTx, 222-yTx),true);
		businessElement.add(path);
		businessElement.add(new Rectangle2D.Float(260-xTx,85-yTx,140,22));
		businessElement.add(new String("Brokerage"));
		businessElement.add(new java.awt.Point(290-xTx, 103-yTx));
		busModCompIDLinks = new short[2];
		busModCompIDLinks[0]=ProjectType.BUSINESS_MODEL_ROLE_CONSUMER;
		busModCompIDLinks[1]=ProjectType.BUSINESS_MODEL_ROLE_BROKER;
	    //ref point consumer
		businessElement.add(new Ellipse2D.Float(103-xTx,223-yTx,10,10));
	    //ref broker
	    businessElement.add(new Ellipse2D.Float(538-xTx,100-yTx,10,10));
		this.businessModelComponents.add(new BusinessModelComponent(ProjectType.BUSINESS_MODEL_RELATIONSHIP_BROKERAGE_CONSUMER,false,false,busModCompIDLinks,businessElement,this.businessModelComponents));

		businessElement.removeAllElements();
		path = new GeneralPath(new Line2D.Double(476-xTx, 222-yTx, 571-xTx, 130-yTx));
	    path.append(new Line2D.Double(475-xTx, 222-yTx, 570-xTx, 130-yTx),true);
	    businessElement.add(path);
		businessElement.add(new Rectangle2D.Float(500-xTx,185-yTx,100,22));
		businessElement.add(new String("Brokerage"));
		businessElement.add(new java.awt.Point(510-xTx, 202-yTx));
		busModCompIDLinks = new short[2];
		busModCompIDLinks[0]=ProjectType.BUSINESS_MODEL_ROLE_SERVICE_CONTENT_PROVIDER;
		busModCompIDLinks[1]=ProjectType.BUSINESS_MODEL_ROLE_BROKER;
	    //ref point serv. cont prov
		businessElement.add(new Ellipse2D.Float(467-xTx,222-yTx,10,10));
	    //ref point broker
		businessElement.add(new Ellipse2D.Float(570-xTx,122-yTx,10,10));
		this.businessModelComponents.add(new BusinessModelComponent(ProjectType.BUSINESS_MODEL_RELATIONSHIP_BROKERAGE_SERV_CONT_PROVIDER,false,false,busModCompIDLinks,businessElement,this.businessModelComponents));

		businessElement.removeAllElements();
		path = new GeneralPath(new Line2D.Double(604-xTx, 359-yTx, 604-xTx, 136-yTx));
	    path.append(new Line2D.Double(603-xTx, 359-yTx, 603-xTx, 136-yTx),true);
	    businessElement.add(path);
		businessElement.add(new Rectangle2D.Float(604-xTx,212-yTx,95,22));
		businessElement.add(new String("Brokerage"));
		businessElement.add(new java.awt.Point(605-xTx, 250-yTx));
		busModCompIDLinks = new short[2];
		busModCompIDLinks[0]=ProjectType.BUSINESS_MODEL_ROLE_CONNECTIVITY_PROVIDER;
		busModCompIDLinks[1]=ProjectType.BUSINESS_MODEL_ROLE_BROKER;
	    //ref point conn. prov.
	    businessElement.add(new Ellipse2D.Float(599-xTx,360-yTx,10,10));
	    //ref point broker
	    businessElement.add(new Ellipse2D.Float(599-xTx,126-yTx,10,10));
		this.businessModelComponents.add(new BusinessModelComponent(ProjectType.BUSINESS_MODEL_RELATIONSHIP_BROKERAGE_CONNECTIVITY_PROVIDER,false,false,busModCompIDLinks,businessElement,this.businessModelComponents));

		businessElement.removeAllElements();
		path = new GeneralPath(new Line2D.Double(666-xTx, 103-yTx, 687-xTx, 103-yTx));
	    path.append(new Line2D.Double(666-xTx, 104-yTx, 687-xTx, 104-yTx),true);
	    path.append(new Line2D.Double(687-xTx, 103-yTx, 687-xTx, 158-yTx),true);
	    path.append(new Line2D.Double(686-xTx, 103-yTx, 686-xTx, 158-yTx),true);
	    path.append(new Line2D.Double(687-xTx, 158-yTx, 632-xTx, 158-yTx),true);
	    path.append(new Line2D.Double(687-xTx, 157-yTx, 632-xTx, 157-yTx),true);
	    path.append(new Line2D.Double(632-xTx, 158-yTx, 632-xTx, 132-yTx),true);
	    path.append(new Line2D.Double(633-xTx, 158-yTx, 633-xTx, 132-yTx),true);
	    businessElement.add(path);
		businessElement.add(new Rectangle2D.Float(606-xTx,157-yTx,90,22));
		businessElement.add(new String("Brokerage"));
		businessElement.add(new java.awt.Point(606-xTx, 174-yTx));
		busModCompIDLinks = new short[1];
		busModCompIDLinks[0]=ProjectType.BUSINESS_MODEL_ROLE_BROKER;
	    //ref point broker
	    businessElement.add(new Ellipse2D.Float(656-xTx,98-yTx,10,10));
		this.businessModelComponents.add(new BusinessModelComponent(ProjectType.BUSINESS_MODEL_RELATIONSHIP_BROKERAGE_BROKER,false,false,busModCompIDLinks,businessElement,this.businessModelComponents));

		businessElement.removeAllElements();
		path = new GeneralPath(new Line2D.Double(147-xTx, 231-yTx, 147-xTx, 219-yTx));
	    path.append(new Line2D.Double(146-xTx, 231-yTx, 146-xTx, 219-yTx),true);
	    path.append(new Line2D.Double(147-xTx, 219-yTx, 397-xTx, 219-yTx),true);
	    path.append(new Line2D.Double(147-xTx, 220-yTx, 397-xTx, 220-yTx),true);
	    path.append(new Line2D.Double(397-xTx, 219-yTx, 397-xTx, 233-yTx),true);
	    path.append(new Line2D.Double(398-xTx, 219-yTx, 398-xTx, 233-yTx),true);
	    businessElement.add(path);
		businessElement.add(new Rectangle2D.Float(210-xTx,198-yTx,130,22));
		businessElement.add(new String("Service Usage"));
		businessElement.add(new java.awt.Point(210-xTx, 215-yTx));
		busModCompIDLinks = new short[2];
		busModCompIDLinks[0]=ProjectType.BUSINESS_MODEL_ROLE_CONSUMER;
		busModCompIDLinks[1]=ProjectType.BUSINESS_MODEL_ROLE_SERVICE_CONTENT_PROVIDER;
	    //ref point consumer
	    businessElement.add(new Ellipse2D.Float(142-xTx,232-yTx,10,10));
	    //ref point serv.cont prov
	    businessElement.add(new Ellipse2D.Float(392-xTx,234-yTx,10,10));
		this.businessModelComponents.add(new BusinessModelComponent(ProjectType.BUSINESS_MODEL_RELATIONSHIP_SERVICE_USAGE_SERV_CONT_PROV,false,false,busModCompIDLinks,businessElement,this.businessModelComponents));

		businessElement.removeAllElements();
		path = new GeneralPath(new Line2D.Double(397-xTx, 276-yTx, 397-xTx, 293-yTx));
	    path.append(new Line2D.Double(398-xTx, 276-yTx, 398-xTx, 293-yTx),true);
	    path.append(new Line2D.Double(398-xTx, 293-yTx, 147-xTx, 293-yTx),true);
	    path.append(new Line2D.Double(398-xTx, 294-yTx, 147-xTx, 294-yTx),true);
	    path.append(new Line2D.Double(147-xTx, 294-yTx, 147-xTx, 280-yTx),true);
	    path.append(new Line2D.Double(146-xTx, 294-yTx, 146-xTx, 280-yTx),true);
	    businessElement.add(path);
		businessElement.add(new Rectangle2D.Float(160-xTx,272-yTx,210,22));
		businessElement.add(new String("Access Management"));
		businessElement.add(new java.awt.Point(180-xTx, 289-yTx));
		busModCompIDLinks = new short[2];
		busModCompIDLinks[0]=ProjectType.BUSINESS_MODEL_ROLE_CONSUMER;
		busModCompIDLinks[1]=ProjectType.BUSINESS_MODEL_ROLE_SERVICE_CONTENT_PROVIDER;
	    //ref point consumer
	    businessElement.add(new Ellipse2D.Float(142-xTx,270-yTx,10,10));
	    //ref point serv.cont prov
	    businessElement.add(new Ellipse2D.Float(392-xTx,266-yTx,10,10));
		this.businessModelComponents.add(new BusinessModelComponent(ProjectType.BUSINESS_MODEL_RELATIONSHIP_ACCESS_MANAGEMENT,false,false,busModCompIDLinks,businessElement,this.businessModelComponents));

		businessElement.removeAllElements();
		path = new GeneralPath(new Line2D.Double(415-xTx,282-yTx,415-xTx,376-yTx));
	    path.append(new Line2D.Double(414-xTx,282-yTx,414-xTx,376-yTx),true);
	    path.append(new Line2D.Double(415-xTx,375-yTx,553-xTx,375-yTx),true);
	    path.append(new Line2D.Double(415-xTx,376-yTx,553-xTx,376-yTx),true);
	    businessElement.add(path);
		businessElement.add(new Rectangle2D.Float(415-xTx,323-yTx,108,52));
		businessElement.add(new String("3rd Party"));
		businessElement.add(new java.awt.Point(418-xTx, 339-yTx));
		businessElement.add(new String("Service"));
		businessElement.add(new java.awt.Point(418-xTx, 356-yTx));
		businessElement.add(new String("Provisioning"));
		businessElement.add(new java.awt.Point(418-xTx, 372-yTx));
		busModCompIDLinks = new short[2];
		busModCompIDLinks[0]=ProjectType.BUSINESS_MODEL_ROLE_SERVICE_CONTENT_PROVIDER;
		busModCompIDLinks[1]=ProjectType.BUSINESS_MODEL_ROLE_CONNECTIVITY_PROVIDER;
	    //ref point serv. cont prov
	    businessElement.add(new Ellipse2D.Float(410-xTx,272-yTx,10,10));
	    //ref point conn prov
	    businessElement.add(new Ellipse2D.Float(554-xTx,370-yTx,10,10));
		this.businessModelComponents.add(new BusinessModelComponent(ProjectType.BUSINESS_MODEL_RELATIONSHIP_3rdPARTY_SERVICE_PROVISIONING,false,false,busModCompIDLinks,businessElement,this.businessModelComponents));

		businessElement.removeAllElements();
		path = new GeneralPath(new Line2D.Double(536-xTx, 276-yTx, 587-xTx, 361-yTx));
	    path.append(new Line2D.Double(536-xTx, 277-yTx, 586-xTx, 361-yTx),true);
	    businessElement.add(path);
		businessElement.add(new Rectangle2D.Float(435-xTx,283-yTx,130,35));
		businessElement.add(new String("Usage of"));
		businessElement.add(new java.awt.Point(456-xTx, 298-yTx));
		businessElement.add(new String("Net. Services"));
		businessElement.add(new java.awt.Point(437-xTx, 316-yTx));
		busModCompIDLinks = new short[2];
		busModCompIDLinks[0]=ProjectType.BUSINESS_MODEL_ROLE_SERVICE_CONTENT_PROVIDER;
		busModCompIDLinks[1]=ProjectType.BUSINESS_MODEL_ROLE_CONNECTIVITY_PROVIDER;
	    //ref point serv. cont prov
	    businessElement.add(new Ellipse2D.Float(528-xTx,268-yTx,10,10));
	    //ref point conn prov
	    businessElement.add(new Ellipse2D.Float(586-xTx,361-yTx,10,10));
		this.businessModelComponents.add(new BusinessModelComponent(ProjectType.BUSINESS_MODEL_RELATIONSHIP_USAGE_NET_SERVICES_SERV_CONT_PROVIDER,false,false,busModCompIDLinks,businessElement,this.businessModelComponents));

		businessElement.removeAllElements();
		path = new GeneralPath(new Line2D.Double(114-xTx, 390-yTx, 539-xTx, 390-yTx));
	    path.append(new Line2D.Double(114-xTx, 391-yTx, 539-xTx, 391-yTx),true);
	    path.append(new Line2D.Double(114-xTx, 390-yTx, 114-xTx, 289-yTx),true);
	    path.append(new Line2D.Double(115-xTx, 390-yTx, 115-xTx, 289-yTx),true);
	    businessElement.add(path);
		businessElement.add(new Rectangle2D.Float(150-xTx,370-yTx,240,20));
		businessElement.add(new String("Usage of Network Services"));
		businessElement.add(new java.awt.Point(150-xTx, 387-yTx));
		busModCompIDLinks = new short[2];
		busModCompIDLinks[0]=ProjectType.BUSINESS_MODEL_ROLE_CONSUMER;
		busModCompIDLinks[1]=ProjectType.BUSINESS_MODEL_ROLE_CONNECTIVITY_PROVIDER;
	    //ref point consumer
	    businessElement.add(new Ellipse2D.Float(109-xTx,279-yTx,10,10));
	    //ref point conn prov
	    businessElement.add(new Ellipse2D.Float(540-xTx,385-yTx,10,10));
		this.businessModelComponents.add(new BusinessModelComponent(ProjectType.BUSINESS_MODEL_RELATIONSHIP_USAGE_NET_SERVICES_CONSUMER,false,false,busModCompIDLinks,businessElement,this.businessModelComponents));

		businessElement.removeAllElements();
		path = new GeneralPath(new Line2D.Double(87-xTx, 411-yTx, 553-xTx, 411-yTx));
	    path.append(new Line2D.Double(87-xTx, 412-yTx, 553-xTx, 412-yTx),true);
	    path.append(new Line2D.Double(87-xTx, 411-yTx, 87-xTx, 288-yTx),true);
	    path.append(new Line2D.Double(88-xTx, 411-yTx, 88-xTx, 288-yTx),true);
	    businessElement.add(path);
		businessElement.add(new Rectangle2D.Float(220-xTx,412-yTx,150,20));
		businessElement.add(new String("Service Usage"));
		businessElement.add(new java.awt.Point(230-xTx, 428-yTx));
		busModCompIDLinks = new short[2];
		busModCompIDLinks[0]=ProjectType.BUSINESS_MODEL_ROLE_CONSUMER;
		busModCompIDLinks[1]=ProjectType.BUSINESS_MODEL_ROLE_CONNECTIVITY_PROVIDER;
	    //ref point cunsumer
	    businessElement.add(new Ellipse2D.Float(82-xTx,278-yTx,10,10));
	    //ref point conn. prov
	    businessElement.add(new Ellipse2D.Float(554-xTx,406-yTx,10,10));
		this.businessModelComponents.add(new BusinessModelComponent(ProjectType.BUSINESS_MODEL_RELATIONSHIP_SERVICE_USAGE_CONNECTIVITY_PROV,false,false,busModCompIDLinks,businessElement,this.businessModelComponents));

		businessElement.removeAllElements();
		path = new GeneralPath(new Line2D.Double(669-xTx, 392-yTx, 685-xTx, 392-yTx));
	    path.append(new Line2D.Double(669-xTx, 393-yTx, 685-xTx, 393-yTx),true);
	    path.append(new Line2D.Double(685-xTx, 392-yTx, 685-xTx, 447-yTx),true);
	    path.append(new Line2D.Double(684-xTx, 392-yTx, 684-xTx, 447-yTx),true);
	    path.append(new Line2D.Double(685-xTx, 447-yTx, 630-xTx, 447-yTx),true);
	    path.append(new Line2D.Double(685-xTx, 446-yTx, 630-xTx, 446-yTx),true);
	    path.append(new Line2D.Double(630-xTx, 447-yTx, 630-xTx, 423-yTx),true);
	    path.append(new Line2D.Double(631-xTx, 447-yTx, 631-xTx, 423-yTx),true);
	    businessElement.add(path);
		businessElement.add(new Rectangle2D.Float(390-xTx,423-yTx,250,20));
		businessElement.add(new String("Usage of Network Services"));
		businessElement.add(new java.awt.Point(395-xTx, 440-yTx));
		busModCompIDLinks = new short[1];
		busModCompIDLinks[0]=ProjectType.BUSINESS_MODEL_ROLE_CONNECTIVITY_PROVIDER;
	    //ref point laccio conn. prov.
	    businessElement.add(new Ellipse2D.Float(659-xTx,387-yTx,10,10));
		this.businessModelComponents.add(new BusinessModelComponent(ProjectType.BUSINESS_MODEL_RELATIONSHIP_USAGE_NET_SERVICES_CONNECTIVITY_PROVIDER,false,false,busModCompIDLinks,businessElement,this.businessModelComponents));

		// una bus. rela. fittizia, perch  altro estremo del laccio
		//del Broker. L'unico elemento grafico che viene disegnato  il ref. point
		//il resto rimane solamente per non dover modificare anche le parti
		//relative ai check del mouse sui componenti.
		businessElement.removeAllElements();
		path = new GeneralPath(new Line2D.Double(666-xTx, 103-yTx, 687-xTx, 103-yTx));
	    path.append(new Line2D.Double(666-xTx, 104-yTx, 687-xTx, 104-yTx),true);
	    path.append(new Line2D.Double(687-xTx, 103-yTx, 687-xTx, 158-yTx),true);
	    path.append(new Line2D.Double(686-xTx, 103-yTx, 686-xTx, 158-yTx),true);
	    path.append(new Line2D.Double(687-xTx, 158-yTx, 632-xTx, 158-yTx),true);
	    path.append(new Line2D.Double(687-xTx, 157-yTx, 632-xTx, 157-yTx),true);
	    path.append(new Line2D.Double(632-xTx, 158-yTx, 632-xTx, 127-yTx),true);
	    path.append(new Line2D.Double(633-xTx, 158-yTx, 633-xTx, 127-yTx),true);
	    businessElement.add(path);
		businessElement.add(new Rectangle2D.Float(606-xTx,157-yTx,90,22));
		businessElement.add(new String("Brokerage"));
		businessElement.add(new java.awt.Point(606-xTx, 174-yTx));
		busModCompIDLinks = new short[1];
		busModCompIDLinks[0]=ProjectType.BUSINESS_MODEL_ROLE_BROKER;
	    //ref point broker
	    businessElement.add(new Ellipse2D.Float(627-xTx,122-yTx,10,10));
		this.businessModelComponents.add(new BusinessModelComponent(ProjectType.BUSINESS_MODEL_RELATIONSHIP_BROKERAGE_BROKER_BIS,false,false,busModCompIDLinks,businessElement,this.businessModelComponents));

		// una bus. rela. fittizia, perch  altro estremo del laccio
		//del Connectivity Provider. L'unico elemento grafico che viene disegnato  il ref. point
		//il resto rimane solamente per non dover modificare anche le parti
		//relative ai check del mouse sui componenti.
		businessElement.removeAllElements();
		path = new GeneralPath(new Line2D.Double(664-xTx, 392-yTx, 685-xTx, 392-yTx));
	    path.append(new Line2D.Double(664-xTx, 393-yTx, 685-xTx, 393-yTx),true);
	    path.append(new Line2D.Double(685-xTx, 392-yTx, 685-xTx, 447-yTx),true);
	    path.append(new Line2D.Double(684-xTx, 392-yTx, 684-xTx, 447-yTx),true);
	    path.append(new Line2D.Double(685-xTx, 447-yTx, 630-xTx, 447-yTx),true);
	    path.append(new Line2D.Double(685-xTx, 446-yTx, 630-xTx, 446-yTx),true);
	    path.append(new Line2D.Double(630-xTx, 447-yTx, 630-xTx, 423-yTx),true);
	    path.append(new Line2D.Double(631-xTx, 447-yTx, 631-xTx, 423-yTx),true);
	    businessElement.add(path);
		businessElement.add(new Rectangle2D.Float(530-xTx,423-yTx,150,20));
		businessElement.add(new String("Connection"));
		businessElement.add(new java.awt.Point(532-xTx, 440-yTx));
		busModCompIDLinks = new short[1];
		busModCompIDLinks[0]=ProjectType.BUSINESS_MODEL_ROLE_CONNECTIVITY_PROVIDER;
	    //ref point laccio conn. prov.
	    businessElement.add(new Ellipse2D.Float(625-xTx,413-yTx,10,10));
		this.businessModelComponents.add(new BusinessModelComponent(ProjectType.BUSINESS_MODEL_RELATIONSHIP_USAGE_NET_SERVICES_CONNECTIVITY_PROVIDER_BIS,false,false,busModCompIDLinks,businessElement,this.businessModelComponents));
	}

	void this_mousePressed(MouseEvent e)    {
		BusinessModelComponent businessComponent=null;

		if(e.getModifiers()==MouseEvent.BUTTON1_MASK + MouseEvent.CTRL_MASK)   {
			//sto selezionando pi componenti per il download,
			//ho premuto CTRL + tasto sx del mouse
			Enumeration componentList= this.businessModelComponents.elements();
			while(componentList.hasMoreElements())  {
	    		businessComponent= (BusinessModelComponent)componentList.nextElement();
			    //ctrl se il click  interno al componente
				if(businessComponent.CheckClickMouse(new Point((int)e.getPoint().getX()-this.traslX,(int)e.getPoint().getY()-this.traslY))) {
				    if(!businessComponent.IsAllResourcesLocal())    {
						// se non ci sono delle risorse da scaricare non ha senso fargli prendere
						//alla multi-selezione
						if(businessComponent.IsMultiSelected()) {
						// fa parte di una multi selezione, voglio toglierlo
							businessComponent.SetIsMultiSelected(false);
							this.currentPrj.busModCompMultiSelection.remove(businessComponent);
						}
						else    {
						//non fa parte di una multiselezione, voglio aggiungerlo
							businessComponent.SetIsMultiSelected(true);
							this.currentPrj.busModCompMultiSelection.add(businessComponent);
						}
						if(this.currentPrj.busModCompMultiSelection.isEmpty())  {
							// la multiselezione  vuota
							this.currentPrj.politellaFrame.menuProjectItemDownSelectedComp.setEnabled(false);
							this.currentPrj.politellaFrame.menuPopUpManageItemDownSelectedComp.setEnabled(false);
						}
						else    {
							//la multiselezione  attiva
							this.currentPrj.politellaFrame.menuProjectItemDownSelectedComp.setEnabled(true);
							this.currentPrj.politellaFrame.menuPopUpManageItemDownSelectedComp.setEnabled(true);
						}
						this.repaint();
				    }
				}
			}
		}
		else    {
			switch(e.getModifiers())    {
				case MouseEvent.BUTTON3_MASK:   {
	    			// serve per dare un riferimento all'operazione di managing dei componenti che possono essere avviate dal popup che faccio comprire
//		    		if(this.currentPrj.IsEditing()) {
						Enumeration componentList= this.businessModelComponents.elements();
	    				while(componentList.hasMoreElements())  {
				    		businessComponent= (BusinessModelComponent)componentList.nextElement();
	    				    //ctrl se il click  interno al componente
							if(businessComponent.CheckClickMouse(new Point((int)e.getPoint().getX()-this.traslX,(int)e.getPoint().getY()-this.traslY)))
								this.currentPrj.busModCompManaging=businessComponent;
						}
//  				}
					// visualizzo il menupop di edit
					this.currentPrj.politellaFrame.menuPopUpManageRoot.show(e.getComponent(),e.getX(),e.getY());
					break;
				}
				case MouseEvent.BUTTON1_MASK:   {
					Enumeration componentList= this.businessModelComponents.elements();
					while(componentList.hasMoreElements())  {
			    		businessComponent= (BusinessModelComponent)componentList.nextElement();
					    //ctrl se il click  interno al componente
						if(businessComponent.CheckClickMouse(new Point((int)e.getPoint().getX()-this.traslX,(int)e.getPoint().getY()-this.traslY)))    {
							if(businessComponent.IsActive())    {
							//il componente  incluso nel Business Model
								if(e.getClickCount()== 2)    {
	    							if(this.currentPrj.IsEditing()) {
				    					//sto editando il progetto
										    this.currentPrj.busModCompManaging=businessComponent;
	    							    this.currentPrj.politellaFrame.menuProjectItemEditThisComp_actionPerformed(null);
					    				}
	    						    else    {
										if(!this.currentPrj.IsEditable())   {
											//il progetto  uno di quelli down, voglio scaricare le risorse associate
									        if(!businessComponent.IsAllResourcesLocal())    {
											// ctrl se ho ancora delle risorse da scaricare
												this.currentPrj.busModCompManaging=businessComponent;
											    this.currentPrj.politellaFrame.menuProjectItemDownThisComp_actionPerformed(null);
										    }
										}
								}
								}
							    else    {
									if(e.getClickCount()==1)    {
										if(businessComponent.IsSelected())
											//voglio togliere lo stato di selezionato
											this.currentPrj.ReLoadInfoTree(null);
										else
											//visualizzo le info relative al progetto
											this.currentPrj.ReLoadInfoTree(businessComponent);
										this.repaint();
									}
							    }
							}
							else    {
							//il componente non  incluso nel Business Model
								if(e.getClickCount()==2 && this.currentPrj.IsEditing())    {
									//aggiungo il componente al progetto
								    this.currentPrj.busModCompManaging=businessComponent;
									this.currentPrj.politellaFrame.menuProjectItemAddThisComp_actionPerformed(null);
								}
								//se ci clicco un volta sola non succede nulla
							}
							return;
						}
				    }
	    			break;
				}// chiudo case
			}//chiudo switch
		}//chiudo if ALT_PRESSED
	}

	void this_mouseMoved(MouseEvent e)  {
		BusinessModelComponent businessComponent=null;
		Enumeration componentList= this.businessModelComponents.elements();
		while(componentList.hasMoreElements())  {
			businessComponent= (BusinessModelComponent)componentList.nextElement();
			//ctrl se il cursore  sopra ad un componente
			if(businessComponent.CheckClickMouse(new Point((int)e.getPoint().getX()-this.traslX,(int)e.getPoint().getY()-this.traslY))) {
				if(this.currentPrj.IsEditable())    {
					if(this.currentPrj.IsEditing()) {
	    				//posso aggiungere a modificare dei componenti -> attivo i menu
						if(businessComponent.IsActive())  {
							//il component fa gi parte del prj, posso editarlo
							this.currentPrj.politellaFrame.menuProjectItemAddThisComp.setEnabled(false);
							this.currentPrj.politellaFrame.menuPopUpManageItemAddThisComp.setEnabled(false);
						    this.currentPrj.politellaFrame.menuProjectItemEditThisComp.setEnabled(true);
							this.currentPrj.politellaFrame.menuPopUpManageItemEditThisComp.setEnabled(true);
						}
						else    {
							//il component non fa parte del prj, posso aggiungerlo
						    this.currentPrj.politellaFrame.menuProjectItemEditThisComp.setEnabled(false);
							this.currentPrj.politellaFrame.menuPopUpManageItemEditThisComp.setEnabled(false);
							this.currentPrj.politellaFrame.menuProjectItemAddThisComp.setEnabled(true);
							this.currentPrj.politellaFrame.menuPopUpManageItemAddThisComp.setEnabled(true);
						}
					}
				}
				else    {
					// un progetto down, posso solo scaricare quel componente o tutti i compoenti
					//  -> attivo i componenti
				    if(!businessComponent.IsAllResourcesLocal())    {
						this.currentPrj.politellaFrame.menuProjectItemDownThisComp.setEnabled(true);
						this.currentPrj.politellaFrame.menuPopUpManageItemDownThisComp.setEnabled(true);
				    }
				}
				return;
			}
		}
		//se esco dal while normalmente vuol dire che il mouse  fuori da ogni componente,
		//senza altri ctrl disabilito tutti i menu che posso abilitare sopra
	    this.currentPrj.politellaFrame.menuProjectItemEditThisComp.setEnabled(false);
		this.currentPrj.politellaFrame.menuPopUpManageItemEditThisComp.setEnabled(false);

		this.currentPrj.politellaFrame.menuProjectItemAddThisComp.setEnabled(false);
		this.currentPrj.politellaFrame.menuPopUpManageItemAddThisComp.setEnabled(false);

		this.currentPrj.politellaFrame.menuProjectItemDownThisComp.setEnabled(false);
		this.currentPrj.politellaFrame.menuPopUpManageItemDownThisComp.setEnabled(false);
	}

	/**
	 * Resetta ai valori di default tutti i componenti del Business Model
	 */
	 protected void ResetAll()  {
		this.businessModelComponents=null;

		this.businessModelComponents= new Vector();
		this.fillDefaultBusinessModelComponents();
	 }

	 /**
	  * Imposta l'attributo businessModelComponents al contenuto opportuno
	  */
	 protected void SetBusinessModelComponents(Vector prjBusModelComponents)    {
//		if(prjBusModelComponents.isEmpty()) {
			//  empty se sto visualizzando un progetto in creazione: non ho ancora specificato da quali business component  composto
//			prjBusModelComponents.addAll(this.businessModelComponents);
//		}
//		else    {
		    // in tutti gli altri casi non pu mai essere empty
			Enumeration listDefault=this.businessModelComponents.elements();
			Enumeration listActual=prjBusModelComponents.elements();
			while(listDefault.hasMoreElements())   {
				BusinessModelComponent busModCompDefault = (BusinessModelComponent)listDefault.nextElement();
//				((BusinessModelComponent)listActual.nextElement()).SetReplaceBusModCompGraphicalElements(curr.GetBusModCompGraphicalElements());
				 BusinessModelComponent busModCompActual = (BusinessModelComponent)listActual.nextElement();
				 busModCompActual.SetReplaceBusModCompInfos(busModCompDefault);
				 if(!busModCompActual.IsBusinessRole()) {
					//se  una bus rela allora devo ripristinare le info sui link con i bus role
					//con cui  in relazione
					busModCompActual.SetBusModComp_All_ResourcesForBusRelationship(prjBusModelComponents);
				 }
			}
			this.businessModelComponents.removeAllElements();
			this.businessModelComponents.addAll(prjBusModelComponents);


//		}
	 }

	/**
	 * ctrl se tutte le risorse associate ai diversi bus comp sono locali, viene invocato solo in fase di chiusura di un
	 * prj down.
	 * Se ritorna true il progetto down viene trasferito tra quelli share
	 */
	public boolean CheckAllBusModCompResourcesLocal()   {
		Enumeration list = this.businessModelComponents.elements();
		while(list.hasMoreElements())   {
			BusinessModelComponent curr = (BusinessModelComponent)list.nextElement();
			if(!curr.IsAllResourcesLocal())
			// se anche un solo componente non ha tutte le risorse in locale posso uscdire subito dal metodo
				return false;
		}
		return true;
	}

	Vector GetBusinessModelComponents() {
		return this.businessModelComponents;
	}

}