/*
*                            Copyright (c) 2001
*                        <B>Politecnico di Torino</B>
*                            All Rights Reserved
* <p>
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that: (1) source code distributions
* retain the above copyright notice and this paragraph in its entirety,(2)
* distributions including binary code include the above copyright notice and
* this paragraph in its entirety in the documentation or other materials
* provided with the distribution, and (3) all advertising materials mentioning
* features or use of this software display the following acknowledgement:
* ``This product includes software developed by Politecnico
* di Torino, and its contributors.'' (4) Neither the name of
* the University nor the names of its contributors may be used to endorse
* or promote products derived from this software without specific prior
* written permission.
* THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
* This product includes code licensed from RSA Data Security (Java Secure Socket Extension)
*
* Sun grants you ("Licensee") a non-exclusive, royalty free, license to use, and redistribute
* this software graphics artwork, as individual graphics or as a collection, as part of
* software code or programs that you develop, provided that
* i) this copyright notice and license accompany the software graphics artwork; and
* ii) you do not utilize the software graphics artwork in a manner which is disparaging to Sun.
* Unless enforcement is prohibited by applicable law, you may not modify the graphics,
* and must use them true to color and unmodified in every way.
*
*<p>
*/
package Interfaccia;

import Utility.BusinessModelComponent;
import Utility.PolitellaException;
import Utility.ProjectType;

import java.io.File;
import java.io.IOException;

import java.util.Vector;
import java.util.Enumeration;

import java.awt.Color;
import java.awt.Component;
import java.awt.Point;
import java.awt.BorderLayout;
import java.awt.event.MouseEvent;

import javax.swing.JTree;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JLabel;
import javax.swing.UIManager;
import javax.swing.border.BevelBorder;
import javax.swing.BorderFactory;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.DefaultMutableTreeNode;

/**
   Identifica il progetto attualmente visualizzato
 */
class ProjectViewing {
    /**
       Identificativo del progetto
     */
    private Long prjID = null;
    /**
       Categorie di classificazione a cui il progetto appartiene
     */
    private String[] prjType = new String[2];
    /**
       Descrizione del progetto
     */
    private String prjDescr = null;
    /**
	 * Identificatore del file su cui  salvato il progetto
	 */
	private File prjFile = null;
	/**
       Radice dell'albero contenente le informazioni relative al porgetto e al
	   componente del business model correntemente selezionato
     */
    private DefaultMutableTreeNode prjInfosTreeRoot = null;
	/**
	 * Rappresenta il modello su cui  costruito l'albero delle informazioni
	 */
	private DefaultTreeModel prjInfosTreeModel = null;
	/**
	 * Indica il componente del business model correntemente selezionato e di cui
	 * prjInfosTreeRoot possiede le informazioni
	 */
	private BusinessModelComponent busModCompSelected=null;
	/**
	 * Indica il componente del business model che sto correntemente editando
	 */
	BusinessModelComponent busModCompManaging=null;
	/**
	 * Contiene i componenti del business model facenti parte di una selezione multipla
	 * finalizzata al download delle risorse ad esse associate
	 */
	 Vector busModCompMultiSelection =null;
    /**
       Specifica se sto editando o meno il progetto
     */
    private boolean isEditing = false;
	/**
	 * Specifica se il progetto  editabile:
	 * - 1 progetto share  editabile
	 * - 1 progetto down non  editabile
	 */
	private boolean isEditable = false;
	/**
	 * Specifica se sto visualizzando o meno un progetto
	 */
	private boolean isViewed = false;
	/**
	 * Identificativo dell'interfaccia di Politella
	 */
    PolitellaFrame politellaFrame = null;
	/**
	 * Identificativo dell'immagine del bus model
	 */
    private BusinessModelImage prjBusinessModelImg = null;

    /**
       Costruttore
     */
    public ProjectViewing(PolitellaFrame politellaFrame) {
		this.politellaFrame=politellaFrame;

		this.busModCompMultiSelection = new Vector();

		this.prjBusinessModelImg=new BusinessModelImage(this.politellaFrame.createImage(BusinessModelImage.width,BusinessModelImage.height),this);
		this.politellaFrame.scrollPaneViewPrjBusinessModelComponents.getViewport().add(this.prjBusinessModelImg);

		this.fillPrjInfosTree();
		this.prjInfosTreeModel = new DefaultTreeModel(this.prjInfosTreeRoot);
		this.politellaFrame.treeViewProjectInfos= new JTree(this.prjInfosTreeModel);
	    this.politellaFrame.treeViewProjectInfos.setCellRenderer(new treeViewProjectInfosRender());
    }

	/**
	 * Visualizza l'immagine del Business Model relativo al progetto corrente
	 */
	 protected void SetVisibleBusinessModelImg(boolean state)  {
		this.prjBusinessModelImg.setVisible(state);
	 }
	/**
	 * Restituisce l'ID del prj correntemente visualizzato
	 */
	 public Long GetPrjID()    {
		return this.prjID;
	 }

	/**
	 * Restuituisce il valore dell'attributo isViewed
	 */
	 public boolean IsViewed()  {
		return this.isViewed;
	 }
	 /**
	  * Restituisce il valore dell'attributo isEditing
	  */
	  public boolean IsEditing()  {
		return this.isEditing;
	  }
	  /**
	   * Restituisce il valore dell'attributo isEditable
	   */
	  public boolean IsEditable()  {
		return this.isEditable;
	  }
	/**
	 * Il componente selezionato  stato modificato quindi devo ricaricare le informazioni dell'albero
	 */
	 protected void ReLoadInfoTree(BusinessModelComponent busModCompSelected)   {
		if(this.busModCompSelected!=null)
		//  = null quando carico un progetto
			this.busModCompSelected.SetIsSelected(false);
		this.busModCompSelected=busModCompSelected;
		if(this.busModCompSelected!=null)
		//  = null quando carico un progetto
			this.busModCompSelected.SetIsSelected(true);
		this.fillPrjInfosTree();
		this.prjInfosTreeModel.reload();
	 }
	/**
       Successivamente alla visualizzazione di un progetto, reimposta tutti gli
	   attributi ai valori di default
     */
    protected void ResetProjectViewingObj() {
		this.prjBusinessModelImg.removeMouseListener(this.prjBusinessModelImg.mouseListener);
		this.prjBusinessModelImg.removeMouseMotionListener(this.prjBusinessModelImg.mouseMotionListener);

		this.busModCompManaging=null;
		this.busModCompSelected=null;
		this.CancelMultiSelection();

		this.prjBusinessModelImg.ResetAll();

		this.prjDescr=null;
		this.prjID=null;
		this.prjType=new String[2];
		this.isEditing=false;
		this.isViewed=false;

		this.repaint();
    }

	/**
	 * Annulla la seleziona multipla sul business model
	 */
	 void CancelMultiSelection() {
		if(!this.busModCompMultiSelection.isEmpty())    {
			Enumeration list = this.busModCompMultiSelection.elements();
			while(list.hasMoreElements())   {
				BusinessModelComponent curr = (BusinessModelComponent)list.nextElement();
				curr.SetIsMultiSelected(false);
			}

			this.politellaFrame.menuProjectItemDownSelectedComp.setEnabled(false);
			this.politellaFrame.menuPopUpManageItemDownSelectedComp.setEnabled(false);
			this.busModCompMultiSelection.clear();
		}
	 }

    /**
       Imposta gli attributi ai valori del progetto corrente
     */
    protected void SetProjectViewingObj(Long prjID, String[] prjType, String prjDescr, File prjFile, Vector prjBusModelComponents/*,String prjPeerLocation*/, boolean isEditable/*,boolean isNew*/) throws PolitellaException{
		this.prjID=prjID;
		this.prjFile=prjFile;
		this.prjType=prjType;
		this.prjDescr=prjDescr;

		this.isEditable=isEditable;

		this.prjBusinessModelImg.SetBusinessModelComponents(prjBusModelComponents);

//		if(!isNew)
		// un progetto scaricato o gia creato in precedenza
		//carico le informazioni relative al progetto
//		    XMLProcessor.LoadProject(prjFile,this.prjBusinessModelImg.businessModelComponents);
		//se isNew=true allora sto creando un progetto nuovo, non ho ancora alcun file XML da caricare

/*		if(prjPeerLocation!=null)   {
			Enumeration componentList = this.prjBusinessModelImg.businessModelComponents.elements();
			while(componentList.hasMoreElements())  {
				BusinessModelComponent tmp = (BusinessModelComponent)componentList.nextElement();
				if(tmp.GetBusModCompSourceLoc()!=null)
					tmp.SetBusModCompSourceLoc(prjPeerLocation.concat("@").concat(tmp.GetBusModCompSourceLoc()));
				if(tmp.GetBusModCompBinaryLoc()!=null)
					tmp.SetBusModCompBinaryLoc(prjPeerLocation.concat("@").concat(tmp.GetBusModCompBinaryLoc()));
				if(tmp.GetBusModCompDocLoc()!=null)
					tmp.SetBusModCompDocLoc(prjPeerLocation.concat("@").concat(tmp.GetBusModCompDocLoc()));
			}
		}
*/
		this.prjBusinessModelImg.addMouseListener(this.prjBusinessModelImg.mouseListener= new java.awt.event.MouseAdapter() {
			public void mousePressed(MouseEvent e){
				this_mousePressed(e);
			}
		 });

		this.prjBusinessModelImg.addMouseMotionListener(this.prjBusinessModelImg.mouseMotionListener = new java.awt.event.MouseMotionAdapter() {
			public void mouseMoved(MouseEvent e)    {
				this_mouseMoved(e);
			}
		 });
		this.isViewed=true;

		this.repaint();
    }

	/**
	 * Ridisegna il business model e ricariche le info nel tree
	 */
	void repaint()  {
		this.ReLoadInfoTree(this.busModCompSelected);
		this.prjBusinessModelImg.repaint();
	}

	/**
	 * Imposta il valore dell'attibuto isEditing
	 */
	 protected void SetIsEditing(boolean newState)  {
		this.isEditing=newState;
	 }

	 /**
	  * Passo l'evento click del mouse all'obj BusinessModelImage
	  */
	 private void this_mousePressed(MouseEvent e)   {
		this.prjBusinessModelImg.this_mousePressed(e);
	 }

	 /**
	  * Gestisco il movimento del mouse sull'immagine
	  */
	private void this_mouseMoved(MouseEvent e)  {
		this.prjBusinessModelImg.this_mouseMoved(e);
	}

    /**
       Imposta ai valori di default il contenuto dell'albero
     */
    private void fillPrjInfosTree() {
	    String defaultMsg  = "Not Available";
		String currMsg=null;
		String[] arrayOfMsg=null;

		if(this.prjInfosTreeRoot!=null)
			this.prjInfosTreeRoot.removeAllChildren();
		else
			this.prjInfosTreeRoot =  new DefaultMutableTreeNode("Project");

	    DefaultMutableTreeNode child =  new DefaultMutableTreeNode("Description");
		if(this.prjDescr==null)
			currMsg=defaultMsg;
		else
			currMsg=this.prjDescr;
		DefaultMutableTreeNode leaf =  new DefaultMutableTreeNode(currMsg);

		child.add(leaf);
		this.prjInfosTreeRoot.add(child);

		child =  new DefaultMutableTreeNode("Commercial Model");
		if(this.prjType[0]==null)
			currMsg=defaultMsg;
		else
			currMsg=this.prjType[0];
		leaf =  new DefaultMutableTreeNode(currMsg);
		child.add(leaf);
		this.prjInfosTreeRoot.add(child);

		child =  new DefaultMutableTreeNode("Architectural Model");
		if(this.prjType[1]==null)
			currMsg=defaultMsg;
		else
			currMsg=this.prjType[1];
		leaf =  new DefaultMutableTreeNode(currMsg);
		child.add(leaf);
		this.prjInfosTreeRoot.add(child);

		child =  new DefaultMutableTreeNode("Business Component Selected");
		DefaultMutableTreeNode nipote = null;
		if(this.busModCompSelected==null)   {
		//non ho selezionato nessun componente
		    nipote =  new DefaultMutableTreeNode(defaultMsg);
		}
		else    {
			nipote =  new DefaultMutableTreeNode("Type");
			if(this.busModCompSelected!=null)   {
				if(this.busModCompSelected.IsBusinessRole())
				    leaf =  new DefaultMutableTreeNode("Business Role");
				else
					leaf =  new DefaultMutableTreeNode("Business Relationship");
			}
			else
				leaf =  new DefaultMutableTreeNode(defaultMsg);
			nipote.add(leaf);
			child.add(nipote);

			//info specifiche dell'intero bus comp
			nipote =  new DefaultMutableTreeNode("Description");
			if(this.busModCompSelected!=null)   {
			    if((currMsg=this.busModCompSelected.GetBusModCompDescription())==null)
					currMsg=defaultMsg;
				leaf =  new DefaultMutableTreeNode(currMsg);
			}
			else
				leaf =  new DefaultMutableTreeNode(defaultMsg);
			nipote.add(leaf);
			child.add(nipote);

			boolean addSource_Binary_Infos =false;
			if(this.busModCompSelected==null)
			    addSource_Binary_Infos =true;
			else    {
				if(this.busModCompSelected.IsBusinessRole())
				    addSource_Binary_Infos =true;
			}

			if(addSource_Binary_Infos)  {
				nipote =  new DefaultMutableTreeNode("Source Location");
				if(this.busModCompSelected!=null)   {
				    if((arrayOfMsg=this.busModCompSelected.GetBusModCompSourceLoc())==null)    {
						leaf =  new DefaultMutableTreeNode(defaultMsg);
						nipote.add(leaf);
				    }
					else    {
					    for(int y=0;y<arrayOfMsg.length;y++)    {
				    		leaf =  new DefaultMutableTreeNode(arrayOfMsg[y]);
				    		nipote.add(leaf);
				    	}
					}
				}
				else    {
					leaf =  new DefaultMutableTreeNode(defaultMsg);
					nipote.add(leaf);
				}
				child.add(nipote);

				nipote =  new DefaultMutableTreeNode("Binary Location");
				if(this.busModCompSelected!=null)   {
				    if((arrayOfMsg=this.busModCompSelected.GetBusModCompBinaryLoc())==null)    {
						leaf =  new DefaultMutableTreeNode(defaultMsg);
						nipote.add(leaf);
				    }
					else    {
					    for(int y=0;y<arrayOfMsg.length;y++)    {
				    		leaf =  new DefaultMutableTreeNode(arrayOfMsg[y]);
				    		nipote.add(leaf);
				    	}
					}
				}
				else    {
					leaf =  new DefaultMutableTreeNode(defaultMsg);
					nipote.add(leaf);
				}
				child.add(nipote);
			}

			nipote =  new DefaultMutableTreeNode("Documentation Location");
			if(this.busModCompSelected!=null)   {
			    if((arrayOfMsg=this.busModCompSelected.GetBusModCompDocLoc())==null)    {
					leaf =  new DefaultMutableTreeNode(defaultMsg);
					nipote.add(leaf);
			    }
				else    {
				    for(int y=0;y<arrayOfMsg.length;y++)    {
			    		leaf =  new DefaultMutableTreeNode(arrayOfMsg[y]);
			    		nipote.add(leaf);
			    	}
				}
			}
			else    {
				leaf =  new DefaultMutableTreeNode(defaultMsg);
				nipote.add(leaf);
			}
			child.add(nipote);

			//info specifiche dei singoli gruppi di risorse associate al bus comp
			nipote =  new DefaultMutableTreeNode("Resources");
			if(this.busModCompSelected==null)   {
				leaf =  new DefaultMutableTreeNode(defaultMsg);
				nipote.add(leaf);
			}
			else    {
				short[] busModCompIDLinks = this.busModCompSelected.GetBusModCompIDLinks();
				DefaultMutableTreeNode nipoteII = null;
				DefaultMutableTreeNode nipoteIII = null;
				for(int i=0;i<busModCompIDLinks.length;i++) {
					if(busModCompIDLinks[i]>=ProjectType.numBusinessRole)
					//il link si riferisce ad una bus relationship
					//il +2 serve perch nell'elenco dei nomi dei bus comp ho due volte un campo Any prima delle bus rela
						 nipoteII =  new DefaultMutableTreeNode(ProjectType.prjTypeDescription[2][busModCompIDLinks[i]+2]);
					else
				    //il link si riferisce ad un bus role
					//il +1 serve perch nell'elenco dei nomi dei bus comp ho un campo Any prima dei bus role
						 nipoteII =  new DefaultMutableTreeNode(ProjectType.prjTypeDescription[2][busModCompIDLinks[i]+1]);

					if(this.busModCompSelected.GetBusModComp_1_ResSourceLoc(busModCompIDLinks[i])==null &&
						this.busModCompSelected.GetBusModComp_1_ResBinaryLoc(busModCompIDLinks[i])==null &&
						this.busModCompSelected.GetBusModComp_1_ResDocLoc(busModCompIDLinks[i])==null &&
						this.busModCompSelected.GetBusModComp_1_ResDescription(busModCompIDLinks[i])==null)  {
						// questo link non ha risorse associate
//  					leaf =  new DefaultMutableTreeNode(defaultMsg);

//	    				nipoteII.add(leaf);
					}
					else    {
						//ci sono delle risorse associate
						nipoteIII =  new DefaultMutableTreeNode("Description");
						if((currMsg=this.busModCompSelected.GetBusModComp_1_ResDescription(busModCompIDLinks[i]))==null)
							currMsg=defaultMsg;
						leaf =  new DefaultMutableTreeNode(currMsg);
						nipoteIII.add(leaf);
						nipoteII.add(nipoteIII);

						nipoteIII =  new DefaultMutableTreeNode("Source Location");
				        if((arrayOfMsg=this.busModCompSelected.GetBusModComp_1_ResSourceLoc(busModCompIDLinks[i]))==null)   {
							leaf =  new DefaultMutableTreeNode(defaultMsg);
							nipoteIII.add(leaf);
				        }
						else    {
							for(int y=0;y<arrayOfMsg.length;y++)    {
								leaf =  new DefaultMutableTreeNode(arrayOfMsg[y]);
								nipoteIII.add(leaf);
							}
						}
						nipoteII.add(nipoteIII);

						nipoteIII =  new DefaultMutableTreeNode("Binary Location");
				        if((arrayOfMsg=this.busModCompSelected.GetBusModComp_1_ResBinaryLoc(busModCompIDLinks[i]))==null)   {
							leaf =  new DefaultMutableTreeNode(defaultMsg);
							nipoteIII.add(leaf);
				        }
						else    {
							for(int y=0;y<arrayOfMsg.length;y++)    {
								leaf =  new DefaultMutableTreeNode(arrayOfMsg[y]);
								nipoteIII.add(leaf);
							}
						}
						nipoteII.add(nipoteIII);

						nipoteIII =  new DefaultMutableTreeNode("Documentation Location");
				        if((arrayOfMsg=this.busModCompSelected.GetBusModComp_1_ResDocLoc(busModCompIDLinks[i]))==null) {
							leaf =  new DefaultMutableTreeNode(defaultMsg);
							nipoteIII.add(leaf);
				        }
						else    {
							for(int y=0;y<arrayOfMsg.length;y++)    {
								leaf =  new DefaultMutableTreeNode(arrayOfMsg[y]);
								nipoteIII.add(leaf);
							}
						}
						nipoteII.add(nipoteIII);

			    		nipote.add(nipoteII);
					}
//  	    		nipote.add(nipoteII);
				}
			}
			if(nipote.isLeaf()) {
				//se  una foglia vuol dire che ai link di questo bus comp, non sono state
				//associate risorse, faccio comparire "Not Available"
				leaf =  new DefaultMutableTreeNode(defaultMsg);
				nipote.add(leaf);
			}
		}

		child.add(nipote);

		this.prjInfosTreeRoot.add(child);
    }

	/**
	 * ctrl se tutte le risorse associate ai diversi bus comp sono locali, viene invocato solo in fase di chiusura di un
	 * prj down.
	 * Se ritorna true il progetto down viene trasferito tra quelli share
	 */
	public boolean CheckAllBusModCompResourcesLocal()   {
		return this.prjBusinessModelImg.CheckAllBusModCompResourcesLocal();
	}

	/**
	 * Seleziona tutti i componenti del bus mod per il download
	 */
	 public void SelectAllBusModComp()  {
		this.busModCompMultiSelection = (Vector)this.prjBusinessModelImg.GetBusinessModelComponents().clone();
	 }

	/**
	 * Classe interna per il rendering dell'albero
	 */
	private class treeViewProjectInfosRender extends JPanel implements TreeCellRenderer    {
		protected JTextArea text=null;
		protected JLabel label=null;

		public treeViewProjectInfosRender()   {
			this.setLayout(new BorderLayout(5,0));
			this.add(this.label= new JLabel(),"West");
			this.add(this.text= new JTextArea(),"Center");
			this.setBackground(Color.white);
		}

		public Component getTreeCellRendererComponent(JTree tree, Object value,
			    boolean isSelected, boolean expanded,
				boolean leaf, int row, boolean hasFocus) {
			tree.setRowHeight(20);

			if (leaf) {
				label.setIcon(UIManager.getIcon("Tree.leafIcon"));
				this.text.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED,Color.black,Color.white,new Color(148, 145, 140),new Color(103, 101, 98)));
			} else  {
				this.text.setBorder(BorderFactory.createLineBorder(UIManager.getColor("Tree.background"),1));
				if (expanded)
					label.setIcon(UIManager.getIcon("Tree.openIcon"));
				else
					label.setIcon(UIManager.getIcon("Tree.closedIcon"));
			}
		    if (isSelected) {
				this.text.setBackground(UIManager.getColor("Tree.selectionBackground"));
				this.text.setForeground(UIManager.getColor("Tree.selectionForeground"));
			    if(!leaf)
	    		    this.text.setBorder(BorderFactory.createLineBorder(UIManager.getColor("Tree.selectionBorderColor"),1));
			}
			else    {
		    	this.text.setBackground(UIManager.getColor("Tree.textBackground"));
		    	if(tree.isEnabled())
					this.text.setForeground(UIManager.getColor("Tree.textForeground"));
				else
					this.text.setForeground(Color.lightGray);
			}
			this.text.setText(tree.convertValueToText(value, isSelected,expanded, leaf, row, hasFocus));
			return this;
		}
	}// fine classe treeViewProjectInfosRender
}
