/*
*                            Copyright (c) 2001
*                        <B>Politecnico di Torino</B>
*                            All Rights Reserved
* <p>
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that: (1) source code distributions
* retain the above copyright notice and this paragraph in its entirety,(2)
* distributions including binary code include the above copyright notice and
* this paragraph in its entirety in the documentation or other materials
* provided with the distribution, and (3) all advertising materials mentioning
* features or use of this software display the following acknowledgement:
* ``This product includes software developed by Politecnico
* di Torino, and its contributors.'' (4) Neither the name of
* the University nor the names of its contributors may be used to endorse
* or promote products derived from this software without specific prior
* written permission.
* THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
* This product includes code licensed from RSA Data Security (Java Secure Socket Extension)
*
* Sun grants you ("Licensee") a non-exclusive, royalty free, license to use, and redistribute
* this software graphics artwork, as individual graphics or as a collection, as part of
* software code or programs that you develop, provided that
* i) this copyright notice and license accompany the software graphics artwork; and
* ii) you do not utilize the software graphics artwork in a manner which is disparaging to Sun.
* Unless enforcement is prohibited by applicable law, you may not modify the graphics,
* and must use them true to color and unmodified in every way.
*
*<p>
*/
package Interfaccia;

import Project.ProjectManager;
import Network.NetworkManager;
import Utility.PolitellaException;

import java.util.Enumeration;
import java.util.Vector;
import java.io.File;
import java.io.DataOutputStream;
import java.io.FileOutputStream;
import java.io.DataInputStream;
import java.io.FileInputStream;
import javax.swing.JOptionPane;
import javax.swing.Timer;
import java.awt.event.ActionEvent;
import javax.swing.ImageIcon;

import java.io.IOException;

/**
   Interfaccia utente.
 */
public class UserInterface {
	/**
	 * Identificatore dell'oggetto ProjectManager
	 */
    ProjectManager projectManager;
	/**
	 * Identificatore dell'oggetto NetworkManager
	 */
    private NetworkManager networkManager;
	/**
	 * Identificativo dell'interfaccia di Politella
	 */
	private PolitellaFrame politellaFrame=null;
	/**
	 * Indirizzo di accesso alla rete
	 */
	String accessPeer=null;
    /**
     * Percorso di salvataggio dei progetti
     */
     String prjSavingPath=null;
     /**
      * Timer per l'invio dei messaggi di annuncio
      */
    Timer annuncioTimer=null;

    public UserInterface() {
        //imposto il timer per l'invio degli annunci, di default 1 ogni 30 secondi
    	this.annuncioTimer= new Timer(1,null);
        this.annuncioTimer.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                    annuncioTimer_actionPerformed(e);
            }
        });

        try {
		// carico le impostazioni precedentemente salvate

        	this.politellaFrame=new PolitellaFrame(this);

			this.networkManager=new NetworkManager(this);


			this.LoadSetup();
			this.projectManager=new ProjectManager(prjSavingPath/*,this.networkManager.GetProjectTypeDescription()*/);
			this.SaveSetup();

		}catch(PolitellaException e)    {
			PolitellaException.ProcessException(this.politellaFrame,e);
			if(e.GetType()==PolitellaException.NO_DB_LOAD ||
				e.GetType()==PolitellaException .NO_PROJECT_LOAD)
				System.exit(-1);
		}

        this.prjSavingPath=this.projectManager.GetProjectFile(0,false,false).getAbsolutePath();
        this.politellaFrame.UpdateTables();

		this.politellaFrame.connectionDownloadsInProgressDialog.setVisible(false);

		this.politellaFrame.setVisible(true);
    }

	/**
	 * Carico alcune impostazioni precendetemente salvate
	 * - default TTL
	 * - numero max di connessioni
	 * - frequenza dei messaggi
	 * - access Peer
	 */
	private void LoadSetup()  throws PolitellaException  {
		File setupFile= new File("setup.ini");
		if(setupFile.exists())  {
			//ho gi salvato una seri di impostazioni, posso caricarle
			try {
                DataInputStream setupIn= new DataInputStream(new FileInputStream("setup.ini"));
	    		short defaultMsgTTL=setupIn.readShort();
				short numMaxActiveConnections=setupIn.readShort();
				short numMaxDownloadInCorso=setupIn.readShort();
				int delay = setupIn.readInt();

                java.io.BufferedReader tmp= new java.io.BufferedReader(new java.io.InputStreamReader(setupIn));

                //leggo il percorso di salvataggio dei progetti
                this.prjSavingPath=tmp.readLine();

				try{
                    this.accessPeer=tmp.readLine();
				}catch(IOException e2){
                    //non c'e' la linea relativa al punto di accesso
				}

                tmp.close();
//                setupIn.close();

				this.networkManager.SetDefaultMsgTTL(defaultMsgTTL);
				this.networkManager.SetNumMaxActiveConnections(numMaxActiveConnections);
				this.networkManager.SetNumMaxDownloadInCorso(numMaxDownloadInCorso);
				this.annuncioTimer.setDelay(delay*1000);

//	    		byte[] buf=new byte[256];
//			    int read=setupIn.read(buf);
//				if(read!=-1)
//					this.accessPeer= new String(buf,0,read);
			}catch(IOException e){
				setupFile.delete();
				throw new PolitellaException(PolitellaException.NO_SETUP_LOAD,null);
			}
		}
        else    {
            //e' la prima volta che avvio il programma e devo specificare il percorso dove salvare i progetti
            JOptionPane.showMessageDialog(this.politellaFrame,"You must still specify the project saving path","Warning",JOptionPane.WARNING_MESSAGE,new ImageIcon("Interfaccia/Icons/WarningDialog.gif"));
			new SetupDialog(this.politellaFrame);
        }
	}
	/**
	 * Salvo alcune impostazioni
	 * - default TTL
	 * - numero max di connessioni
	 * - frequenza dei messaggi
	 * - access Peer
	 */
	 private void SaveSetup() throws PolitellaException   {
		File setupFile=new File("setup.ini");
		try {
			DataOutputStream setupOut= new DataOutputStream(new FileOutputStream(setupFile));
			setupOut.writeShort(this.networkManager.GetDefaultMsgTTL());
	    	setupOut.writeShort(this.networkManager.GetNumMaxActiveConnections());
			setupOut.writeShort(this.networkManager.GetNumMaxDownloadInCorso());
			setupOut.writeInt(this.annuncioTimer.getDelay()/1000);
            setupOut.writeBytes(this.projectManager.GetProjectFile(0,false,false).getAbsolutePath());
            setupOut.writeBytes("\n");
	    	if(this.accessPeer!=null)
				setupOut.writeBytes(this.accessPeer);
			setupOut.close();
		}catch(IOException e){
			setupFile.delete();
			throw new PolitellaException(PolitellaException.NO_SETUP_SAVE,null);
			}
	 }

    /**
       Automaticamente all'avvio dell'applicazione oppure manualmente tramite l'utente, il software tenta di connettersi ad almeno un altro peer per accedere alla rete.
       @roseuid 3AED58F302A6
     */
    public boolean NewConnect() {
		if(this.accessPeer!=null)   {
			if(this.accessPeer.compareTo("")!=0)    {
				try {
			        this.networkManager.NewMsgConnection(this.accessPeer);
                    //avvio il timer di invio dei PING
//                    this.SetStatusAnnuncioTimer(true);
					return true;
				}catch(PolitellaException e)    {
	    			// l'indirizzo specificato per la connessione  errato, devo modificarlo
		    		PolitellaException.ProcessException(this.politellaFrame,e);
					return false;
	    		}
			}
			else    {
	    		//non ho inserito l'access peer
				try {
				//serve per far in modo che il dialog successivo si posizioni sopra la
				//dialog con lo slider di attesa
					Thread.sleep(1);
				}catch(java.lang.InterruptedException e)    {}
				JOptionPane.showMessageDialog(this.politellaFrame,"You must insert the IP address of the access peer","Warning",JOptionPane.WARNING_MESSAGE,new ImageIcon("Interfaccia/Icons/WarningDialog.gif"));
		        return false;
			}
		}
		else    {
			//non ho inserito l'access peer
			try {
			//serve per far in modo che il dialog successivo si posizioni sopra la
			//dialog con lo slider di attesa
				Thread.sleep(1);
			}catch(java.lang.InterruptedException e)    {}
			JOptionPane.showMessageDialog(this.politellaFrame,"You must insert the IP address of the access peer","Warning",JOptionPane.WARNING_MESSAGE,new ImageIcon("Interfaccia/Icons/WarningDialog.gif"));
		    return false;
		}
    }

    /**
       Aggiorna la quantita di progetti sicuramente reperibili ( a conoscenza solamente di alcuni peer dell'intera rete e solamente di questi conosce la quantit di progetti condivisi).
       @roseuid 3AEDBB2A0165
     */
//    public void UpdateShareInformation(long projectShare) {
		//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
//    }

    /**
       Inzia una nuova ricerca.
       @roseuid 3AEDC0E600C6
     */
    public void NewSearch(short[] prjType, String prjDescr) {
		this.networkManager.NewMsgSearch(prjType,prjDescr);
    }

    /**
       Mostra i risultati ottenuti relativi ad una precedente ricerca.
       @roseuid 3AEDCA3F037A
     */
    public void UpdateSearchResult(String peerLocation,Vector queryHits) {
		//nel queryHit c' nell'ordine
		/**
		 * 0 - indexTypes[]
		 * 1 - prjDescr
		 * 2 - wasDown
		 *      se wasDown == true
		 *          3 - prjOrigID
		 *          4 - prjOrigPeerLocation
		 *      se wasDown == false
		 *          3 - prjID
		 */

		Enumeration list=queryHits.elements();
		Vector tmp=null;
		while(list.hasMoreElements())   {
		    tmp=(Vector)list.nextElement();
			short[] indexTypes = new short[2];
			indexTypes[0]=((Short[])tmp.elementAt(0))[0].shortValue();
			indexTypes[1]=((Short[])tmp.elementAt(0))[1].shortValue();

			//verifico se il l'occorenza  di un prj down o no
			if(((Boolean)tmp.elementAt(2)).booleanValue())  {
				// l'occorrenza di 1 prj di tipo down
				//recupere i riferimenti alla versione originale
				// ID e peer

				//devo ctrl se il peer originario del prj di cui ho trovato l'occorrenza
				//non sia quello corrente
				String prjOrigPeerLocation = (String)tmp.elementAt(4);
				if(prjOrigPeerLocation.compareTo(this.networkManager.GetMyIPAddress())!=0)
				//chi ha creato quel progetto non  il peer corrente, posso aggiungere l'occorenza
					this.projectManager.AddPrjToDbPrjFound(indexTypes,(String)tmp.elementAt(1),((Long)tmp.elementAt(3)).longValue(),prjOrigPeerLocation);
			}
			else    {
				// l'occorenza di 1 prj creato localmente
				this.projectManager.AddPrjToDbPrjFound(indexTypes,(String)tmp.elementAt(1),((Long)tmp.elementAt(3)).longValue(),peerLocation);
			}
		}
        this.politellaFrame.UpdateTables();
        this.politellaFrame.tabbedPaneWorking.setSelectedIndex(0);
    }

    /**
       Inizia il download della definizione di un progetto
       @roseuid 3AEE6BCF008C
     */
    public void NewDownloadProject(String iPDest, long prjID) {
//		try {
		    this.networkManager.NewDownloadConnection(iPDest,prjID);
//		}catch(PolitellaException e)    {
//			PolitellaException.ProcessException(e);
//		}
    }

	/**
	 * Inizia il download di un elemento di un Business Component
	 */
	 public void NewDownloadComponentElement(long prjID, Vector busModCompInfos/*short busModCompID,String busCompElementLocation*/)   {
//		try {
			this.networkManager.NewDownloadConnection(prjID,busModCompInfos/*busModCompID,busCompElementLocation*/);
//		}catch(PolitellaException e)    {
//			PolitellaException.ProcessException(e);
//		}
	 }

	/**
	 * Aggiorna lo stato delle connessioni attive
	 */
	public void UpdateActiveConnectionsState(int activeConnections)    {
		this.politellaFrame.UpdateStatusBar(activeConnections,-1,-1);
	}
    /**
       Aggiorna lo stato dei download ll'utente.
       Il campo state
       - true -> inizia download, gli altri campi null
       - false -> termina download
       @roseuid 3AEE6E030033
     */
    public void UpdateDownloadState(int activeDownloads, File[] prjFile, long prjFoundID, String prjFoundPeerLocation) {
		this.politellaFrame.UpdateStatusBar(-1,activeDownloads,-1);
		if(prjFile!=null)       {
		// ho terminato di scaricare qualcosa ...
			try {
				if(prjFoundPeerLocation.compareTo("LOCAL")==0)  {
					//ho finito di scaricare una risorsa associata ad un business component
                    this.projectManager.UpdateOneBusModCompElementLocation(prjFile,true);
					this.politellaFrame.currentPrjViewing.repaint();
				}
				else    {
					//ho finito di scaricare una prj
//			    this.projectManager.AddPrjToDbPrjShare(prjFile,prjFoundID,prjFoundPeerLocation);
				this.projectManager.AddPrjToDbPrjDown(prjFile[0],prjFoundID,prjFoundPeerLocation);
				}
			}   catch(PolitellaException e) {
				PolitellaException.ProcessException(this.politellaFrame,e);
			}
			this.politellaFrame.UpdateTables();
		}
		// se prjFile==null il download  terminato senza successo
		//non mi preoccupo di ci che ho scaricato perch tanto di trova nella dir temp del sistema
		//e in auto verr cancellato alla chiusura della java virtual machine in esecuzione

		// null anche quando inzio un download e devo aggiornare solo i dati della staus bar
    }

    /**
       Aggiorna lo stato degli upload all'utente.
       - true -> inizia upload
       - false -> termina upload
       @roseuid 3AEE6E840075
     */
    public void UpdateUploadState(int activeUploads) {
		this.politellaFrame.UpdateStatusBar(-1,-1,activeUploads);
    }

    /**
       Visualizza il progetto.
       @roseuid 3AEE6ECD016A
     */
    public void ViewProject(Long prjID, String[] prjType, String prjDescr, Vector prjBusModelComponents,/*String prjPeerLocation, */boolean isEditable) {
		File prjFile= this.projectManager.GetProjectFile(prjID.longValue(),isEditable,!isEditable);
		try {
			this.politellaFrame.ViewProject(prjID,prjType,prjDescr,prjFile,prjBusModelComponents/*,prjPeerLocation*/,isEditable/*,false*/);
		}catch (PolitellaException e)   {
			PolitellaException.ProcessException(this.politellaFrame,e);
		}
	}

    /**
       Restituisce il valore del numero di progetti condivisi localmente
       @roseuid 3B038E3B00BF
     */
    public long GetNumPrjShare() {
		return this.projectManager.GetNumPrjShare();
    }

	/**
	 * Esce chiude l'applicazione
	 */
	 public void Exit() {
		try {
//			this.projectManager.SaveDbPrj();
			this.SaveSetup();
			this.projectManager.Exit();
		}catch(PolitellaException e)    {
			PolitellaException.ProcessException(this.politellaFrame,e);
		}
		this.networkManager.Exit();
	 }
	 /**
	  * Disconnette l'applicazione dalla rete Politella
	  */
	public void Disconnect() {
	    this.networkManager.Disconnect();
        //blocco il timer per l'invio dei PING
//        this.SetStatusAnnuncioTimer(false);
	}
	/**
	 * Salvo i db dei progetti e il file delle impostazioni
	 */
	 public void Save() {
		try {
			this.SaveSetup();
			this.projectManager.SaveDbPrj();
		}catch(PolitellaException e)    {
			PolitellaException.ProcessException(this.politellaFrame,e);
		}
	 }

	 /**
	  * Salva il progetto correntemente visualizzato
	  */
	public void SaveCurrPrj(Long prjID,boolean isShare) {
		try {
			this.projectManager.SaveCurrPrj(prjID,isShare);
		}catch(PolitellaException e)    {
			PolitellaException.ProcessException(this.politellaFrame,e);
		}
	}
	/**
	 * Ricerca localmente uno pi progetti in base ai parametri passti in ingresso
	 */
	public Vector SearchProjectInLocal(short[] searchType, String searchKey)  {
		return this.projectManager.SearchProjectInLocal(searchType,searchKey);
	}
    /**
       Restituisce il file descriptor relativo ad un progetto specificato
     */
    public File GetProjectFile(long projectID, boolean isShare, boolean isDown) {
		return this.projectManager.GetProjectFile(projectID,isShare, isDown);
    }
    /**
       Fornisce informazioni relative alle connessioni attive
       @roseuid 3B14FDD40163
     */
    Vector GetActiveConnectionsInfo() {
		return this.networkManager.GetActiveConnectionsInfo();
    }
	/**
	 * Fornisce il numero delle connessioni attive
	 */
	short GetActiveConnectionsNum()   {
		return this.networkManager.GetActiveConnectionsNum();
	 }

    /**
       Fornisce informazioni relative ai download in corso
       @roseuid 3B14FE32012C
     */
     Vector GetActiveDownloadsInfo() {
		return this.networkManager.GetActiveDownloadsInfo();
    }

    /**
       Fornisce informazioni relative agli upload in corso
       @roseuid 3B14FE320209
     */
    Vector GetActiveUploadsInfo() {
		return this.networkManager.GetActiveUploadsInfo();
    }
    /**
     * Metodo attivato allo scattare del timer per l'invio dei messaggi di annuncio
	 * salvo anche i db dei progetti condivisi e trovati
     */
     private void annuncioTimer_actionPerformed(ActionEvent e)   {
        this.Save();
		this.networkManager.NewMsgAnnuncio();
     }
     /**
      * Avvio e blocco il timer di per l'invio dei messaggi di annuncio
      * TRUE -> avvio (se gia' non avviato)
      * FALSE -> blocco (se gia' non bloccato)
      */
     synchronized public void SetStatusAnnuncioTimer(boolean state)   {
        if(state)   {
            //richiesta di avvio timer
            if(!this.annuncioTimer.isRunning())
                this.annuncioTimer.start();
        }
        else    {
            //richiesta di bloccaggio timer
            if(this.annuncioTimer.isRunning())
                this.annuncioTimer.stop();
        }
     }
	 /**
	  * Restituisce le descrizioni per esteso delle tipologie di progetto
	  */
//	 String [][] GetProjectTypeDescription() {
//		return this.networkManager.GetProjectTypeDescription();
//	  }
	/**
	 * Imposta il valore del campo defaultMsgTTL
	 */
	void SetDefaultMsgTTL(short defaultMsgTTL)   {
		this.networkManager.SetDefaultMsgTTL(defaultMsgTTL);
	}
	/**
	 * Restituisce il valore del campo defaultMsgTTL
	 */
	short GetDefaultMsgTTL() {
		return this.networkManager.GetDefaultMsgTTL();
	}
	/**
	 * Restituisce il valore del campo numMaxDownloadInCorso
	 */
	 int GetNumMaxDownloadInCorso()    {
		return this.networkManager.GetNumMaxDownloadInCorso();
	 }
	 /**
	  * Imposta il valore dell'attributo numMaxDownloadInCorso
	  */
	  void SetNumMaxDownloadInCorso(short newValue)    {
		this.networkManager.SetNumMaxDownloadInCorso(newValue);
	  }
	/**
	 * Restituisce il valore del campo numMaxActiveConnections
	 */
	 int GetNumMaxActiveConnections()    {
		return this.networkManager.GetNumMaxActiveConnections();
	 }
	/**
	 * Imposta il valore del campo numMaxActiveConnections
	 */
	void SetNumMaxActiveConnections(short numMaxActiveConnections)   {
		this.networkManager.SetNumMaxActiveConnections(numMaxActiveConnections);
	}
	/**
	 * Crea un nuovo progetto ricevendo con le caratteristiche specificategli
	 */
	 public void NewProjectShare(short[] prjIndexTypes, String prjDescr, File prjFile)  {
		this.projectManager.AddPrjToDbPrjShare((short[])prjIndexTypes,prjDescr,prjFile);
		this.politellaFrame.UpdateTables();
		Vector tmp=this.projectManager.GetInfoOnePrjShare(new Long(prjFile.getName().substring(0,prjFile.getName().length()-4)));

		String [] prjTypes= new String[2];
		prjTypes[0] = (String) tmp.elementAt(0);
		prjTypes[1] = (String) tmp.elementAt(1);
		try {
			this.politellaFrame.ViewProject((Long)tmp.elementAt(4),prjTypes,prjDescr,prjFile,(Vector)tmp.elementAt(5),/*null,*/true/*,true*/);
		}catch(PolitellaException e)    {
			PolitellaException.ProcessException(this.politellaFrame,e);
		}
	 }

}
