/*
*                            Copyright (c) 2001
*                        <B>Politecnico di Torino</B>
*                            All Rights Reserved
* <p>
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that: (1) source code distributions
* retain the above copyright notice and this paragraph in its entirety,(2)
* distributions including binary code include the above copyright notice and
* this paragraph in its entirety in the documentation or other materials
* provided with the distribution, and (3) all advertising materials mentioning
* features or use of this software display the following acknowledgement:
* ``This product includes software developed by Politecnico
* di Torino, and its contributors.'' (4) Neither the name of
* the University nor the names of its contributors may be used to endorse
* or promote products derived from this software without specific prior
* written permission.
* THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
* This product includes code licensed from RSA Data Security (Java Secure Socket Extension)
*
* Sun grants you ("Licensee") a non-exclusive, royalty free, license to use, and redistribute
* this software graphics artwork, as individual graphics or as a collection, as part of
* software code or programs that you develop, provided that
* i) this copyright notice and license accompany the software graphics artwork; and
* ii) you do not utilize the software graphics artwork in a manner which is disparaging to Sun.
* Unless enforcement is prohibited by applicable law, you may not modify the graphics,
* and must use them true to color and unmodified in every way.
*
*<p>
*/
package Network;

//import Protocollo.Header;
import Network.Protocol.Header;
//import Protocollo.Ping;
import Network.Protocol.Ping;
//import Protocollo.Query;
import Network.Protocol.Query;
import Utility.ProjectType;
import Utility.PolitellaException;
import Interfaccia.UserInterface;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import java.net.InetAddress;
import java.net.UnknownHostException;

import com.sun.net.ssl.*;
import java.security.KeyStore;

import java.io.File;
import java.io.InterruptedIOException;

/**
   Modulo che gestisce ogni tipo di interazione tra i peer.
 */
public class NetworkManager {
	/**
	 * TTL
	 */
	static short defaultMsgTTL=64;
	/**
	  * Numero degli upload in corso
	  */
	private static int upLoadInCorso = 0;
	/**
	 * Numero dei download in corso
	 */
//	private static int downLoadInCorso = 0;
	private static SemaphoreOnDownloadInCorso downloadInCorso = null;

	/**
	 * Numero di connessioni attive
	 */
	private static short activeConnections=0;
	/**
	 * Numero massimo di connessioni a gestione dei messaggi
	 */
	static short numMaxActiveConnections=10;

	/**
		Porta per la gestione dei messaggi
	*/
	static final int msgListenPort = 5000;

	/**
		Porta per i download dei progetti
	*/
	static final int downloadPort = 5555;
	/**
		Cache dei messaggi
	 */
	private static Vector cacheMessage = null;
	/**
	Elenco dei Thread a gestione delle connessioni attive
	 */
	static ThreadGroup dbThreadManageConnection = null;
	/**
	Elenco dei Thread a gestione dei download in corso
	 */
	static ThreadGroup dbThreadDownload = null;
	/**
	Elenco dei Thread a gestione degli upload in corso
	 */
	static ThreadGroup dbThreadUpload = null;
	/**
	 * Contiene il proprio indirizzo IP
	 */
	String myIPAddress = null;
	/**
	 * Vettore per i Thread in ascolto di:
	 * - delle nuove richieste di connessione per messaggi (porta 5000)
	 * - delle nuove richieste di connessione per download (porta 5555)
	*/
	private ThreadIncomingNewConnection threadIncomingNewConnection[];
	/**
	 * Identificatore dell'oggetto UserInterface
	 */
	 private UserInterface userInterface=null;

	/**
       Costruttore
     */
    public NetworkManager(UserInterface userInterface) throws PolitellaException{

		this.downloadInCorso = new SemaphoreOnDownloadInCorso();

		this.cacheMessage = new Vector();
		this.userInterface=userInterface;
		MessageRif.networkManager=this;
		this.dbThreadManageConnection= new ThreadGroup("DbThreadManageConnection");
		this.dbThreadDownload= new ThreadGroup("DbThreadDownload");
		this.dbThreadUpload= new ThreadGroup("DbThreadUpload");

		try	{

		    this.InitSSLEnvironment();

			this.threadIncomingNewConnection=new ThreadIncomingNewConnection[2];
			// thread per gestione delle nuove richieste di connessione per messaggi
			this.threadIncomingNewConnection[0]= new ThreadIncomingNewConnection(NetworkManager.msgListenPort,this);
			this.threadIncomingNewConnection[0].start();
			// thread per gestione delle nuove richieste di connessione per download
			this.threadIncomingNewConnection[1]= new ThreadIncomingNewConnection(NetworkManager.downloadPort,this);
			this.threadIncomingNewConnection[1].start();
		}catch(PolitellaException e)	{
			if(e.GetType()!=PolitellaException.NO_SSL_INITIALIZATION)   {
				if(this.threadIncomingNewConnection[0]!=null)
					this.threadIncomingNewConnection[0].stop();
				if(this.threadIncomingNewConnection[1]!=null)
					this.threadIncomingNewConnection[1].stop();
			}
			throw e;
		}
    }

	/**
	 * Recupera l'IP address assegnato al peer su cui il programma  in esecuzione
	 */
	synchronized void SetMyIPAddress(String myIPAddress)   {
		this.myIPAddress=myIPAddress;
	}
	/**
	 * Inizializza l'ambiente SSL
	 */
	 private void InitSSLEnvironment() throws PolitellaException{
		try {
			java.security.Security.addProvider(new com.sun.net.ssl.internal.ssl.Provider());

			char[] passphrase = "Politella".toCharArray();

			KeyStore ks = KeyStore.getInstance("JKS");
			ks.load(new java.io.FileInputStream(new File(System.getProperty("user.dir"),"keystoreDSA1024.keystore")), passphrase);

			KeyManagerFactory kmf = KeyManagerFactory.getInstance("SunX509");
			kmf.init(ks, passphrase);

			TrustManagerFactory tmf = TrustManagerFactory.getInstance("SunX509");
			tmf.init(ks);

			SSLContext ctx	= SSLContext.getInstance("TLS");
			ctx.init(kmf.getKeyManagers(), tmf.getTrustManagers(), null);

			ThreadIncomingNewConnection.InitSSLEnvironment(ctx);
			ThreadDirectDownload.InitSSLEnvironment(ctx);
			ThreadManageConnection.InitSSLEnvironment(ctx);

		 }catch(java.lang.Exception e)  {
			throw new PolitellaException(PolitellaException.NO_SSL_INITIALIZATION,null);
		 }
	 }


	/**
       Crea una nuova connessione per il semplice dialogo tra peer.
       @roseuid 3AED3C4C0295
     */
    public void NewMsgConnection(String iPDest) throws PolitellaException{
		(new ThreadManageConnection(iPDest,this)).start();
    }

    /**
       Crea una nuova connessione per il download di un progetto.
       @roseuid 3AED3E970068
     */
	public void NewDownloadConnection(String iPDest, long prjID)/* throws PolitellaException*/{
			(new ThreadDirectDownload(iPDest,prjID,this)).start();
    }

    /**
       Crea una nuova connessione per il download di un elemento di un business component.
     */
	public void NewDownloadConnection(long prjID,Vector busModCompInfos/* short busModCompID, String busCompElementLocation*/)/* throws PolitellaException*/{
			(new ThreadDirectDownload(prjID,busModCompInfos/*busModCompID,busCompElementLocation*/,this)).start();
    }

    /**
       Ricerca il riferimento ad un messaggio all'interno della cache.
       @roseuid 3AED66B5017E
     */
    public boolean SearchMsgInCache(Object msg) {
		Header msgHeader = null;
		msgHeader = (Header) msg;

        return this.ManageMsgInCache(false,true,new MessageRif(msgHeader.GetUniqueID(),msgHeader.GetType(),msgHeader.GetIPSource()));
    }

    /**
       Salva il riferimento di un messaggio all'interno della cache.
       @roseuid 3AED672B014B
     */
    public void SaveMsgInCache(Object msg) {
		Header msgHeader = (Header) msg;
        this.ManageMsgInCache(true,false,new MessageRif(msgHeader.GetUniqueID(),msgHeader.GetType(),msgHeader.GetIPSource()));

//        java.util.Calendar tmp= java.util.Calendar.getInstance();
//       System.out.println(msgHeader.GetUniqueID()+" : Annuncio alle: "+ tmp.getTime().toGMTString());

    }
	/**
	 * Cancella il riferimento ad un messaggio all'interno della cache
	 */
	 public void DelMsgInCache(MessageRif msgRif)  {
        this.ManageMsgInCache(false,false,msgRif);
	 }
    /**
     * Esegue delle operazioni sulla cache dei messaggi
     */
     synchronized private boolean ManageMsgInCache(boolean saveIt, boolean searchIt, MessageRif msgRif)  {
        if(saveIt)  {
        //devo salvare un nuovo messaggio nella cache
            this.cacheMessage.addElement(msgRif);
        }
        else    {
            if(searchIt)    {
            //devo cercare un messaggio nella cache
               	MessageRif tmp=null;
                Enumeration listMsgRif= this.cacheMessage.elements();
        		while(listMsgRif.hasMoreElements())	{
        			tmp=(MessageRif)listMsgRif.nextElement();
                    short type=msgRif.GetType();
                    short itemType=tmp.GetType();

		            if(msgRif.GetUniqueID()==tmp.GetUniqueID()) {
                        if(((type==Header.PONG)&&(itemType==Header.PING))||((type==Header.QUERY_REPLY)&&(itemType==Header.QUERY)))
            				//nel caso di PONG e QUERY_REPLY  sufficiente il ctrl su
		            		//uniqueID e Type
			    	       	return true;
        		    	else    {
                            //questo else e' inutile ma si mette in caso di evoluzione del protocollo applicativo.
                            //nella cache vengono infatti solamente salvati i PING e i QUERY
    		    			if((type==itemType)
	    				        &&(msgRif.GetIPSource().compareTo(tmp.GetIPSource())==0))
            					//il messaggio gi esiste
		            			return true;
        		    	}
        			}
        		}
                return false;
            }
            else    {
            //devo cancellare un messaggio dalla cache
                this.cacheMessage.removeElement(msgRif);
            }
        }
        return false;
     }
    /**
       Invia un messaggio a tutti i peer conness,
       @roseuid 3AED6766036D
     */
    synchronized public void SendMsgToAllPeer(Object msg, String lessThisPeer)	{
		//il 2* serve nel caso in cui quando invoco questo metodo ci siano dei thread
		//attivi per richieder euna nuova connessioni
		Thread[] threadList= new Thread[this.numMaxActiveConnections*2];
		ThreadManageConnection tmpThread=null;
		for(int i=this.dbThreadManageConnection.enumerate(threadList);i>0;i--)	{
			tmpThread = (ThreadManageConnection)threadList[i-1];
			if(tmpThread.GetConnectionAccepted())   {
                //sulla connessione gestita da questo thread e' terminata con successo la procedura
                // di transazione della connessione...e' stata acccettata
                if(lessThisPeer==null)
	    			try	{
		    			// devo inviare il msg a tutti i peer, senza alcuna esclusione
			    		tmpThread.SendMsg(msg);
    				}catch(PolitellaException e)	{
	    				tmpThread.stop();
		    		}
    			else
	    			if(tmpThread.getName().compareTo(lessThisPeer)!=0)
    					try	{
	    					tmpThread.SendMsg(msg);
    					}catch(PolitellaException e)	{
	    					tmpThread.stop();
		    			}
			}
		}
    }

    /**
       Inizia una nuova ricerca sulla rete.
       @roseuid 3AEDC1F4039F
     */
    public void NewMsgSearch(short[] prjType, String prjDescr) {
		Query query= new Query(this.myIPAddress,this.defaultMsgTTL,prjType,prjDescr);
        this.SaveMsgInCache(query);
		this.SendMsgToAllPeer(query,null);
    }

    /**
       Cerca il progetto tra quelli condivisi localmente.
       @roseuid 3AED45600051
     */
    Vector SearchProjectInLocal(short[] searchType, String searchKey) {
		//chiama l'interfaccia
		return this.userInterface.SearchProjectInLocal(searchType,searchKey);
    }

    /**
       Mostra i risultati ottenuti relativi ad una precedente ricerca.
       @roseuid 3AEE802003A2
     */
    synchronized void UpdateSearchResult(String peerLocation,Vector queryHits) {
		this.userInterface.UpdateSearchResult(peerLocation,queryHits);
    }
	/**
	 * Aggiorna lo stato delle connessioni attive
     * Il campo state
     * - true -> inizia la connessione
     * - false -> termina la connessione
	 */
	synchronized void UpdateActiveConnectionsState(boolean state)    {
		if(state)
			this.activeConnections++;
		else    {
			if(this.activeConnections!=-1)
                this.activeConnections--;
		}
		this.userInterface.UpdateActiveConnectionsState(this.activeConnections);
	}
    /**
     * Aggiorna lo stato dei download dell'utente.
     * Il campo state
     * - true -> inizia download, gli altri campi null
     * - false -> termina download
     */
    synchronized void UpdateDownloadState(boolean state,File[] prjFile, long prjID, String prjPeerLocation) {
		if(state)   {
//			this.downLoadInCorso++;
		}
		else
//			this.downLoadInCorso--;
		    this.downloadInCorso.Decrement();
		//aggiorno sull'interfaccia
		this.userInterface.UpdateDownloadState(this.downloadInCorso.GetDownloadInCorso(),prjFile,prjID,prjPeerLocation);
    }

    /**
     * Aggiorna lo stato degli upload dell'utente.
     * - true -> inizia upload
     * - false -> termina upload
     */
    synchronized void UpdateUploadState(boolean state) {
		if(state)   {
			this.upLoadInCorso++;
		}
		else    {
			this.upLoadInCorso--;
		}
		//aggiorno sull'interfaccia
		this.userInterface.UpdateUploadState(this.upLoadInCorso);
    }
    /**
     * Chiude tutte le connessioni attive
     */
     public void Disconnect()   {
    	this.dbThreadManageConnection.stop();
     }
	/**
	 * Chiude tutti i socket attivi, termina tutti i thread
	 */
	public void Exit() {
		// blocco la ricezione di nuove connessioni, sia x messaggi che x download
//        this.threadIncomingNewConnection[0].stop();
//		this.threadIncomingNewConnection[1].stop();
//        this.Disconnect();
	}
    /**
       Restituisce il file descriptor relativo ad un progetto specificato
       @roseuid 3AF80DC201AE
     */
    File GetProjectFile(long projectID,boolean isShare, boolean isDown) {
		//chiama l'interfaccia
		return this.userInterface.GetProjectFile(projectID,isShare,isDown);
    }
   /**
    * Restituisce il valore del numero di progetti condivisi localmente
    */
   long GetNumPrjShare() {
	   //chiama l'interfaccia
		return this.userInterface.GetNumPrjShare();
	}

   /**
	 * Restituisce delle informazioni relative ai ThreadManageConnection attivi
	*/
	public Vector GetActiveConnectionsInfo() {
		//il 2* serve nel caso in cui quando invoco questo metodo ci siano dei thread
		//attivi per richieder euna nuova connessioni
		Thread[] threadList= new Thread[this.numMaxActiveConnections*2];
		ThreadManageConnection tmpThread=null;
		Vector info= new Vector();
		for(int i=this.dbThreadManageConnection.enumerate(threadList);
			i>0;i--)	{
			tmpThread = (ThreadManageConnection)threadList[i-1];
			if(tmpThread.GetConnectionAccepted())
                //la connessione non e' attiva e autenticata
                info.addElement(tmpThread.GetInfoConnection());
		}
		return info;
	}
	/**
	 * Restituisce il valore di activeConnections
	 */
	 public short GetActiveConnectionsNum()   {
		return this.activeConnections;
	 }
	/**
	 * Restituisce il valore dell'attributo myIPAddress
	 */
	public String GetMyIPAddress()  {
		return this.myIPAddress;
	}
   /**
	 * Restituisce delle informazioni relative ai
	 * ThreadDirectDownload attivi (in fase di download)
	*/
	public Vector GetActiveDownloadsInfo() {
		// devo moltiplicare x 2 il risultato di ActiveCount perch quel
		// metodo mi fornisce solamente una stima dei Thread attivi
		Thread[] threadList= new Thread[2*this.dbThreadDownload.activeCount()];
		ThreadDirectDownload tmpThread=null;
		Vector info= new Vector();
		for(int i=this.dbThreadDownload.enumerate(threadList);
			i>0;i--)	{
			tmpThread = (ThreadDirectDownload)threadList[i-1];
			info.addElement(tmpThread.GetInfoConnection());
		}
		return info;
	}
   /**
	 * Restituisce delle informazioni relative ai
	 * ThreadDirectDownload attivi (in fase di upload)
	*/
	public Vector GetActiveUploadsInfo() {
		// devo moltiplicare x 2 il risultato di ActiveCount perch quel
		// metodo mi fornisce solamente una stima dei Thread attivi
		Thread[] threadList= new Thread[2*this.dbThreadUpload.activeCount()];
		ThreadDirectDownload tmpThread=null;
		Vector info= new Vector();
		for(int i=this.dbThreadUpload.enumerate(threadList);
			i>0;i--)	{
			tmpThread = (ThreadDirectDownload)threadList[i-1];
			info.addElement(tmpThread.GetInfoConnection());
		}
		return info;
	}

	/**
	 * Imposta il valore del campo defaultMsgTTL
	 */
	public void SetDefaultMsgTTL(short defaultMsgTTL)   {
		this.defaultMsgTTL=defaultMsgTTL;
	}
	/**
	 * Restituisce il valore del campo defaultMsgTTL
	 */
	public short GetDefaultMsgTTL() {
		return this.defaultMsgTTL;
	}
	/**
	 * Restituisce il valore del campo numMaxDownloadInCorso
	 */
	 public short GetNumMaxDownloadInCorso()    {
		return this.downloadInCorso.GetNumMaxDownloadInCorso();
	 }
	/**
	 * Restituisce il valore del campo numMaxActiveConnections
	 */
	 public short GetNumMaxActiveConnections()    {
		return this.numMaxActiveConnections;
	 }
	/**
	 * Imposta il valore del campo numMaxActiveConnections
	 */
	public void SetNumMaxActiveConnections(short numMaxActiveConnections)   {
		this.numMaxActiveConnections=numMaxActiveConnections;
	}
    /**
     * Invia un nuovo annuncio (PING) sulla rete
     */
     public void NewMsgAnnuncio()   {
        Ping ping = new Ping(this.myIPAddress,this.defaultMsgTTL);
        this.SaveMsgInCache(ping);
        this.SendMsgToAllPeer(ping,null);
     }
      /**
       * Controlla un indirizzo IP per una eventuale nuova connessione.
       * Si chiama all'arrivo di ogni nuovo PING
       */
       synchronized void CheckPeerAddressForNewConnection(String iPAddress)    {
		    //il 2* serve nel caso in cui quando invoco questo metodo ci siano dei thread
	    	//attivi per richieder euna nuova connessioni
		    if(this.activeConnections<this.numMaxActiveConnections) {
                Thread[] threadList= new Thread[2*this.numMaxActiveConnections];
    		    ThreadManageConnection tmpThread=null;
	    	    boolean newConnection=true;
                for(int i=this.dbThreadManageConnection.enumerate(threadList);i>0;i--)	{
    			    tmpThread = (ThreadManageConnection)threadList[i-1];
    			    if(tmpThread.GetConnectionAccepted())
                        //sulla connessione gestita da questo thread e' terminata con successo la procedura
                        // di transazione della connessione...e' stata acccettata
                        if(iPAddress.compareTo(tmpThread.iPDest)==0)    {
                            newConnection=false;
                            break;
                        }
    		    }
                if(newConnection)
                    //ho ricevuto un messaggio (un PING) da un peer a me sconosciuto
                    try {
                        (new ThreadManageConnection(iPAddress,this)).start();
                    }catch(PolitellaException e){}
		    }
       }

	   /**
	    * Verifica la disponibilita a creare una nuova connessione per il download
	    */
		void CheckForNewDownload()  {
			this.downloadInCorso.CheckForNewDownload();
		}
	 /**
	  * Imposta il valore dell'attributo numMaxDownloadInCorso
	  */
	  public void SetNumMaxDownloadInCorso(short newValue)    {
		this.downloadInCorso.SetNumMaxDownloadInCorso(newValue);
	  }
}
