/*
*                            Copyright (c) 2001
*                        <B>Politecnico di Torino</B>
*                            All Rights Reserved
* <p>
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that: (1) source code distributions
* retain the above copyright notice and this paragraph in its entirety,(2)
* distributions including binary code include the above copyright notice and
* this paragraph in its entirety in the documentation or other materials
* provided with the distribution, and (3) all advertising materials mentioning
* features or use of this software display the following acknowledgement:
* ``This product includes software developed by Politecnico
* di Torino, and its contributors.'' (4) Neither the name of
* the University nor the names of its contributors may be used to endorse
* or promote products derived from this software without specific prior
* written permission.
* THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
* This product includes code licensed from RSA Data Security (Java Secure Socket Extension)
*
* Sun grants you ("Licensee") a non-exclusive, royalty free, license to use, and redistribute
* this software graphics artwork, as individual graphics or as a collection, as part of
* software code or programs that you develop, provided that
* i) this copyright notice and license accompany the software graphics artwork; and
* ii) you do not utilize the software graphics artwork in a manner which is disparaging to Sun.
* Unless enforcement is prohibited by applicable law, you may not modify the graphics,
* and must use them true to color and unmodified in every way.
*
*<p>
*/
//package Protocollo;
package Network.Protocol;

import java.io.Serializable;

/**
   Intestazione comune a tutti i messaggi del protocollo Politella.
 */
public class Header implements Serializable{
    //Tipi di messaggi
	public static final short CONNECT = 1;
	public static final short CONNECT_REPLY = 2;
	public static final short PING = 3;
	public static final short PONG = 4;
	public static final short QUERY = 5;
	public static final short QUERY_REPLY = 6;
	/**
       Permette di identificare in modo univoco un messaggio. E' impostato solamente dal peer che genera il messaggio dopodiche non viene pi modificato. E' generato casualmente.
     */
    private long uniqueID;
    /**
       Identifica il tipo di messaggio
       - CONNECT: richiesta di connessione;
       - CONNECT_REPLY: risposta alla richiesta di connessione;
       - PING: annuncio;
       - PONG: risposta all'annuncio;
       - QUERY: ricerca di un progetto;
       - QUERY_HIT: progetto ricercato individuato.
     */
    private short type;
    /**
       Time To Live
     */
    protected short TTL = -1;
    /**
       Indirizzo IP del peer mittente
     */
    protected String iPSource;
    /**
       Numero della porta TCP su cui il peer mittente  in ascolto
     */
    private static final int portSource = 5000;

	public Header(short msgType) {
		this.uniqueID=(new java.util.Random()).nextLong();
		this.type=msgType;
    }
    /**
       Restituisce il valore del campo uniqueID
       @roseuid 3AEACD2301F7
     */
    public long GetUniqueID() {
		return this.uniqueID;
    }

    /**
       Restituisce il valore del campo type.
       @roseuid 3AEACD2C0019
     */
    public short GetType() {
		return this.type;
    }

    /**
       Restituisce il valore del campo TTL
       @roseuid 3AEACD3402EC
     */
    public short GetTTL() {
		return this.TTL;
    }

    /**
       Restituisce il valore del campo iPSource.
       @roseuid 3AEACD3B03BE
     */
    public String GetIPSource() {
		return this.iPSource;
    }

    /**
       Restituisce il valore del campo portSource.
       @roseuid 3AEACD420350
     */
    public int GetPortSource() {
		return this.portSource;
    }



    /**
       Imposta il valore del campo TTL
       @roseuid 3AEACDE90346
     */
    void SetTTL(short TTL) {
		this.TTL=TTL;
    }

    /**
       Decrementa di una unit il valore del campo TTL
       @roseuid 3AEACDF00075
     */
    public boolean DecreaseTTL() {
		this.TTL--;
		if(this.TTL==0)
			// il messaggio non pu essere ulteriormente inoltrato
			return false;
		// il messaggio pu essere ulteriormente inoltrato
		return true;
    }

    /**
       Imposta il valore del campo iPSource.
       @roseuid 3AEACDFA02D2
     */
    void SetIPSource(String iPSource) {
		this.iPSource=iPSource;
    }

    /**
       Imposta il valore del campo uniqueID.
     */
	void SetUniqueID(long uniqueID) {
		this.uniqueID=uniqueID;
	}
}
