/*
*                            Copyright (c) 2001
*                        <B>Politecnico di Torino</B>
*                            All Rights Reserved
* <p>
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that: (1) source code distributions
* retain the above copyright notice and this paragraph in its entirety,(2)
* distributions including binary code include the above copyright notice and
* this paragraph in its entirety in the documentation or other materials
* provided with the distribution, and (3) all advertising materials mentioning
* features or use of this software display the following acknowledgement:
* ``This product includes software developed by Politecnico
* di Torino, and its contributors.'' (4) Neither the name of
* the University nor the names of its contributors may be used to endorse
* or promote products derived from this software without specific prior
* written permission.
* THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
* This product includes code licensed from RSA Data Security (Java Secure Socket Extension)
*
* Sun grants you ("Licensee") a non-exclusive, royalty free, license to use, and redistribute
* this software graphics artwork, as individual graphics or as a collection, as part of
* software code or programs that you develop, provided that
* i) this copyright notice and license accompany the software graphics artwork; and
* ii) you do not utilize the software graphics artwork in a manner which is disparaging to Sun.
* Unless enforcement is prohibited by applicable law, you may not modify the graphics,
* and must use them true to color and unmodified in every way.
*
*<p>
*/
package Network;

/**
 * Implementa un semaforo sul numero di download contemporaneamente attivabili
 */
class SemaphoreOnDownloadInCorso {
	/**
	 * Numero massimo di download contemporanei
	 */
	private short numMaxDownloadInCorso=10;
	/**
	 * Numero dei download in corso
	 */
	private short downLoadInCorso = 0;

    public SemaphoreOnDownloadInCorso() {
    }

	/**
	 * Ctrl se  possibile istanziare una nuova connessione per il download
	 */
	public void CheckForNewDownload()  {
		while(this.downLoadInCorso>=this.numMaxDownloadInCorso)    {
			//non potr mai essere >, al pi =
			this.WaitForNewDownload();
		}
		this.InDecrementDownloadInCorso(true);
	}

	/**
	 * Se non  possibile avviare subito un nuovo download, pone in attesa il
	 * thread
	 */
	private synchronized void WaitForNewDownload()    {
			try {
//				System.out.println("Aspetto");
				this.wait();
			}catch(InterruptedException e) {
//				System.out.println("Ho finito di aspettare");
				}
	}

	/**
	 * Aumenta e diminusice il numero dei download in corso
	 */
	private synchronized void InDecrementDownloadInCorso (boolean mustIncrement) {
		if(mustIncrement)
			this.downLoadInCorso++;
		else    {
			this.downLoadInCorso--;
			try {
				this.notify();
			}catch(IllegalMonitorStateException e){}
		}
	}

	/**
	 * Diminuisce il valore dei download in corso
	 */
	public void Decrement() {
		this.InDecrementDownloadInCorso(false);
	}

	/**
	 * Aumenta il valore dei download in corso
	 */
	public void Increment() {
		this.InDecrementDownloadInCorso(true);
	}

	/**
	 * Restituisce il valore dell'attributo downloadInCorso
	 */
	public short GetDownloadInCorso()  {
		return this.downLoadInCorso;
	}
	/**
	 * Restituisce il valore del campo numMaxDownloadInCorso
	 */
	 public short GetNumMaxDownloadInCorso()    {
		return this.numMaxDownloadInCorso;
	 }
	 /**
	  * Imposta il valore dell'attributo numMaxDownloadInCorso
	  */
	  protected void SetNumMaxDownloadInCorso(short newValue)    {
		this.numMaxDownloadInCorso=newValue;
	  }
}