/*
*                            Copyright (c) 2001
*                        <B>Politecnico di Torino</B>
*                            All Rights Reserved
* <p>
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that: (1) source code distributions
* retain the above copyright notice and this paragraph in its entirety,(2)
* distributions including binary code include the above copyright notice and
* this paragraph in its entirety in the documentation or other materials
* provided with the distribution, and (3) all advertising materials mentioning
* features or use of this software display the following acknowledgement:
* ``This product includes software developed by Politecnico
* di Torino, and its contributors.'' (4) Neither the name of
* the University nor the names of its contributors may be used to endorse
* or promote products derived from this software without specific prior
* written permission.
* THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
* This product includes code licensed from RSA Data Security (Java Secure Socket Extension)
*
* Sun grants you ("Licensee") a non-exclusive, royalty free, license to use, and redistribute
* this software graphics artwork, as individual graphics or as a collection, as part of
* software code or programs that you develop, provided that
* i) this copyright notice and license accompany the software graphics artwork; and
* ii) you do not utilize the software graphics artwork in a manner which is disparaging to Sun.
* Unless enforcement is prohibited by applicable law, you may not modify the graphics,
* and must use them true to color and unmodified in every way.
*
*<p>
*/
package Network;

import Utility.PolitellaException;

import java.io.DataOutputStream;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedInputStream;
import java.util.Vector;
import java.util.Random;

//import java.net.Socket;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.SSLSocket;

import com.sun.net.ssl.SSLContext;
//import com.sun.net.ssl.*;
//import java.security.KeyStore;

import java.net.UnknownHostException;
import java.io.IOException;

/**
   Thread che gestisce il trasferimento dei progetti tra i peer
 */
final class ThreadDirectDownload extends Thread{
	/**
	 * Dimensione massima dei pacchetti
	 */
	static final private int DIMMAXPACKET = 20480;
	/**
	 * Identificativo del networkManager
	 */
	private static NetworkManager networkManager = null;
	/**
	 * Generatore di numeri random
	 */
	 private static Random genRandom = null;
	/**
	 * Socket di connessione
	 */
//	private Socket peerSocket = null;
	private SSLSocket peerSocket = null;
	/**
	 * SSL Socket factory
	 */
	 private static SSLSocketFactory sslSktFactory = null;
	/**
	 * Stream di Input
	 */
	private DataInputStream in = null;
	/**
	 * Stream di Output
	 */
	private DataOutputStream out = null;
	/**
	 * Identificativo progetto da scaricare
	 */
	private long prjID;
	/**
	 * Identificativo, sul peer remoto, della risorsa associata al business component che si vuole scaricare
	 */
	private String busCompElementRemoteLocation =null;
	/**
	 * Identifica il percorso in cui la risorsa, associata ad un bus comp e che si vuol scaricare, verr salvata localmente
	 */
	private File busCompElementLocalPath =null;
	/**
	 * Specifica se il thread serve per ricevere o inviare qualcosa
	 */
	private boolean isSending;
	/**
	 * Specifica l'indirizzo IP a cui ci si connette.
	 */
	private String iPDest =null;
	/**
	 * Contiene delle info sulla risorsa che il thread sta scaricando
	 */
	 private Vector resTransferInProgressInfos = new Vector();

    /**
	 * Costruttore.
	 * Invocato dal peer che vuole scaricare un elemento di un business component.
	 */
    public ThreadDirectDownload(long prjID, Vector busModCompInfos, NetworkManager networkManager)/* throws PolitellaException*/{
		super(networkManager.dbThreadDownload,"DownloadThread"+(new java.util.Random()).nextLong());
							// in questo modo in caso di uscita durante il download
							 //il thread pu morire senza problemi, senza che il db
		this.setDaemon(true);//dei progetti rimanga disallineato rispetto a quello
							 //che realmente  presente sull'HD

		this.isSending=false;
		this.networkManager=networkManager;
		this.prjID=prjID;

		 // nel vector ci sono nell'ordine:
		 // - short busModCompID
		 // - Vector busCompElementLocation (ha elementi in # pari al numero di file associati a quel particolare tipo di risorsa, sono tutti null tranne il file che si vuole scaricare)
		 //Se l'elemento da scaricare non fa parte dei gruppi di risorse ->  la doc di un intero bus comp
			 // - String busCompElementType
		 //se fa parte di un gruppo di risorse
			// - String RESOURCES
			// - short busModCompIDLink
			// - String busCompElementType
		Short busModCompID = (Short)busModCompInfos.get(0);

		String busCompElementLocation=null;
		Vector fileList = (Vector)busModCompInfos.get(1);
		for(int i=0;i<fileList.capacity();i++)  {
			if((busCompElementLocation=(String)fileList.elementAt(i))!=null)    {
			//ho recuperato il nome del file
				this.resTransferInProgressInfos.add(null);
				this.resTransferInProgressInfos.add(new Integer(fileList.capacity()));
				this.resTransferInProgressInfos.add(new Integer(i));
				break;
			}
		}

//		String busCompElementLocation = (String)busModCompInfos.get(1);

		String RESOURCES=null;
		String busCompElementType = null;
		Short busModCompIDLink = null;

		String buf = (String)busModCompInfos.get(2);
		if(buf.compareTo("RESOURCES")!=0)
			busCompElementType= buf.toString();
		else    {
		//l'elemento da scaricare fa parte dei gruppi di risorse dei bus comp
			RESOURCES=buf.toString();
			busModCompIDLink = (Short)busModCompInfos.get(3);
			busCompElementType = (String)busModCompInfos.get(4);
		}

		//il carattere @ separa l'indirizzo del peer su cui si trova la risorsa cercata dal nome del file
		this.iPDest = busCompElementLocation.substring(0,busCompElementLocation.indexOf('@'));
		this.busCompElementRemoteLocation = busCompElementLocation.substring(busCompElementLocation.indexOf('@')+1,busCompElementLocation.length());

		this.resTransferInProgressInfos.remove(0);
		this.resTransferInProgressInfos.add(0,(new File(this.busCompElementRemoteLocation)).getName());
//		this.resTransferInProgress = (new File(this.busCompElementRemoteLocation)).getName();

		//determino il percorso dove andr a scaricare il file, sar della forma [num random]/percorso
		//il [numrandom] serve perch percorso contiene dir del tipo IDx, che successivamente dovranno essere convertite
		//nei nomi corretti dei bus. comp., scaricando pi componenti appartenenti allo stesso bus comp
		//c' il rischio (se non metto il random), che nella procedura di conversione IDx a nomi si cancellino risorse
		//in fase di download
		File prjDownRootPath = this.networkManager.GetProjectFile(0,false,true);
		File prjDownPath = new File(prjDownRootPath,new String(""+this.genRandom.nextInt()+""));
		while(prjDownPath.exists()) {
//			System.out.println("esiste gi");
			prjDownPath = new File(prjDownRootPath,new String(""+this.genRandom.nextInt()+""));
		}
//		System.out.println(prjDownPath.getAbsolutePath());

		if(RESOURCES==null)
		//la risorsa  riferita all'intero bus comp
			this.busCompElementLocalPath = new File(prjDownPath,""+this.prjID+System.getProperty("file.separator")+"ID"+busModCompID+System.getProperty("file.separator")+busCompElementType+"");
		else
		//l'elemento fa parte di una dei gruppi di risorse associate al bus comp
			this.busCompElementLocalPath = new File(prjDownPath,""+this.prjID+System.getProperty("file.separator")+"ID"+busModCompID+System.getProperty("file.separator")+RESOURCES+System.getProperty("file.separator")+"ID"+busModCompIDLink+System.getProperty("file.separator")+busCompElementType+"");
    }

    /**
       Costruttore.
       Invocato dal peer che vuole scaricare un progetto.
       @roseuid 3AEE6C5802EC
     */
    public ThreadDirectDownload(String iPDest, long prjID, NetworkManager networkManager)/* throws PolitellaException*/{
		super(networkManager.dbThreadDownload,"DownloadThread"+(new java.util.Random()).nextLong());
							// in questo modo in caso di uscita durante il download
							 //il thread pu morire senza problemi, senza che il db
		this.setDaemon(true);//dei progetti rimanga disallineato rispetto a quello
							 //che realmente  presente sull'HD

		this.isSending=false;
		this.networkManager=networkManager;
		this.prjID=prjID;
		this.iPDest=iPDest;

		this.resTransferInProgressInfos.add(new String(this.prjID + ".xml"));
    }

    /**
       Costruttore.
       Invocato dal peer da cui viene scaricato un progetto.
       @roseuid 3AEE6D000149
     */
//    public ThreadDirectDownload(Socket peerSocket, NetworkManager networkManager) throws PolitellaException{
    public ThreadDirectDownload(SSLSocket peerSocket, NetworkManager networkManager) throws PolitellaException{
		super(networkManager.dbThreadUpload,"UploadThread"+(new java.util.Random()).nextLong());
		this.setDaemon(true);// in questo modo in caso di uscita durante l'upload
							 //il thread pu morire senza problemi,

		this.isSending=true;
		this.peerSocket=peerSocket;
		this.networkManager=networkManager;

		this.iPDest=this.peerSocket.getInetAddress().getHostAddress();

		try	{
			in = new DataInputStream(peerSocket.getInputStream());
			out = new DataOutputStream(peerSocket.getOutputStream());
		}catch(IOException	e)	{
			// impossibile connettersi con l'host specificato
//			e.printStackTrace();
			this.Close();
			throw new PolitellaException(PolitellaException.NO_UPLOAD_CONNECTION,this.iPDest);
		}

//        System.out.println(this.DIMMAXPACKET);
    }
	/**
	 * Inizializza l'SSLSocketFactory
	 */
	 static void InitSSLEnvironment(SSLContext ctx){
/*	 static void InitSSLEnvironment()throws PolitellaException {

///////////////////
try {
			char[] passphrase = "Politella".toCharArray();

			KeyStore ks = KeyStore.getInstance("JKS");
			ks.load(new java.io.FileInputStream(new File(System.getProperty("user.dir"),"keystoreDSA1024.keystore")), passphrase);

			KeyManagerFactory kmf = KeyManagerFactory.getInstance("SunX509");
			kmf.init(ks, passphrase);

			TrustManagerFactory tmf = TrustManagerFactory.getInstance("SunX509");
			tmf.init(ks);

			SSLContext ctx	= SSLContext.getInstance("TLS");
			ctx.init(kmf.getKeyManagers(), tmf.getTrustManagers(), null);
			sslSktFactory = ctx.getSocketFactory();
		 }catch(java.lang.Exception e)  {
			throw new PolitellaException(PolitellaException.NO_SSL_INITIALIZATION,null);
		 }
			////////////////////
*/

			sslSktFactory = ctx.getSocketFactory();

			genRandom = new Random();
	 }

	/**
	 * Ctrl se  possibile effettuare una connessione per il download.
	 * In caso affermativo la effettua subito, diversamente pone il thread in attesa fino a quando
	 * ci non  possibile
	 */
	private void MakeTheConnection() throws PolitellaException{
//		System.out.println("posso connettermi ?");
		this.networkManager.CheckForNewDownload();
//		System.out.println("Ok mi connetto alle " + java.util.Calendar.getInstance().getTime().toGMTString());

		try	{
//			peerSocket = new Socket(this.iPDest,NetworkManager.downloadPort);
			this.peerSocket=(SSLSocket)this.sslSktFactory.createSocket(iPDest,NetworkManager.downloadPort);

			this.iPDest=this.peerSocket.getInetAddress().getHostAddress();

			in = new DataInputStream(peerSocket.getInputStream());
			out = new DataOutputStream(peerSocket.getOutputStream());
		}catch(UnknownHostException e1){
			// impossibile connettersi con l'host specificato
			e1.printStackTrace();
			this.Close();
			throw new PolitellaException(PolitellaException.NO_DOWNLOAD_CONNECTION,iPDest);
		} catch(IOException e2)	{
			// impossibile connettersi con l'host specificato
			e2.printStackTrace();
			this.Close();
			throw new PolitellaException(PolitellaException.NO_DOWNLOAD_CONNECTION,iPDest);
		}
	}


	public void run(){
		try	{
			if(this.isSending)	{
				// il thread  stato creato per fornire ...
				this.networkManager.UpdateUploadState(true);
				try {
//					System.out.println("che vuole ? ");
					if(this.in.readBoolean())   {
						// se ricevo true voglio 1 progetto
//					System.out.println("1 prj");

						//mistero
//						this.out.writeInt(5);
//						this.out.flush();

						this.Search_SendTheProject();
					}   else    {
						// se ricevo false voglio un elemento di un business component
	    				this.SendBusCompElement();
					}
				this.networkManager.UpdateUploadState(false);
				}catch(IOException e)   {
					// pu essere genereato dal readBoolean
//					e.printStackTrace();
					this.networkManager.UpdateUploadState(false);
					PolitellaException.ProcessException (null,new PolitellaException(PolitellaException.NO_UPLOAD_TRANSFER,this.iPDest));
				}
			}else	{
				//il thread  stato creato per richiedere...
				this.MakeTheConnection();

				this.networkManager.UpdateDownloadState(true,null,0,null);
				try {
					if(this.busCompElementRemoteLocation==null) {
						// ...1 progetto

						// specifica che cosa mi serve...
						// true -> voglio un prj
						//false -> risorsa associata ad un business component
						this.out.writeBoolean(true);
						this.out.flush();

						//mistero
//						this.in.readInt();

						File[] prjFile = new File[1];
						prjFile[0]=this.GetTheProject();

						this.networkManager.UpdateDownloadState(false,prjFile,this.prjID,this.iPDest);

					}   else    {
						// un elemento di un business component

						// specifica che cosa mi serve...
						// true -> voglio un prj
						//false -> risorsa associata ad un business component
						this.out.writeBoolean(false);
						this.out.flush();

  					    File busCompElementFileTmp = this.GetBusCompElement();

						String busCompElementFileName = this.busCompElementRemoteLocation.substring(this.busCompElementRemoteLocation.lastIndexOf(System.getProperty("file.separator").charAt(0))+1,this.busCompElementRemoteLocation.length());

						File busCompElementFile = new File(this.busCompElementLocalPath,busCompElementFileName);

						this.busCompElementLocalPath.mkdirs();
						//sposto il fiel scaricato dalla dir temp a quella definitiva
						busCompElementFileTmp.renameTo(busCompElementFile);

	        			//per gli indici usati in get vedi costruttore
						File[] fileList = new File[((Integer)this.resTransferInProgressInfos.get(1)).intValue()];

//						System.out.println(this.resTransferInProgressInfos);

						for(int i=0;i<fileList.length;i++)  {
							if(i==((Integer)this.resTransferInProgressInfos.lastElement()).intValue())
								fileList[i]=busCompElementFile;
							else
								fileList[i]=null;
						}

						this.networkManager.UpdateDownloadState(false,fileList,this.prjID,"LOCAL");

//						System.out.println("down terminato");
					}
				}catch(IOException e)   {
					//pu essere generato dal writeBoolean o dal flush
//					e.printStackTrace();
					this.networkManager.UpdateDownloadState(false,null,0,null);
				    PolitellaException.ProcessException(null,new PolitellaException(PolitellaException.NO_DOWNLOAD_TRANSFER,this.iPDest));
				}
			}
		}catch(PolitellaException e){
			if(e.GetType()==PolitellaException.NO_DOWNLOAD_TRANSFER)
				this.networkManager.UpdateDownloadState(false,null,0,null);
			else    {
				if(e.GetType()==PolitellaException.NO_UPLOAD_TRANSFER)
			// un'eccezione di Upload
					this.networkManager.UpdateUploadState(false);
			}
			// l'eccezione pu essere anche generato dal metodo MakeConnection
			PolitellaException.ProcessException(null,e);
		} finally	{
			this.Close();
		}
	}

	/**
       Libera le risorse allocate per la gestione di un canale TCP di download
       @roseuid 3AED3AA00163
     */
    public void Close() {
		try	{
			if(this.in!=null)
				this.in.close();
			if(this.out!=null)
				this.out.close();
			if(this.peerSocket!=null)
				this.peerSocket.close();
		}catch(IOException e)	{}
    }

	/**
       Richiede un progetto specifico ad un altro peer.
       @roseuid 3AEE810501C1
     */
    public File GetTheProject() throws PolitellaException{
		File prjFile=null;
		try	{
//			System.out.println("Spcificao il prj che voglio");
			// richiede un progetto specifico
			this.out.writeLong(this.prjID);
			this.out.flush();

//			System.out.println("Atendo la lunghezza del prj...");
			//attende di ricevere la lunghezza del file del progetto
			long sizeFilePrj = this.in.readLong();

//		    System.out.print(sizeFilePrj);

            if(sizeFilePrj==-1)
            //il file non esiste piu', e' stato cancellato tra la ricerca precedente e la successiva richiesta di download
                throw new PolitellaException(PolitellaException.NO_DOWNLOAD_TRANSFER,""+this.prjID+".xml");
			else    {
               //il file esiste lo posso scaricare
//                sizeBuffer=100;
//				int sizeBuffer=sizeFilePrj;
	            byte[] buf= new byte[1024];

				// attende di ricevere il nome del progetto
	            int bytes_read=0;
	            bytes_read=this.in.read(buf);

	            String filenamePrj = new String(buf,0,bytes_read);

				prjFile = File.createTempFile(filenamePrj,null);
				prjFile.deleteOnExit();

                //mi serve per eliminare un problema di ricezione sui socket...mistero
				this.out.writeShort(5);
				this.out.flush();

    			//attende di ricevere il file del progetto
    			BufferedOutputStream filePrjOut = new BufferedOutputStream(new FileOutputStream(prjFile),10*ThreadDirectDownload.DIMMAXPACKET);

//                int file=0;
	            buf= new byte[ThreadDirectDownload.DIMMAXPACKET];
                bytes_read=0;
//    			while(bytes_read<sizeFilePrj)   {
    			while(sizeFilePrj>0)   {
//                    System.out.print("|" + bytes_read);
//    				bytes_read+=this.in.read(buf,bytes_read,sizeFilePrj-bytes_read);
    				bytes_read=this.in.read(buf);
	    			sizeFilePrj-=bytes_read;
					filePrjOut.write(buf,0,bytes_read);
//                    file++;
    			}
    			filePrjOut.close();
				return prjFile;
//                System.out.println("Letti:" + file);
            }
		}catch(IOException e)	{
			throw new PolitellaException(PolitellaException.NO_DOWNLOAD_TRANSFER,""+this.prjID+".xml");
		}
    }

    /**
       Controlla l'esistenza del progetto richiesto ed eventualmente lo invia.
       @roseuid 3AEE81BC032C
     */
    public boolean Search_SendTheProject() throws PolitellaException{
		try	{
			// ricerca progetto specifico
//			System.out.println("Attendo il nome del prj");
			this.prjID=this.in.readLong();

			this.resTransferInProgressInfos.add(new String(this.prjID + ".xml"));

//			System.out.println("Cerco il prj specificato");
			File filePrj=this.networkManager.GetProjectFile(this.prjID,true,false);

            if(filePrj==null)   {
            //il file richiesto non esite piu', e' stato cancellato tra la ricerca fatta e la successiva richiesta di download
    			this.out.writeInt(-1);
				this.out.flush();

	    		//invia il nome del file del progetto
//		    	this.out.writeBytes(new String("unavailable"));
                return false;
            }
			else    {
                //invia la lunghezza del file del progetto
    			this.out.writeLong(filePrj.length());
				this.out.flush();

	    		//invia il nome del file del progetto
				this.out.writeBytes(filePrj.getName());
				this.out.flush();

                // se non lo metto in ricezione riceve il nome + una parte del pacchetto successivo,
                //cosi' non ho problemi
                this.in.readShort();

	    		//invio progetto specificco
				BufferedInputStream filePrjIn = new BufferedInputStream (new FileInputStream(filePrj),10*ThreadDirectDownload.DIMMAXPACKET);
    			byte[] buf= new byte[ThreadDirectDownload.DIMMAXPACKET];

//                int file=0;

                int bytes_read=0;
    			while((bytes_read=filePrjIn.read(buf))!=-1) {
    				this.out.write(buf,0,bytes_read);
//                    file++;
    			}
    			this.out.flush();
				filePrjIn.close();

//                System.out.println("Fatti:" + file);
    			return true;
            }
		}catch(IOException e)	{
//		    e.printStackTrace();
			throw new PolitellaException(PolitellaException.NO_UPLOAD_TRANSFER,this.iPDest);
		}
    }

	/**
	 * Invia una risorsa associata ad un business component
	 */
	public boolean SendBusCompElement()  throws PolitellaException{
	    try {
			byte[] buf= new byte[1024];

			// attende di ricevere il nome della risorsa da inviare
	        int bytes_read=0;
	        bytes_read=this.in.read(buf);
	        String busCompElementFilename = new String(buf,0,bytes_read);

			//ctrl se esiste la risorsa richiesta
			File busCompElementFile = new File(busCompElementFilename);
			if(!busCompElementFile.exists()) {
				//il file non esiste, glielo devo comunicare
    			this.out.writeInt(-1);
				this.out.flush();

				return false;
			}
			else    {
				this.resTransferInProgressInfos.add(busCompElementFile.getName());
				//il file esiste adesso lo devo inviare

                //invia la lunghezza del file della risorsa
    			this.out.writeLong(busCompElementFile.length());
				this.out.flush();

	    		//invio la risorsa richiesta
				BufferedInputStream prjFileIn = new BufferedInputStream (new FileInputStream(busCompElementFile),10*ThreadDirectDownload.DIMMAXPACKET);
    			buf= new byte[ThreadDirectDownload.DIMMAXPACKET];

                bytes_read=0;
    			while((bytes_read=prjFileIn.read(buf))!=-1)
    				this.out.write(buf,0,bytes_read);
				this.out.flush();
    			prjFileIn.close();

				return true;
			}
		}catch(IOException e)	{
//			e.printStackTrace();
			throw new PolitellaException(PolitellaException.NO_UPLOAD_TRANSFER,this.iPDest);
		}

	}

	/**
	 * Riceve una risorsa associata ad un business component
	 */
	public File GetBusCompElement() throws PolitellaException{
		File prjFile=null;

		try {
	    	//invio il nome della risorsa che voglio
			this.out.writeBytes(this.busCompElementRemoteLocation);
			this.out.flush();

			//attende di ricevere la lunghezza del file della risorsa
			long sizeFilePrj = this.in.readLong();

            if(sizeFilePrj==-1)
            //il file non esiste piu', e' stato cancellato tra la ricerca precedente e la successiva richiesta di download
                throw new PolitellaException(PolitellaException.NO_DOWNLOAD_TRANSFER,this.busCompElementRemoteLocation);
			else    {
				//ricavo il nome della risorsa che sto per ricevere
				String fileName = this.busCompElementRemoteLocation.substring(this.busCompElementRemoteLocation.lastIndexOf(System.getProperty("file.separator").charAt(0))+1,this.busCompElementRemoteLocation.length());
				prjFile = File.createTempFile(fileName,null);
				prjFile.deleteOnExit();

	            //attende di ricevere il file del progetto
				byte[] buf= new byte[ThreadDirectDownload.DIMMAXPACKET];

    			BufferedOutputStream filePrjOut = new BufferedOutputStream(new FileOutputStream(prjFile),10*ThreadDirectDownload.DIMMAXPACKET);

                int bytes_read=0;
    			while(sizeFilePrj>0)   {
    				bytes_read=this.in.read(buf);
	    			sizeFilePrj-=bytes_read;
					filePrjOut.write(buf,0,bytes_read);
    			}
    			filePrjOut.close();

				return prjFile;
			}
		}catch(IOException e)	{
//			e.printStackTrace();
			throw new PolitellaException(PolitellaException.NO_DOWNLOAD_TRANSFER,this.iPDest+"@"+this.busCompElementRemoteLocation);
		}
	}

	/**
	 * Fornisce delle informazioni relative alla connessione in corso:
	 * - l'indirizzo IP
	 * - la risorsa che viene trasferita
	 */
	public Vector GetInfoConnection() {
		Vector tmp = new Vector();
		tmp.addElement(this.iPDest);
		if(this.resTransferInProgressInfos.firstElement()==null)
			// ho appena aperto la connessione non so ancora quale progetto gestisco in upload
			//o download
			tmp.addElement(new String("Not yet available"));
		else
			tmp.addElement((String)this.resTransferInProgressInfos.firstElement());
		return tmp;
	}
}
