/*
*                            Copyright (c) 2001
*                        <B>Politecnico di Torino</B>
*                            All Rights Reserved
* <p>
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that: (1) source code distributions
* retain the above copyright notice and this paragraph in its entirety,(2)
* distributions including binary code include the above copyright notice and
* this paragraph in its entirety in the documentation or other materials
* provided with the distribution, and (3) all advertising materials mentioning
* features or use of this software display the following acknowledgement:
* ``This product includes software developed by Politecnico
* di Torino, and its contributors.'' (4) Neither the name of
* the University nor the names of its contributors may be used to endorse
* or promote products derived from this software without specific prior
* written permission.
* THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
* This product includes code licensed from RSA Data Security (Java Secure Socket Extension)
*
* Sun grants you ("Licensee") a non-exclusive, royalty free, license to use, and redistribute
* this software graphics artwork, as individual graphics or as a collection, as part of
* software code or programs that you develop, provided that
* i) this copyright notice and license accompany the software graphics artwork; and
* ii) you do not utilize the software graphics artwork in a manner which is disparaging to Sun.
* Unless enforcement is prohibited by applicable law, you may not modify the graphics,
* and must use them true to color and unmodified in every way.
*
*<p>
*/
package Network;

import Utility.PolitellaException;

//import java.net.ServerSocket;
//import java.net.Socket;
import javax.net.ssl.SSLServerSocketFactory;
import javax.net.ssl.SSLServerSocket;
import javax.net.ssl.SSLSocket;

import com.sun.net.ssl.SSLContext;
//import com.sun.net.ssl.*;
//import java.security.KeyStore;

import java.io.InterruptedIOException;
import java.io.IOException;
import java.io.File;

/**
   Thread che gestisce le nuove richieste di comunicazioni in ingresso. Semplicemente accetta la connessione a livello TCP e genera un thread ThreadManageConnection
 */
final class ThreadIncomingNewConnection extends Thread{
	/**
	 * Porta di ascolto
	 */
	private int listenPort;
	/**
	 * Socket di ascolto
	 */
//	private ServerSocket listenSocket = null;
	private SSLServerSocket listenSocket = null;
	/**
	 * SSL Server Socket factory
	 */
	private static SSLServerSocketFactory sslSrvSktFactory =null;
	/**
	 * Identificativo del networkManager
	 */
	private static NetworkManager networkManager = null;
    /**
     * Costruttore
     */
	public ThreadIncomingNewConnection(int listenPort, NetworkManager networkManager) throws PolitellaException{
		this.networkManager=networkManager;
		this.listenPort=listenPort;
		try {
//			this.listenSocket = new ServerSocket(this.listenPort);

		    this.listenSocket=(SSLServerSocket)this.sslSrvSktFactory.createServerSocket(this.listenPort);

			this.listenSocket.setNeedClientAuth(true);

		}catch(IOException e1)	{
			 /**
			  * Impossible creare il socket
			  */
			throw new PolitellaException(PolitellaException.NO_LISTEN_SOCKET,(new Integer(this.listenPort)).toString());
		}
    }

	/**
	 * Inizializza l'SSLServerSocketFactory
	 */
	 static void InitSSLEnvironment(SSLContext ctx){
/*	 static void InitSSLEnvironment()throws PolitellaException {

///////////////////
try {
			char[] passphrase = "Politella".toCharArray();

			KeyStore ks = KeyStore.getInstance("JKS");
			ks.load(new java.io.FileInputStream(new File(System.getProperty("user.dir"),"keystoreDSA1024.keystore")), passphrase);

			KeyManagerFactory kmf = KeyManagerFactory.getInstance("SunX509");
			kmf.init(ks, passphrase);

			TrustManagerFactory tmf = TrustManagerFactory.getInstance("SunX509");
			tmf.init(ks);

			SSLContext ctx	= SSLContext.getInstance("TLS");
			ctx.init(kmf.getKeyManagers(), tmf.getTrustManagers(), null);
			sslSrvSktFactory = ctx.getServerSocketFactory();
		 }catch(java.lang.Exception e)  {
			throw new PolitellaException(PolitellaException.NO_SSL_INITIALIZATION,null);
		 }
			////////////////////
*/
			sslSrvSktFactory = ctx.getServerSocketFactory();
	 }

	public void run()	{
//		Socket peerSocket=null;
		SSLSocket peerSocket=null;
		try {
			while(true)	{
				//System.out.println("attendo");
				try{
					this.listenSocket.setSoTimeout(9899);

//					peerSocket = this.listenSocket.accept();
	    			peerSocket = (SSLSocket)this.listenSocket.accept();
					}catch(InterruptedIOException e){
//	    				System.out.println(this.getName()+" interrotto");
				}
				if(peerSocket!=null)	{
					if(this.listenPort==NetworkManager.msgListenPort)
						// ho ricevuto una richiesta di connessione per messaggi
						(new ThreadManageConnection(peerSocket,this.networkManager)).start();
					else
						// ho ricevuto una richiesta di connessione per download
						(new ThreadDirectDownload(peerSocket,this.networkManager)).start();
					peerSocket=null;
					}
	    		}
		}catch(SecurityException e2)	{
			/**
			 * thread bloccato, il NetworkManager ha lanciato il metodo Exit()
			 */
//			 System.out.println("1 Thread ucciso");
		}catch(PolitellaException e3)	{
			 //non  stato possibile creare un thread x download o gestione messaggi
//			 System.out.println("Thread riceve politellaexception");
			 PolitellaException.ProcessException(null,e3);
//			 System.out.println("2 Thread ucciso");
		}catch(IOException e1)	{
			 /**
			  * Impossible creare il socket
			  */
			PolitellaException e = new PolitellaException(PolitellaException.NO_LISTEN_SOCKET,(new Integer(this.listenPort)).toString());
			PolitellaException.ProcessException(null,e);
//			 System.out.println("3 Thread ucciso");
		}finally {
			try	{
//				System.out.println("chiudo");
				this.listenSocket.close();
				if(peerSocket!=null)
					peerSocket.close();
			}catch(IOException e4)	{}
		}
	}
}
