/*
*                            Copyright (c) 2001
*                        <B>Politecnico di Torino</B>
*                            All Rights Reserved
* <p>
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that: (1) source code distributions
* retain the above copyright notice and this paragraph in its entirety,(2)
* distributions including binary code include the above copyright notice and
* this paragraph in its entirety in the documentation or other materials
* provided with the distribution, and (3) all advertising materials mentioning
* features or use of this software display the following acknowledgement:
* ``This product includes software developed by Politecnico
* di Torino, and its contributors.'' (4) Neither the name of
* the University nor the names of its contributors may be used to endorse
* or promote products derived from this software without specific prior
* written permission.
* THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
* This product includes code licensed from RSA Data Security (Java Secure Socket Extension)
*
* Sun grants you ("Licensee") a non-exclusive, royalty free, license to use, and redistribute
* this software graphics artwork, as individual graphics or as a collection, as part of
* software code or programs that you develop, provided that
* i) this copyright notice and license accompany the software graphics artwork; and
* ii) you do not utilize the software graphics artwork in a manner which is disparaging to Sun.
* Unless enforcement is prohibited by applicable law, you may not modify the graphics,
* and must use them true to color and unmodified in every way.
*
*<p>
*/
package Network;

//import Protocollo.*;
import Network.Protocol.*;
import Utility.PolitellaException;

import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.File;

import java.util.Vector;

import java.io.EOFException;
import java.io.IOException;
import java.io.InterruptedIOException;
import java.io.OptionalDataException;
import java.net.SocketException;
import java.net.ConnectException;
import java.net.UnknownHostException;
import java.net.InetAddress;

//import java.net.Socket;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.SSLSocket;

import com.sun.net.ssl.SSLContext;
//import com.sun.net.ssl.*;
//import java.security.KeyStore;



/**
   Thread che gestisce la comunicazione tra i peer, ma non il download dei progetti.
 */
final class ThreadManageConnection extends Thread{
	/**
	 * Identificativo del networkManager
	 */
	private static NetworkManager networkManager = null;
	/**
	 * Stream di Input
	 */
	private ObjectInputStream in = null;
	/**
	 * Stream di Output
	 */
	private ObjectOutputStream out = null;
	/**
	 * Socket di connessione
	 */
//	private Socket peerSocket = null;
	private SSLSocket peerSocket = null;
	/**
	 * SSL Socket factory
	 */
	 private static SSLSocketFactory sslSktFactory = null;
	/**
	 * Numero di progetti condivisi dal peer con cui si  connessi
	 */
	 private long numPrjShare=-1;
     /**
      * Indica lo stato della fase di autenticazione:
      * - true -> e'  terminata
      * - false -> non terminata
      */
     private boolean connectionAccepted = false;
     /**
      * Indirizzo IP a cui il thread e' connesso
      */
      String iPDest=null;
    /**
       Costruttore.
       Avviata quando il peer apre volontariamente una nuova connessione.
       @roseuid 3AED59720367
     */
    public ThreadManageConnection(String iPDest, NetworkManager networkManager) throws PolitellaException{
		super(networkManager.dbThreadManageConnection,"MsgThread"+(new java.util.Random()).nextLong());

        this.networkManager=networkManager;

		try	{

//            this.peerSocket = new Socket(InetAddress.getByName(iPDest),NetworkManager.msgListenPort);
			this.peerSocket=(SSLSocket)this.sslSktFactory.createSocket(iPDest,NetworkManager.msgListenPort);

		    this.networkManager.SetMyIPAddress(this.peerSocket.getLocalAddress().getHostAddress());

    		this.iPDest=this.peerSocket.getInetAddress().getHostAddress();
			this.out = new ObjectOutputStream(peerSocket.getOutputStream());
			this.in = new ObjectInputStream(peerSocket.getInputStream());

			Connect connect= new Connect(this.networkManager.myIPAddress,this.networkManager.defaultMsgTTL);
			this.SendMsg(connect);
//            System.out.println("Inviato una CONNECT");

//            System.out.println("Riceve:" + this.peerSocket.getReceiveBufferSize()+ "Trasmette: " +this.peerSocket.getSendBufferSize());
      }catch(UnknownHostException e1)	{
			// impossibile connettersi con l'host specificato
//			System.out.println("impossibile connettersi con l'host specificato 1");
            this.Close();
			throw new PolitellaException(PolitellaException.NO_MESSAGE_CONNECTION,iPDest);
		} catch(IOException e2)	{
			// impossibile connettersi con l'host specificato
//            e2.printStackTrace();
//			System.out.println("impossibile connettersi con l'host specificato 2");
			this.Close();
			throw new PolitellaException(PolitellaException.NO_MESSAGE_CONNECTION,iPDest);
		}
    }
    /**
       Costruttore.
       Avviata quando il peer riceve una nuova richiesta di connessione.
       @roseuid 3AED5DED001D
     */
//    public ThreadManageConnection(Socket peerSocket, NetworkManager networkManager) throws PolitellaException{
    public ThreadManageConnection(SSLSocket peerSocket, NetworkManager networkManager) throws PolitellaException{
		super(networkManager.dbThreadManageConnection,"MsgThread"+(new java.util.Random()).nextLong());

//        System.out.println("nuovo");
		this.peerSocket=peerSocket;
		this.iPDest=this.peerSocket.getInetAddress().getHostAddress();
		this.networkManager=networkManager;

	    this.networkManager.SetMyIPAddress(this.peerSocket.getLocalAddress().getHostAddress());

		try	{
            this.in = new ObjectInputStream(peerSocket.getInputStream());
			this.out = new ObjectOutputStream(peerSocket.getOutputStream());
		}catch(IOException e)	{
//            e.printStackTrace();
//			System.out.println("errore nella creazione del socket - ricevuto");
			this.Close();
			throw new PolitellaException(PolitellaException.NO_MESSAGE_CONNECTION,this.peerSocket.getInetAddress().getHostAddress());
		}
    }

	/**
	 * Inizializza l'SSLSocketFactory
	 */
	 static void InitSSLEnvironment(SSLContext ctx){
/*	 static void InitSSLEnvironment() throws PolitellaException{

///////////////////
try {
			char[] passphrase = "Politella".toCharArray();

			KeyStore ks = KeyStore.getInstance("JKS");
			ks.load(new java.io.FileInputStream(new File(System.getProperty("user.dir"),"keystoreDSA1024.keystore")), passphrase);

			KeyManagerFactory kmf = KeyManagerFactory.getInstance("SunX509");
			kmf.init(ks, passphrase);

			TrustManagerFactory tmf = TrustManagerFactory.getInstance("SunX509");
			tmf.init(ks);

			SSLContext ctx	= SSLContext.getInstance("TLS");
			ctx.init(kmf.getKeyManagers(), tmf.getTrustManagers(), null);
			sslSktFactory = ctx.getSocketFactory();
		 }catch(java.lang.Exception e)  {
			throw new PolitellaException(PolitellaException.NO_SSL_INITIALIZATION,null);
		 }


////////////////////
*/

			sslSktFactory = ctx.getSocketFactory();
	 }

	public void run()	{
		Object objTmp = null;
		Header headerTmp = null;

		//con questa variabile distinguo tra i thread creati esclusivamente per
		//la ricezione di PONG e QUERY_REPLY e tutti gli altri.
		boolean continueThread=true;
		try	{
			/**
			 * attendo di ricevere sul socket
			 */
			while(continueThread)	{
				while(true)	{
					try	{
					// serve perch il thread senta lo stop inviato dal network manager
					// quando si vuole chiudere l'applicazione
                        this.peerSocket.setSoTimeout(10000);
                        objTmp=this.in.readObject();
						break;
					}catch(InterruptedIOException e){}
				}
				headerTmp = (Header) objTmp;
				switch(headerTmp.GetType())	{
					case Header.CONNECT:{
//											System.out.println("Ricevuto CONNECT di "+headerTmp.GetIPSource()+" da " + this.peerSocket.getInetAddress().getHostAddress());
                                            continueThread=this.ProcessCONNECT((Connect)objTmp);
											break;
										}
					case Header.CONNECT_REPLY:{
//									            java.util.Calendar tmp= java.util.Calendar.getInstance();
//                                                System.out.println("Ricevuto un CONNECT_REPLY alle" + tmp.getTime().toGMTString());
//											System.out.println("Ricevuto CONNECT_REPLY di "+headerTmp.GetIPSource()+" da " + this.peerSocket.getInetAddress().getHostAddress());
												continueThread=this.ProcessCONNECT_REPLY((ConnectReply)objTmp);
												break;
											}
					case Header.PING:{
//											System.out.println("Ricevuto PING di "+headerTmp.GetIPSource()+" da " + this.peerSocket.getInetAddress().getHostAddress());
                                        continueThread=this.ProcessPING((Ping)objTmp);
										break;
									}
					case Header.PONG:{
//											System.out.println("Ricevuto PONG di "+headerTmp.GetIPSource()+" da " + this.peerSocket.getInetAddress().getHostAddress());
                                        continueThread=this.ProcessPONG((Pong)objTmp);
//										if(!onlyForPONGorQUERY_REPLY)
//											continueThread=true;
										break;
									}
					case Header.QUERY:{
//											System.out.println("Ricevuto QUERY di "+headerTmp.GetIPSource()+" da " + this.peerSocket.getInetAddress().getHostAddress());
										continueThread=this.ProcessQUERY((Query)objTmp);
										break;
									}
					case Header.QUERY_REPLY:{
//											System.out.println("Ricevuto QUERY_REPLY di "+headerTmp.GetIPSource()+" da " + this.peerSocket.getInetAddress().getHostAddress());
												continueThread=this.ProcessQUERY_REPLY((QueryReply)objTmp);
//												if(!onlyForPONGorQUERY_REPLY)
//													continueThread=true;
												break;
											}
				}
                objTmp=null;
                headerTmp=null;
			}
		//  terminato il dialogo tra i peer, posso chiudere tutto
		// vado in finally
		}catch(PolitellaException e)    {
//			 System.out.println("1 Thread ucciso");
			PolitellaException.ProcessException(null,e);
		}catch(SecurityException e1)	{
			/**
			 * thread bloccato, il NetworkManager ha lanciato il metodo Exit()
			 */
			e1.printStackTrace();
		}catch(OptionalDataException e)	{
//			 System.out.println("3 Thread ucciso");
//			 e.printStackTrace();
             PolitellaException e1= new PolitellaException(PolitellaException.NO_MESSAGE_TRANSFER,this.peerSocket.getInetAddress().getHostAddress());
			 PolitellaException.ProcessException(null,e1);
		}catch(ClassNotFoundException e)	{
//			 e.printStackTrace();
			 PolitellaException e1= new PolitellaException(PolitellaException.NO_MESSAGE_TRANSFER,this.peerSocket.getInetAddress().getHostAddress());
			 PolitellaException.ProcessException(null,e1);
		}catch(ClassCastException e)	{
			 //ho ricevuto un oggetto sconosciuto
//			 e.printStackTrace();
			 PolitellaException e1= new PolitellaException(PolitellaException.NO_MESSAGE_TRANSFER,this.peerSocket.getInetAddress().getHostAddress());
			 PolitellaException.ProcessException(null,e1);
		}catch(EOFException e)	{
			//connessione chiusa dall'altro peer
//			e.printStackTrace();
		}catch(SocketException e)   {
            //connessione chiusa brutalmente dall'altro peer
//			e.printStackTrace();
        }catch(IOException e)	{
//			 System.out.println("8 Thread ucciso");
//			 e.printStackTrace();
			 PolitellaException e1= new PolitellaException(PolitellaException.NO_MESSAGE_TRANSFER,this.peerSocket.getInetAddress().getHostAddress());
			 PolitellaException.ProcessException(null,e1);
		}finally	{
			if(this.connectionAccepted)
//                System.out.println("Sono nel finally");
            this.Close();
		}
	}
    /**
       Libera le risorse allocate per la gestione di un canale TCP di comunicazione
       @roseuid 3AED393D01F8
     */
    public void Close() {
		try	{
			if(this.in!=null)
				this.in.close();
			if(this.out!=null)
				this.out.close();
			if(this.peerSocket!=null)
				this.peerSocket.close();
		}catch(IOException e)	{}

		//la connessione  chiusa, decremento il contatore delle connessioni attive
        if(this.connectionAccepted) {
           //se e' true allora
	   		this.networkManager.UpdateActiveConnectionsState(false);
//            System.out.println("Chiuso tutto");
        }
    }

    /**
       Invia un messaggio
       @roseuid 3AED5AE50161
     */
    public synchronized void SendMsg(Object msg) throws PolitellaException{
		try	{
			this.out.writeObject(msg);
			this.out.flush();
			this.out.reset();
		}catch (IOException e)	{
//			e.printStackTrace();
            throw new PolitellaException(PolitellaException.NO_MESSAGE_TRANSFER,this.peerSocket.getInetAddress().getHostAddress());
		}
    }
	/**
	 * Processo il CONNECT ricevuto
	 *
	 * se accetto return true
	 * se rifiuto return false
	 */
	private boolean ProcessCONNECT(Connect connect) throws PolitellaException {
		ConnectReply tmp=null;
		if(this.networkManager.GetActiveConnectionsNum()<=this.networkManager.numMaxActiveConnections)  {
			//posso accettare quasta nuova connessione
			if(Connect.CheckProtocol(connect))	{
	    	   //protocollo OK controllo la versione
		       if(Connect.CheckVersion(connect))    {
			    	//versione OK, invio CONNECT_REPLY tipo OK
				tmp= new ConnectReply(this.networkManager.myIPAddress,
				    					 this.networkManager.defaultMsgTTL,
					    				 ConnectReply.OK,null);
		       }
			   else {
	    	       //versione leggermente differente, invio CONNECT_REPLY tipo OK_WRONG_VERSION
		    	   tmp= new ConnectReply(this.networkManager.myIPAddress,
				    					 this.networkManager.defaultMsgTTL,
					    				 ConnectReply.OK_WRONG_VERSION,Connect.GetVersion());
			   }
                this.connectionAccepted=true;
        		//la connessione  attiva, incrmento il contatore delle connessioni attive
        		this.networkManager.UpdateActiveConnectionsState(true);
		    }
		    else
			    //protocollo differente, invio CONNECT_REPLY tipo NO_WRONG_PROTOCOL_OR_VERSION
			    tmp= new ConnectReply(this.networkManager.myIPAddress,
				    					 this.networkManager.defaultMsgTTL,
					    				 ConnectReply.NO_WRONG_PROTOCOL_OR_VERSION,(Connect.GetProtocol()+" "+Connect.GetVersion()));
		}
		else    {
		//non posso accettare quuesta nuova connessione
		//ne ho gi troppe attive in questo momento
			tmp = new ConnectReply(this.networkManager.myIPAddress,this.networkManager.defaultMsgTTL,
								    ConnectReply.NO_TOO_MANY_CONNECTION,null);
//            System.out.println("troppe connessioni");
		}
		this.SendMsg(tmp);
		return true;
	}
	/**
	 * Processo il CONENCT_REPLY ricevuto
	 */
	private boolean ProcessCONNECT_REPLY(ConnectReply connectReply) {
		if((connectReply.GetAnswerType()==ConnectReply.NO_TOO_MANY_CONNECTION) ||
		   (connectReply.GetAnswerType()==ConnectReply.NO_WRONG_PROTOCOL_OR_VERSION))
			// la richiesta di connessione non pu essere accettata
			return false;
		// la richiesta di connessione  stata accettata
        this.connectionAccepted=true;
		//la connessione  attiva, incrmento il contatore delle connessioni attive
		this.networkManager.UpdateActiveConnectionsState(true);
		return true;
	}
	/**
	 * Processo il PING ricevuto
	 */
	private boolean ProcessPING(Ping ping) throws PolitellaException{
		if(!this.networkManager.SearchMsgInCache(ping))	{
			//il PING ricevuto non esiste nella cache
			this.networkManager.SaveMsgInCache(ping);
            if(ping.GetIPSource().compareTo(this.iPDest)!=0)
                //il PING ricevuto non e' del peer con cui sono direttamente connesso
                //puo' darsi che sia un peer nuovo
                this.networkManager.CheckPeerAddressForNewConnection(ping.GetIPSource());
            else    {
                // invio il PONG direttamente al mittente del PING
				Pong pong = new Pong(ping.GetUniqueID(),
									this.networkManager.myIPAddress,
									this.networkManager.defaultMsgTTL,
									this.networkManager.GetNumPrjShare());
					if(ping.GetIPSource().compareTo(this.iPDest)==0)    {
						//se il PING l'ho ricevuto direttamente da chi l'ha generato allora
						//posso invairglielo sulla connessione gi aperta, altrimenti ne
						//istanzio una nuova, che verr chiusa immediatamente dopo
						this.SendMsg(pong);

//						System.out.println("invio PONG");
					}
					else    {
//						Socket tmpSocket=null;
						SSLSocket tmpSocket=null;
						ObjectOutputStream outTmp = null;
						try {
//							tmpSocket = new Socket(ping.GetIPSource(),this.networkManager.msgListenPort);
							tmpSocket=(SSLSocket)this.sslSktFactory.createSocket(ping.GetIPSource(),this.networkManager.msgListenPort);
						    outTmp = new ObjectOutputStream(tmpSocket.getOutputStream());
							outTmp.writeObject(pong);
							outTmp.flush();
	                        Thread.sleep(500);
						}catch(InterruptedException e1){
						}catch(UnknownHostException e)	{
						}catch(IOException e2)  {
						}finally    {
							try {
								outTmp.close();
							    tmpSocket.close();
							}catch(IOException e){}
						}
					}
            }
			//inoltro il PING sulle altre connessioni aperte
			if(ping.DecreaseTTL())
			//posso inoltrare il messaggio
				this.networkManager.SendMsgToAllPeer(ping,this.getName());
		}
		return true;
	}
	/**
	 * Processo il QUERY ricevuto
	 */
	private boolean ProcessQUERY(Query query) throws PolitellaException{
		if(!this.networkManager.SearchMsgInCache(query))	{
			//il QUERY ricevuto non esiste nella cache
			this.networkManager.SaveMsgInCache(query);
            if(query.GetIPSource().compareTo(this.iPDest)!=0)
                //il QUERY ricevuto non e' del peer con cui sono direttamente connesso
                //puo' darsi che sia un peer nuovo
                this.networkManager.CheckPeerAddressForNewConnection(query.GetIPSource());
			Vector queryHits=this.networkManager.SearchProjectInLocal(query.GetProjectType(),query.GetSearchKey());
			if(!queryHits.isEmpty())	{
				//ho trovato almeno una occorrenza del progetto cercato
				// invio il QUERY_REPLY direttamente al mittente del QUERY
				QueryReply queryReply= new QueryReply(query.GetUniqueID(),this.networkManager.myIPAddress,queryHits);
				if(query.GetIPSource().compareTo(this.iPDest)==0)
                   	//se il QUERY l'ho ricevuto direttamente da chi l'ha generato allora
					//posso invairglielo sulla connessione gi aperta, altrimenti ne
					//istanzio una nuova, che verr chiusa immediatamente dopo
					this.SendMsg(queryReply);
				else	{
//					Socket tmpSocket = null;
				    SSLSocket tmpSocket = null;
					ObjectOutputStream outTmp = null;
					try {
//						tmpSocket = new Socket(query.GetIPSource(),this.networkManager.msgListenPort);
						tmpSocket=(SSLSocket)this.sslSktFactory.createSocket(query.GetIPSource(),this.networkManager.msgListenPort);

						outTmp = new ObjectOutputStream(tmpSocket.getOutputStream());
						outTmp.writeObject(queryReply);
						outTmp.flush();
                        Thread.sleep(500);
					}catch(InterruptedException e1){
					}catch(UnknownHostException e)	{
					}catch(IOException e)   {
					}finally    {
						try {
							outTmp.close();
						    tmpSocket.close();
                        }catch(IOException e){}
					}
				}
			}
			//inoltro il QUERY sulle altre connessioni aperte
			if(query.DecreaseTTL())
				//posso inoltrare il messaggio
				this.networkManager.SendMsgToAllPeer(query,this.getName());
		}
		return true;
	}
	/**
	 * Processo il PONG ricevuto
	 */
	private boolean ProcessPONG(Pong pong) {
		if(this.networkManager.SearchMsgInCache(pong))  {
			// il PONG  valido, cio  la risposta ad un precedente PING inviato
		    this.numPrjShare=pong.GetNumSharePrj();
			return true;
			//chi riceve il PONG mantiene in vita il thread anche se il mittente ha chiuso il socket.
			//l'eventuale eccezione EOFException generata dalla chiusura del socket la recupera quando
			//torna a leggere sul socket la volta successiva.
		}
//        System.out.println("Non ho trovato il PING relativo al PONG " + pong.GetUniqueID());
		return false;
	}
	/**
	 * Processo il QUERY_REPLY ricevuto
	 */
	private boolean ProcessQUERY_REPLY(QueryReply queryReply) {
		if(this.networkManager.SearchMsgInCache(queryReply))    {
			// il QUERY_REPLY  valido, cio  la risposta ad un precedente QUERY inviato
			this.networkManager.UpdateSearchResult(queryReply.GetIPSource(),queryReply.GetQueryHits());
//            System.out.println("Ricevuto da "+queryReply.GetIPSource()+" num: "+queryReply.GetQueryHits().size()+" riferimenti");
			return true;
			//chi riceve il QUERY_REPLY mantiene in vita il thread anche se il mittente ha chiuso il socket.
			//l'eventuale eccezione EOFException generata dalla chiusura del socket la recupera quando
			//torna a leggere sul socket la volta successiva.
		}
		return false;
	}

	/**
	 * Fornisce delle informazioni relative alla connessione in corso:
	 * - l'indirizzo IP
	 */
	public Vector GetInfoConnection() {
		Vector tmp=new Vector();
		tmp.addElement(this.iPDest);
		if(this.numPrjShare==-1)
			//ho appena aperto la connessione, non ho ancora ricevuto un PONG su quella connessione
			tmp.addElement(new String("unavailable"));
		else
			tmp.addElement(new Long(this.numPrjShare));
		return tmp;

	}
    /**
     * Restituisce il valore del parametro
     */
     public boolean GetConnectionAccepted() {
        return this.connectionAccepted;
     }
}
