/*
*                            Copyright (c) 2001
*                        <B>Politecnico di Torino</B>
*                            All Rights Reserved
* <p>
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that: (1) source code distributions
* retain the above copyright notice and this paragraph in its entirety,(2)
* distributions including binary code include the above copyright notice and
* this paragraph in its entirety in the documentation or other materials
* provided with the distribution, and (3) all advertising materials mentioning
* features or use of this software display the following acknowledgement:
* ``This product includes software developed by Politecnico
* di Torino, and its contributors.'' (4) Neither the name of
* the University nor the names of its contributors may be used to endorse
* or promote products derived from this software without specific prior
* written permission.
* THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
* This product includes code licensed from RSA Data Security (Java Secure Socket Extension)
*
* Sun grants you ("Licensee") a non-exclusive, royalty free, license to use, and redistribute
* this software graphics artwork, as individual graphics or as a collection, as part of
* software code or programs that you develop, provided that
* i) this copyright notice and license accompany the software graphics artwork; and
* ii) you do not utilize the software graphics artwork in a manner which is disparaging to Sun.
* Unless enforcement is prohibited by applicable law, you may not modify the graphics,
* and must use them true to color and unmodified in every way.
*
*<p>
*/
package Project;

import java.io.File;
import java.io.Serializable;

import java.util.Vector;
import java.util.Enumeration;

import Utility.BusinessModelComponent;
import Utility.ProjectType;

/**
   Elemento del db contenente i progetti scaricati dalla rete ma che non posso ancora condividere perch non ho ancora scaricato tutti i componenti
 */
final class ProjectDown implements Serializable{
	/**
	 * Contiene tutti i componenti del Business Model
	 */
	private transient Vector projectBusModelComponents=null;
    /**
       Descrizione completa del progetto.
     */
    private String projectDescr = null;
    /**
       Identificato del progetto, univoco solamente all'interno del peer su cui risiede
     */
    private long projectID;
    /**
       Specifica il peer su cui si trova questo specifico progetto
     */
    private String projectPeerLocation = null;
    /**
       Tipologia del progetto
     */
    private short[] projectType;
    /**
       Identificatore del file su cui  salvato il progetto.
       Questo identificatore  dato nella forma PATH + FILENAME
     */
    private File projectFile = null;
	/**
	 * Indica l'identificatore che il progetto aveva sul peer da cui  stato scaricato
	 */
	 private long prjOrigID;

    /**
       Costruttore
       @roseuid 3BAC41E30367
     */
    ProjectDown(short[] projectType, String projectDescr, String projectPeerLocation, File projectFile,long prjOrigID) {
		this.projectID=new Long(projectFile.getName().substring(0,projectFile.getName().length()-4)).longValue();
		this.projectDescr=projectDescr;
		this.projectType=projectType;
		this.projectPeerLocation=projectPeerLocation;
		this.projectFile=projectFile;
		this.projectBusModelComponents=new Vector();
		this.prjOrigID=prjOrigID;
    }

    /**
       Restituisce il valore del campo projectType
       @roseuid 3BAC41E30371
     */
    public short[] GetProjectType() {
		return this.projectType;
    }

    /**
       Restituisce il valore del campo projectID
       @roseuid 3BAC41E30372
     */
    public long GetProjectID() {
		return this.projectID;
    }
	/**
	 * Restituisce il valore del campo prjOrigID
	 */
	 public long GetPrjOrigID() {
		return this.prjOrigID;
	 }

    /**
       Restituisce il valore del campo projectDescr
       @roseuid 3BAC41E30373
     */
    public String GetProjectDescr() {
		return this.projectDescr;
    }

    /**
       @roseuid 3BAC41E30374
     */
    public String GetProjectPeerLocation() {
		return this.projectPeerLocation;
    }

    /**
       Restituisce il valore del campo projectFile
       @roseuid 3BAC42B30208
     */
    public File GetProjectFile() {
		return this.projectFile;
    }
	/**
	 * Restituisce il valore dell'attributo  businessModelComponents
	 */
	 public Vector GetProjectBusModelComponents() {
		return this.projectBusModelComponents;
	 }

	/**
	 * Resetta il valore dell'attributo projectBusModelComponents
	 * Viene chiamato quando carico il db dei prj scaricati
	 * oppure quando devo aggiungere un nuovo prj al proprio db
	 */
	 protected void ResetProjectBusModelComponents()    {
		this.projectBusModelComponents=new Vector();
		//i business component sono numerati da 0 e nel vettore sono memorizzati da 0
		for(short busModCompID=0;busModCompID<ProjectType.numBusinessRole+ProjectType.numBusinessRelationship;busModCompID++)   {
			this.projectBusModelComponents.add(new BusinessModelComponent(busModCompID));
		}
	 }

	 /**
	  * Imposta le locazioni delle risorse eventualmente associate ai business component
	  */
	  protected void ResetPrjBusModCompElementsLocation()   {
		Enumeration list = this.projectBusModelComponents.elements();
		short[] busModCompIDLinks=null;
		while(list.hasMoreElements())    {
			BusinessModelComponent curr = (BusinessModelComponent)list.nextElement();

		    String[] arrayOfData=null;

			if(curr.IsActive()) {
				if((arrayOfData=curr.GetBusModCompSourceLoc())!=null) {
					for(int y=0;y<arrayOfData.length;y++)
						arrayOfData[y]=this.projectPeerLocation.concat("@").concat(arrayOfData[y]);
					curr.SetBusModCompSourceLoc(arrayOfData);
				}
				if((arrayOfData=curr.GetBusModCompBinaryLoc())!=null) {
					for(int y=0;y<arrayOfData.length;y++)
						arrayOfData[y]=this.projectPeerLocation.concat("@").concat(arrayOfData[y]);
					curr.SetBusModCompBinaryLoc(arrayOfData);
				}
				if((arrayOfData=curr.GetBusModCompDocLoc())!=null)    {
					for(int y=0;y<arrayOfData.length;y++)
						arrayOfData[y]=this.projectPeerLocation.concat("@").concat(arrayOfData[y]);
					curr.SetBusModCompDocLoc(arrayOfData);
				}

				busModCompIDLinks=curr.GetBusModCompIDLinks();

				if(busModCompIDLinks!=null) {
					for(int i=0;i<busModCompIDLinks.length;i++) {
						if((arrayOfData=curr.GetBusModComp_1_ResSourceLoc(busModCompIDLinks[i]))!=null)   {
							for(int y=0;y<arrayOfData.length;y++)
								arrayOfData[y]=this.projectPeerLocation.concat("@").concat(arrayOfData[y]);
							curr.SetBusModComp_1_ResSourceLoc(arrayOfData,busModCompIDLinks[i]);
					    }
						if((arrayOfData=curr.GetBusModComp_1_ResBinaryLoc(busModCompIDLinks[i]))!=null)   {
							for(int y=0;y<arrayOfData.length;y++)
								arrayOfData[y]=this.projectPeerLocation.concat("@").concat(arrayOfData[y]);
							curr.SetBusModComp_1_ResBinaryLoc(arrayOfData,busModCompIDLinks[i]);
						}
						if((arrayOfData=curr.GetBusModComp_1_ResDocLoc(busModCompIDLinks[i]))!=null)  {
							for(int y=0;y<arrayOfData.length;y++)
								arrayOfData[y]=this.projectPeerLocation.concat("@").concat(arrayOfData[y]);
							curr.SetBusModComp_1_ResDocLoc(arrayOfData,busModCompIDLinks[i]);
						}
					}
				}
			}
		}
	  }

}
