/*
*                            Copyright (c) 2001
*                        <B>Politecnico di Torino</B>
*                            All Rights Reserved
* <p>
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that: (1) source code distributions
* retain the above copyright notice and this paragraph in its entirety,(2)
* distributions including binary code include the above copyright notice and
* this paragraph in its entirety in the documentation or other materials
* provided with the distribution, and (3) all advertising materials mentioning
* features or use of this software display the following acknowledgement:
* ``This product includes software developed by Politecnico
* di Torino, and its contributors.'' (4) Neither the name of
* the University nor the names of its contributors may be used to endorse
* or promote products derived from this software without specific prior
* written permission.
* THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
* This product includes code licensed from RSA Data Security (Java Secure Socket Extension)
*
* Sun grants you ("Licensee") a non-exclusive, royalty free, license to use, and redistribute
* this software graphics artwork, as individual graphics or as a collection, as part of
* software code or programs that you develop, provided that
* i) this copyright notice and license accompany the software graphics artwork; and
* ii) you do not utilize the software graphics artwork in a manner which is disparaging to Sun.
* Unless enforcement is prohibited by applicable law, you may not modify the graphics,
* and must use them true to color and unmodified in every way.
*
*<p>
*/
package Project;

import Utility.*;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.io.File;
import java.io.ObjectInputStream;
import java.io.FileInputStream;
import java.io.ObjectOutputStream;
import java.io.FileOutputStream;

import java.io.EOFException;
import java.io.FileNotFoundException;
import java.io.IOException;

/**
   Modulo di gestione dei db dei propri progetti condivisi e trovati nelle ricerche
 */
public final class ProjectManager {
    /**
       Specifica il PATH in cui tutti i file dei progetti verranno salvati.
     */
    private String defaultPathName = "database";

	/**
	 * Specifica il nome del file in cui vengono salvati i riferimenti
	 * ai progetti condivisi
	 */
	private String dbPrjShareFileName = "prjshare.db";
	/**
	 * Specifica il nome del file in cui vengono salvati i riferimenti
	 * ai progetti trovati
	 */
	private String dbPrjFoundFileName = "prjfound.db";
	/**
	 * Specifica il nome del file in cui vengono salvati i riferimenti
	 * ai progetti scaricati, ma non ancora condivisibili
	 */
	private String dbPrjDownFileName = "prjdown.db";
    /**
       Database dei progetti condivisi
     */
    private Hashtable dbPrjShare = null;
    /**
       Database dei progetti trovati in una precedente ricerca
     */
    private Vector dbPrjFound = null;
	/**
	 * Database dei progetti scaricati, ma non ancora condivisibili
	 */
	 private Hashtable dbPrjDown=null;
    /**
       Numero dei progetti condivisi
     */
    private long numPrjShare = 0;
	/**
	 * Decrizione delle tipologie di progetto
	 */
	private String[][] projectTypeDescription=null;
	/**
	 * Costruttore
	 */
	public ProjectManager(String pathName/*, String[][] projectTypeDescription*/) throws PolitellaException{
//		this.projectTypeDescription=projectTypeDescription;
		this.projectTypeDescription=ProjectType.prjTypeDescription;
		if(pathName!=null)
			this.defaultPathName=pathName;
		else
			this.defaultPathName=System.getProperty("user.dir").concat(System.getProperty("file.separator")).concat(this.defaultPathName);
		File path = new File(this.defaultPathName);
		if(!path.exists())
			//  la prima volta che avvio il programma devo creare le dir di sistema
			// nel caso in cui ho scelto io il percorso creando delle dir nuovo, questo istruzione
			//non viene eseguita anche se  la 1^ volta che avvio il software
			// per tale motivo 2 righe sotto c' un test analogo
			path.mkdir();
		// dir dei progetti condivisi
		path = new File(this.defaultPathName.concat(System.getProperty("file.separator")).concat("prjShare"));
		if(!path.exists())  {
		//  la prima volta che avvio il programma mancano
			path.mkdir();
		    // dir dei progetti scaricati
			path = new File(this.defaultPathName.concat(System.getProperty("file.separator")).concat("prjDown"));
			path.mkdir();

			//devo trasferire il DOM Politella dalla dir user a quella dei prj
			File domFileOrig = new File(System.getProperty("user.dir"),"politella.dtd");
			File domFileNew = new File(this.defaultPathName,"politella.dtd");
			domFileOrig.renameTo(domFileNew);
		}
		this.LoadDbPrj();
	}

	/**
	 * Carica le info relative al progetto memorizzate nel file XML
	 */
	private void LoadPrjInfo(Hashtable prjDb, boolean isShare)  throws PolitellaException{
		Enumeration prjList=null;
		if(isShare)
			prjList = this.dbPrjShare.elements();
		else
			prjList=this.dbPrjDown.elements();
		while(prjList.hasMoreElements())    {
			File prjFile=null;
			Vector prjBusModComponents=null;
			if(isShare) {
				ProjectShare curr=(ProjectShare)prjList.nextElement();
				curr.ResetProjectBusModelComponents();
				prjFile=curr.GetProjectFile();
				prjBusModComponents=curr.GetProjectBusModelComponents();
			}
			else    {
				ProjectDown curr = (ProjectDown)prjList.nextElement();
				curr.ResetProjectBusModelComponents();
				prjFile=curr.GetProjectFile();
				prjBusModComponents=curr.GetProjectBusModelComponents();
			}
			XMLProcessor.LoadProject(prjFile,prjBusModComponents);
		}
	}

	/**
	 * Inizialiazza i db dei progetti trovati e condivisi
	 */
	private void LoadDbPrj() throws PolitellaException{
		// carico il db dei progetti condivisi
		File dbPrjShareFile = new File(this.defaultPathName,this.dbPrjShareFileName);
		ObjectInputStream in=null;
		if(dbPrjShareFile.exists()){
			// ho gi creato o scaricato completamente dei progetti, devo caricarli nel db
			try	{
				in = new ObjectInputStream(new FileInputStream(dbPrjShareFile));

				this.dbPrjShare=(Hashtable)in.readObject();
				this.numPrjShare=this.dbPrjShare.size();

		    	// carico le info relative ai singoli progetti
				this.LoadPrjInfo(this.dbPrjShare,true);

			}catch(EOFException e){
				// ho letto tutto il file
			}catch(ClassNotFoundException e)	{
//                e.printStackTrace();
				throw (new PolitellaException(PolitellaException.NO_DB_LOAD,null));
			}catch(IOException e)	{
//                e.printStackTrace();
				 throw (new PolitellaException(PolitellaException.NO_DB_LOAD,null));
			}finally	{
				try	{
					in.close();
				}catch(IOException e){}
			}
		}
		else {
			// non ho nulla creo un db vuoto
			this.dbPrjShare= new Hashtable();
		}

		//carico il db dei progetti scaricati, ma non ancora condivisibili
		File dbPrjDownFile = new File(this.defaultPathName,this.dbPrjDownFileName);
		in=null;
		if(dbPrjDownFile.exists()){
			// ho gi carcato e scaricato dei progetti devo caricarli nel db
			try	{
				in = new ObjectInputStream(new FileInputStream(dbPrjDownFile));

				this.dbPrjDown=(Hashtable)in.readObject();

		    	// carico le info relative ai singoli progetti
				this.LoadPrjInfo(this.dbPrjDown,false);
			}catch(FileNotFoundException e)	{
				 //questa eccezione non capiter in quyanto viene prevenuta dall'if iniziale
			}catch(EOFException e){
				// ho letto tutto il file
			}catch(ClassNotFoundException e)	{
//                e.printStackTrace();
				throw (new PolitellaException(PolitellaException.NO_DB_LOAD,null));
			}catch(IOException e)	{
//                e.printStackTrace();
				 throw (new PolitellaException(PolitellaException.NO_DB_LOAD,null));
			}finally	{
				try	{
					in.close();
				}catch(IOException e){}
			}
		}
		else {
			// non ho nulla creo un db vuoto
			this.dbPrjDown= new Hashtable();
		}

		//carico il db dei progetti trovati
		File dbPrjFoundFile = new File(this.defaultPathName,this.dbPrjFoundFileName);
		in=null;
		if(dbPrjFoundFile.exists())	{
			// ho gi cercato e individuato dei progetti su altri peer devo caricarli nel db
			try	{
				in = new ObjectInputStream(new FileInputStream(dbPrjFoundFile));

				this.dbPrjFound=(Vector)in.readObject();
			}catch(FileNotFoundException e)	{
				 //questa eccezione non capiter in quyanto viene prevenuta dall'if iniziale
			}catch(EOFException e){
				// ho letto tutto il file
			}catch(ClassNotFoundException e)	{
				 throw (new PolitellaException(PolitellaException.NO_DB_LOAD,null));
			}catch(IOException e)	{
				 throw (new PolitellaException(PolitellaException.NO_DB_LOAD,null));
			}finally	{
				try	{
					in.close();
				}catch(IOException e){}
			}
		}
		else	{
			// non ho nulla creo un db vuoto
			this.dbPrjFound= new Vector();
		}
	}

	/**
	 * Salva in progetto specificato in un file XML
	 */
	public void SaveCurrPrj(Long prjID,boolean isShare) throws PolitellaException{
		if(isShare) {
		//voglio salvare un prj condiviso
			ProjectShare currPrj = (ProjectShare)this.dbPrjShare.get(prjID);

			String[] prjType= new String[2];
	    	// il +1 serve perch nell'elenco delle descrizione c' anche il campo ALL
			// necessario nelle ricerche
		    prjType[0]= new String(this.projectTypeDescription[0][(currPrj.GetProjectType())[0]+1]);
			prjType[1]= new String(this.projectTypeDescription[1][(currPrj.GetProjectType())[1]+1]);

			XMLProcessor.SaveProject(currPrj.GetProjectDescr(),prjType,currPrj.GetProjectFile(),currPrj.GetProjectBusModelComponents());
		}
		else    {
		//voglio salvare un prj scaricato
		// serve per tenere conto di eventuali risorse associate ai business component scaricate
			ProjectDown currPrj = (ProjectDown)this.dbPrjDown.get(prjID);

			String[] prjType= new String[2];
	    	// il +1 serve perch nell'elenco delle descrizione c' anche il campo ALL
			// necessario nelle ricerche
		    prjType[0]= new String(this.projectTypeDescription[0][(currPrj.GetProjectType())[0]+1]);
			prjType[1]= new String(this.projectTypeDescription[1][(currPrj.GetProjectType())[1]+1]);

			XMLProcessor.SaveProject(currPrj.GetProjectDescr(),prjType,currPrj.GetProjectFile(),currPrj.GetProjectBusModelComponents());
		}
	}

	/**
	 * Salva su file il contenuto dei db
	 */
	public void SaveDbPrj() throws PolitellaException{
		try	{
			//salvo il db dei progetti condivisi
			ObjectOutputStream out=null;
			File dbPrjShareFile = new File(this.defaultPathName,this.dbPrjShareFileName);
			out = new ObjectOutputStream(new FileOutputStream(dbPrjShareFile));

			out.writeObject(this.dbPrjShare);
			try	{
				out.flush();
				out.close();
			}catch(IOException e){}

			//salvo il db dei progetti scaricati, ma non condivisibili
			out=null;
			File dbPrjDownFile = new File(this.defaultPathName,this.dbPrjDownFileName);
			out = new ObjectOutputStream(new FileOutputStream(dbPrjDownFile));

			out.writeObject(this.dbPrjDown);
			try	{
				out.flush();
				out.close();
			}catch(IOException e){}

			//salvo il db dei progetti trovati
			out=null;
			File dbPrjFoundFile = new File(this.defaultPathName,this.dbPrjFoundFileName);
			out = new ObjectOutputStream(new FileOutputStream(dbPrjFoundFile));

			out.writeObject(this.dbPrjFound);
			try	{
				out.flush();
				out.close();
			}catch(IOException e){}
		}catch(IOException e)	{
//		    e.printStackTrace();
			throw new PolitellaException(PolitellaException.NO_DB_SAVE,null);
		}
	}

	/**
	 * Salva i db,  lanciato prima della chiusura del programma
	 */
	public void Exit() throws PolitellaException{
			this.SaveDbPrj();
	}
	/**
       Restituisce l'identificatore del file associato ad uno specifico progetto richiesto.
       Se l'iDPrj  0 restituisce solamente il PATH in cui sono salvati i progetti
       @roseuid 3AF8F4B2022E
     */
    public File GetProjectFile(long prjID, boolean isShare, boolean isDown) {
		if(prjID==0)    {
			//voglio uno dei percorsi di salvataggio
			if(isShare)
				// volgio sapere dove salvo i progetti condivisi
				return new File(this.defaultPathName.concat(System.getProperty("file.separator")).concat("prjShare"));
			if(isDown)
				// volgio sapere dove salvo i progetti scaricati, ma non ancora condivisibili
				return new File(this.defaultPathName.concat(System.getProperty("file.separator")).concat("prjDown"));
		    //qui arrivo solo se sia isShare che isDown sono FALSE
			//inm questo caso volgio il percorso alla base delle dir di salvataggio
			return new File(this.defaultPathName);
		}
		else	{
			if(isShare) {

				//devo cercare nel db dei prj condivisi
				ProjectShare prj = (ProjectShare)this.dbPrjShare.get(new Long(prjID));
	            if(prj!=null)
	                return prj.GetProjectFile();
	            else
	                return null;
			}
			if(isDown)  {
				//devo cercare nel db dei prj scaricati
				ProjectDown prj = (ProjectDown)this.dbPrjDown.get(new Long(prjID));
	            if(prj!=null)
	                return prj.GetProjectFile();
	            else
	                return null;
			}
		}
		// serve per completezza, ma non si arriva mai quaggi
		//l'unica possibilit sarebbe invocare il metodo con
		// xx.GetProjectFile(xxx,false,false);
		return null;
    }
    /**
       Aggiunge un nuovo progetto ala db dei progetti trovati.
       @roseuid 3AF964AC026B
     */
	synchronized public void AddPrjToDbPrjFound(short[] prjType, String prjDescr, long prjID, String prjPeerLocation) {
		this.dbPrjFound.addElement(new ProjectFound(prjID, prjType, prjDescr, prjPeerLocation));
    }
    /**
       Aggiunge un nuovo progetto ala db dei progetti scaricati
       @roseuid 3AF964AC026B
     */
	synchronized private void AddPrjToDbPrjDown(ProjectDown newPrjDown) {
		// la chiave dell'hashtable  lo uniqueID del ProjectDown, ma questo coincide
		//con il nome del file - l'estensione, siccome il file  stato crato controllando di
		//non avere duplicati sono sicuro che anche la chiave  univoca
		this.dbPrjDown.put(new Long(newPrjDown.GetProjectID()),newPrjDown);
    }
	/**
       Aggiunge un nuovo progetto al db dei progetti scaricati.
       Il progetto  stato scaricato, quindi utilizza l'ID e l'indirizzo
		IP da dove  stato scaricato per recuperare le informazioni relative
		al tipo e alla descrizione nel db dei progetti trovati nelle ricerche
		precedenti
     */
    public void AddPrjToDbPrjDown(File prjFileTmp, long prjFoundID, String prjFoundPeerLocation) throws PolitellaException{
		ProjectFound prj = this.GetInfoOnePrjFound(prjFoundID,prjFoundPeerLocation);
		if(prj!=null)   {
			File pathBase=this.GetProjectFile(0,false,true);
			//ricavo la dir in cui salvare il prj dal suo nomefile
			String fileName = (new Long(prjFoundID)).toString();
		    File prjFilePath = new File(pathBase,fileName);
			while(prjFilePath.exists()) {
				prjFilePath=null; fileName=null;
			    fileName = (new Long((new java.util.Random()).nextLong())).toString();
				prjFilePath = new File(pathBase,fileName);
			}
			prjFilePath.mkdir();
			File prjFile = new File(prjFilePath,fileName + ".xml");
			prjFileTmp.renameTo(prjFile);

			ProjectDown newPrj = new ProjectDown(prj.GetProjectType(),prj.GetProjectDescr(),prjFoundPeerLocation,prjFile,prj.GetProjectID());

			// carico le info salvate nel file XML associato al prj
			newPrj.ResetProjectBusModelComponents();
			XMLProcessor.LoadProject(prjFile,newPrj.GetProjectBusModelComponents());
			// devo aggiornare le locazioni delle varie risorse associate ai business component
	    	//tenendo presente dal peer su cui si trova
			newPrj.ResetPrjBusModCompElementsLocation();
			this.AddPrjToDbPrjDown(newPrj);

			this.SaveCurrPrj(new Long(newPrj.GetProjectID()),false);

		}
    }
    /**
       Aggiunge un nuovo progetto ala db dei progetti condivisi.
       @roseuid 3AF916BB034F
     */
    synchronized private void AddPrjToDbPrjShare(ProjectShare newPrjShare) {
		// la chiave dell'hashtable  lo uniqueID del ProjectShare, ma questo coincide
		//con il nome del file - l'estensione, siccome il file  stato crato controllando di
		//non avere duplicati sono sicuro che anche la chiave  univoca

		this.dbPrjShare.put(new Long(newPrjShare.GetProjectID()),newPrjShare);
		this.numPrjShare++;
    }
    /**
       Aggiunge un nuovo progetto al db dei progetti condivisi.
		Il progetto  stato creato localmente.
     */
    public void AddPrjToDbPrjShare(short[] prjType, String prjDescr, File prjFile) {
		ProjectShare newPrj = new ProjectShare(prjType,prjDescr,prjFile);
		newPrj.ResetProjectBusModelComponents();
		this.AddPrjToDbPrjShare(newPrj);
	}
    /**
       Restituisce un ProjectFound dati il suo prjID e peerLocation oppure null
       @roseuid 3AFA52010106
     */
    ProjectFound GetInfoOnePrjFound(long prjID, String prjPeerLocation) {
		Enumeration list = this.dbPrjFound.elements();
		ProjectFound prjFound=null;
		while(list.hasMoreElements())	{
			prjFound=(ProjectFound)list.nextElement();
			if((prjFound.GetProjectID()==prjID)&&
			   (prjFound.GetProjectPeerLocation().compareTo(prjPeerLocation)==0))	{
				//lo posso togliere dal db perch questo metodo  chiamato al momento di scaricare il prj
				this.dbPrjFound.removeElement(prjFound);
				//esco dal while
				break;
			}
			else
				prjFound=null;
		}
		return prjFound;
    }
    /**
       Restituisce il valore del campo numPrjShare
       @roseuid 3AFA3D060365
     */
    public long GetNumPrjShare() {
		return this.numPrjShare;
    }
    /**
       Elimina uno progetto da db dei progetti condivisi
       @roseuid 3B03B9240080
     */
    public void DelPrjShare(long prjID) {
		ProjectShare tmp=(ProjectShare)this.dbPrjShare.get(new Long(prjID));

		File prjFile_Dir = tmp.GetProjectFile();
		prjFile_Dir.delete();
		// devo eliminare eventuali altri file nella dir dove si trova il prj
		prjFile_Dir = new File(this.GetProjectFile(0,true,false),new String(""+prjID+""));
		//ctrl se devo svuotare la dir da altro contenuto
		this.EmptyDir(prjFile_Dir);
		//la dir  sicuramente vuota, posso eliminare anch'essa
		prjFile_Dir.delete();

		this.dbPrjShare.remove(new Long(prjID));
		this.numPrjShare--;
    }
    /**
       Elimina uno progetto da db dei progetti scaricati
       @roseuid 3B03B9240080
     */
    public void DelPrjDown(long prjID) {
		ProjectDown tmp=(ProjectDown)this.dbPrjDown.get(new Long(prjID));

		File prjFile_Dir = tmp.GetProjectFile();
		prjFile_Dir.delete();

		// devo eliminare eventuali altri file nella dir dove si trovava il prj
		prjFile_Dir = new File(this.GetProjectFile(0,false,true),new String(""+prjID+""));

		//ctrl se devo svuotare la dir da altro contenuto
		this.EmptyDir(prjFile_Dir);
		//la dir  sicuramente vuota, posso eliminare anch'essa
		prjFile_Dir.delete();

		this.dbPrjDown.remove(new Long(prjID));
    }

	/**
	 * Svuota il contenuto di una dir specificata
	 */
	private void EmptyDir(File curr) {
		File[] list = curr.listFiles();
		for(int i=0;i<list.length;i++)  {
			if(list[i].isDirectory())
				this.EmptyDir(list[i]);
			list[i].delete();
		}
	}

	/**
	 * Sposta il contenuto di una dir specificata in un altra dir specificata
	 */
	 private void MoveDir(File currFile, File newFile)  {
		File[] list = currFile.listFiles();
		for(int i=0;i<list.length;i++)  {
			list[i].renameTo(new File(newFile,list[i].getName()));
		}
		currFile.delete();
	 }

    /**
       Elimina un riferimento ad progetto trovato da db dei progetti trovati
       @roseuid 3B03B9400171
     */
    public void DelPrfFound(long prjID, String prjPeerLocation) {
		Enumeration list = this.dbPrjFound.elements();
		ProjectFound prjFound=null;
		while(list.hasMoreElements())	{
			prjFound=(ProjectFound)list.nextElement();
			if((prjFound.GetProjectID()==prjID)&&
			   (prjFound.GetProjectPeerLocation().compareTo(prjPeerLocation)==0))	{
				//esco dal while
				this.dbPrjFound.removeElement(prjFound);
				break;
			}
		}
    }

    /**
       Svuota il db dei progetti condivisi
       @roseuid 3B03B9FC0294
     */
    public void EmptyDbPrjShare() {
		this.dbPrjShare.clear();
        this.numPrjShare=0;

		this.EmptyDir(this.GetProjectFile(0,true,false));
    }

    /**
       Svuota il db dei progetti scaricati
       @roseuid 3B03B9FC0294
     */
    public void EmptyDbPrjDown() {
		this.dbPrjDown.clear();

		this.EmptyDir(this.GetProjectFile(0,false,true));
  }

    /**
       Svuota il db dei progetti trovati
       @roseuid 3B03BA29037F
     */
    public void EmptyDbPrjFound() {
		this.dbPrjFound.removeAllElements();
    }

	/**
	 * Restituisce le informazioni relative ai progetti condivisi
	 */
	 public Vector GetInfoAllPrjShare()   {
		Vector allPrjInfo = new Vector();

		ProjectShare tmp=null;

		Enumeration list= this.dbPrjShare.elements();
		while(list.hasMoreElements())   {
			Vector onePrjInfo = new Vector();
			tmp=(ProjectShare) list.nextElement();

		    short[] indexTypes=tmp.GetProjectType();
			// il +1 serve perch nell'elenco delle descrizione c' anche il campo ALL
			// necessario nelle ricerche
		    onePrjInfo.addElement(new String(this.projectTypeDescription[0][indexTypes[0]+1]));
		    onePrjInfo.addElement(new String(this.projectTypeDescription[1][indexTypes[1]+1]));
			onePrjInfo.addElement(tmp.GetProjectDescr());
			onePrjInfo.addElement(tmp.GetProjectFile().getName());
			onePrjInfo.addElement(new Long(tmp.GetProjectID()));
		    onePrjInfo.addElement((Vector)tmp.GetProjectBusModelComponents());
			allPrjInfo.addElement(onePrjInfo);

		}
		return allPrjInfo;
	 }

	/**
	 * Restituisce le informazioni relative ad un progetto condiviso
	 */
	 public Vector GetInfoOnePrjShare(Long prjID) {
		ProjectShare tmp=(ProjectShare)this.dbPrjShare.get(prjID);

		Vector onePrjInfo = null;
		if(tmp!=null)   {
			onePrjInfo = new Vector();
			short[] indexTypes=tmp.GetProjectType();
			// il +1 serve perch nell'elenco delle descrizione c' anche il campo ALL
			// necessario nelle ricerche
		    onePrjInfo.addElement(new String(this.projectTypeDescription[0][indexTypes[0]+1]));
		    onePrjInfo.addElement(new String(this.projectTypeDescription[1][indexTypes[1]+1]));
			onePrjInfo.addElement(tmp.GetProjectDescr());
			onePrjInfo.addElement(tmp.GetProjectFile().getName());
			onePrjInfo.addElement(new Long(tmp.GetProjectID()));
		    onePrjInfo.addElement((Vector)tmp.GetProjectBusModelComponents());
		}
		return onePrjInfo;
	 }

	/**
	 * Restituisce le informazioni relative ad un progetto scaricato
	 */
	 public Vector GetInfoOnePrjDown(Long prjID) {
		ProjectDown tmp=(ProjectDown)this.dbPrjDown.get(prjID);

		Vector onePrjInfo = null;
		if(tmp!=null)   {
			onePrjInfo = new Vector();
			short[] indexTypes=tmp.GetProjectType();
			// il +1 serve perch nell'elenco delle descrizione c' anche il campo ALL
			// necessario nelle ricerche
		    onePrjInfo.addElement(new String(this.projectTypeDescription[0][indexTypes[0]+1]));
		    onePrjInfo.addElement(new String(this.projectTypeDescription[1][indexTypes[1]+1]));
			onePrjInfo.addElement(tmp.GetProjectDescr());
			onePrjInfo.addElement(tmp.GetProjectFile().getName());
			onePrjInfo.addElement(new Long(tmp.GetProjectID()));
		    onePrjInfo.addElement((Vector)tmp.GetProjectBusModelComponents());
			onePrjInfo.addElement(tmp.GetProjectPeerLocation());
		}
		return onePrjInfo;
	 }

	/**
	 * Restituisce le informazioni relative ai progetti scaricati
	 */
	 public Vector GetInfoAllPrjDown()   {
		Vector allPrjInfo = new Vector();

		ProjectDown tmp=null;

		Enumeration list= this.dbPrjDown.elements();
		while(list.hasMoreElements())   {
			Vector onePrjInfo = new Vector();
			tmp=(ProjectDown) list.nextElement();

		    short[] indexTypes=tmp.GetProjectType();
			// il +1 serve perch nell'elenco delle descrizione c' anche il campo ALL
			// necessario nelle ricerche
		    onePrjInfo.addElement(new String(this.projectTypeDescription[0][indexTypes[0]+1]));
		    onePrjInfo.addElement(new String(this.projectTypeDescription[1][indexTypes[1]+1]));
			onePrjInfo.addElement(tmp.GetProjectDescr());
			onePrjInfo.addElement(tmp.GetProjectFile().getName());
			onePrjInfo.addElement(new Long(tmp.GetProjectID()));
		    onePrjInfo.addElement((Vector)tmp.GetProjectBusModelComponents());
			onePrjInfo.addElement(tmp.GetProjectPeerLocation());
			allPrjInfo.addElement(onePrjInfo);
		}
		return allPrjInfo;
	 }

	/**
	 * Restituisce le informazioni relative ai progetti trovati
	 */
	 public Vector GetInfoAllPrjFound()   {
		Vector allPrjInfo = new Vector();

		ProjectFound tmp=null;

		Enumeration list= this.dbPrjFound.elements();
		while(list.hasMoreElements())   {
			Vector onePrjInfo = new Vector();
			tmp=(ProjectFound) list.nextElement();

		    short[] indexTypes=tmp.GetProjectType();
			// il +1 serve perch nell'elenco delle descrizione c' anche il campo ALL
			// necessario nelle ricerche
			onePrjInfo.addElement(new String(this.projectTypeDescription[0][indexTypes[0]+1]));
			onePrjInfo.addElement(new String(this.projectTypeDescription[1][indexTypes[1]+1]));
			onePrjInfo.addElement(tmp.GetProjectDescr());
			onePrjInfo.addElement(tmp.GetProjectPeerLocation());
			onePrjInfo.addElement(new Long(tmp.GetProjectID()));
			allPrjInfo.addElement(onePrjInfo);
		}
		return allPrjInfo;
	 }
	 /**
	  * Ricerca localmente uno pi progetti in base ai parametri passti in ingresso
	  */
	  public Vector SearchProjectInLocal(short[] searchType, String searchKey)   {
		// devo distingure i vari tipo di ricerca
		Vector result=new Vector();
		ProjectShare tmp=null;
		Enumeration list= this.dbPrjShare.elements();
		// serve per indicare che il prj considerato corrisponde ai requisiti di
		// ricerca specificati
		boolean commercialModelOK;
		boolean architecturalModelOK;
		boolean businessModelOK;
		boolean descriptionOK;
		while(list.hasMoreElements())   {
		    tmp=(ProjectShare)list.nextElement();
		    short[] indexTypesTmp=tmp.GetProjectType();

			// ctrl il Commercial Model
			switch(searchType[0])   {
				case ProjectType.COMMERCIAL_MODEL_ANY:  {
				    commercialModelOK=true;
					break;
				}
				default:    {
					if(indexTypesTmp[0]==searchType[0])
						commercialModelOK=true;
					else
						commercialModelOK=false;
					break;
				}
			}
			// ctrl l'Architectural Model
			switch(searchType[1])   {
				case ProjectType.ARCHITECTURAL_MODEL_ANY:   {
					architecturalModelOK=true;
					break;
				}
				default:   {
					if(indexTypesTmp[1]==searchType[1])
						architecturalModelOK=true;
					else
						architecturalModelOK=false;
					break;
				}
			}
			// ctrl il Business Model
			switch(searchType[2])   {
				case ProjectType.BUSINESS_MODEL_COMPONENT_ANY:  {
					businessModelOK=true;
					break;
				}
				default:   {
					if(((BusinessModelComponent)tmp.GetProjectBusModelComponents().get(searchType[2])).IsActive())
						businessModelOK=true;
					else
						businessModelOK=false;
					break;
				}
			}
			//ctrl la description
			if(searchKey==null)
				descriptionOK=true;
			else    {
				if(tmp.GetProjectDescr().indexOf(searchKey)!=-1)
					descriptionOK=true;
				else
					descriptionOK=false;
			}

			if(commercialModelOK && architecturalModelOK &&
			    businessModelOK && descriptionOK)    {
		    // il prj considerato corrispodneva ai requisiti di ricerca
			// specificati, lo inserisco nel Vector da inviare
				Vector item=new Vector();
				Short[] indexTypes = new Short[indexTypesTmp.length];
				for(int i=0; i<indexTypesTmp.length; i++)
					indexTypes[i]=new Short(indexTypesTmp[i]);
				item.addElement(indexTypes);
				item.addElement(tmp.GetProjectDescr());

				item.addElement(new Boolean(tmp.WasDown()));
				if(tmp.WasDown())   {
					//il prj non  stato creato in locale, ma scaricato
					item.addElement(new Long(tmp.GetPrjOrigID()));
					item.addElement(tmp.GetPrjOrigPeerLocation());
				}
				else    {
					//il prj  stato creato localmente
					item.addElement(new Long(tmp.GetProjectID()));
				}
				result.addElement(item);
			}
		}
		return result;
	  }

	/**
	 * Aggiorna le informazioni relative alla locazione di una risorsa associata ad un business component quando:
	 * - scarico una risorsa di un prj down
	 * - quando trasformo un prj down in uno share
	 */
//	 public synchronized void UpdateOneBusModCompElementLocation(File busModCompElementFile,boolean isDown) {
	 public synchronized void UpdateOneBusModCompElementLocation(File[] busModCompElementFilesList,boolean isDown) {
		//l'array  necessario per aggiornare le risorse nel caso di trasf da down a share
		//nell'altro caso  un array con elementi tutti null tranne uno, quello il cui persocrso deve aggioranre
		//un'attributo nel vector busmodcomp di un prj down

		//se trasformo un prj down in uno share i file ricevuti hanno tutti lo stesso percorso

		String[] arrayOfInfo = new String[busModCompElementFilesList.length];

		String busModCompElemType =null;
		short busModCompIDLink = -1;

		BusinessModelComponent currBusModComp = null;
		Vector busModComponents = null;

		for(int y=0;y<busModCompElementFilesList.length;y++)    {
			if(busModCompElementFilesList[y]==null)
				//quin dentro si entra solo quando si processano risorse recuperate dalla comunit
				arrayOfInfo[y]=null;
			else    {
				File busModCompElementFile = busModCompElementFilesList[y];

				busModCompElemType =	busModCompElementFile.getParentFile().getName();

				short busModCompID = -1;
				Long prjID =null;
				int numRandom = 0;

				//recupero gli ID relativi ai bus comp e bus rela interessati da questa risorsa
				if(busModCompElementFile.getAbsolutePath().indexOf("RESOURCES")!=-1)    {
					//il file fa parte di uno dei gruppi di risorse associate ad 1 bus comp
					if(isDown)  {
		    			busModCompIDLink =(new Short(busModCompElementFile.getParentFile().getParentFile().getName().substring(2))).shortValue();
		    			//con 3 GetParentFile beccherei la dir RESOURCES, non mi interessa
					    busModCompID = (new Short(busModCompElementFile.getParentFile().getParentFile().getParentFile().getParentFile().getName().substring(2))).shortValue();
						numRandom = new Integer(busModCompElementFile.getParentFile().getParentFile().getParentFile().getParentFile().getParentFile().getParentFile().getName()).intValue();
					}
					else    {
						//sto trasformando un prjDown in un prjShare
						String busModCompIDLinkStr = busModCompElementFile.getParentFile().getParentFile().getName();
//      				System.out.println("Link:"+ busModCompIDLinkStr);
						for(int i=0;i<ProjectType.prjTypeDescription[2].length;i++) {
							if(ProjectType.prjTypeDescription[2][i].compareTo(busModCompIDLinkStr)==0)  {
								//beccato
								if(i<=ProjectType.numBusinessRole)
									//il -1 per Any Business Role
									busModCompIDLink=(short)(i-1);
								else
									//il -2 per Any Business Role e Any Business Relationship
									busModCompIDLink=(short)(i-2);
								break;
							}
						}
//      				System.out.println("Link ID: "+busModCompIDLink);

						String busModCompIDStr = busModCompElementFile.getParentFile().getParentFile().getParentFile().getParentFile().getName();
//      				System.out.println("Comp:"+ busModCompIDStr);
						for(int i=0;i<ProjectType.prjTypeDescription[2].length;i++) {
							if(ProjectType.prjTypeDescription[2][i].compareTo(busModCompIDStr)==0)  {
								//beccato
								if(i<=ProjectType.numBusinessRole)
									//il -1 per Any Business Role
									busModCompID=(short)(i-1);
								else
									//il -2 per Any Business Role e Any Business Relationship
									busModCompID=(short)(i-2);
								break;
							}
						}
//      				System.out.println("Comp ID: "+busModCompID);
					}
					prjID =new Long(busModCompElementFile.getParentFile().getParentFile().getParentFile().getParentFile().getParentFile().getName());
				}
				else    {
					//il file in questione fa parte delle risorse associate all'intero bus comp
				    if(isDown)  {
						//la risorsa fa parte di un prjDown
						busModCompID =(new Short(busModCompElementFile.getParentFile().getParentFile().getName().substring(2))).shortValue();
						numRandom = new Integer(busModCompElementFile.getParentFile().getParentFile().getParentFile().getParentFile().getName()).intValue();
				    }
					else    {
						//sto trasformando un prjDown in un prjShare
						String busModCompIDStr = busModCompElementFile.getParentFile().getParentFile().getName();
//      				System.out.println("Comp:"+ busModCompIDStr);
						for(int i=0;i<ProjectType.prjTypeDescription[2].length;i++) {
							if(ProjectType.prjTypeDescription[2][i].compareTo(busModCompIDStr)==0)  {
								//beccato
								if(i<=ProjectType.numBusinessRole)
									//il -1 per Any Business Role
									busModCompID=(short)(i-1);
								else
									//il -2 per Any Business Role e Any Business Relationship
									busModCompID=(short)(i-2);
								break;
							}
						}
//      				System.out.println("Comp ID: "+busModCompID);
					}
				    prjID =new Long(busModCompElementFile.getParentFile().getParentFile().getParentFile().getName());
				}

				if(isDown)  {
					//ho finito di scarica una risorsa associata ad un progetto down

					//il file  arrivato nella forma
					//  ..\xxxx\[numrandom]\IDy\RESOURCES\IDz\....
					// oppure
					//  ..\xxxx\IDy\...
					// lo devo trasformare in
					// ..\xxxx\nome_componente_y\RESOURCE\nome_componente_z\...
					//oppure
					// ..\xxxx\nome_componente_y\...

					//recupero nome_componente_y
					short TMPbusModCompID=-1;
					if(busModCompID<ProjectType.numBusinessRole)
						//  un bus role, + 1 per saltare Any Business Role
						TMPbusModCompID=(short)(busModCompID+1);
					else
						//  una bus rela, + 2 per saltare Any Business Role e Any Business Relationship
						TMPbusModCompID=(short)(busModCompID+2);

					File finalPathBusModCompElementFile = null;
					if(busModCompElementFile.getAbsolutePath().indexOf("RESOURCES")!=-1)    {
						//il file fa parte di uno dei gruppi di risorse associate ad 1 bus comp

						//devo ancora recuperare nome_componente_z
						short TMPbusModCompIDLink=-1;
						if(busModCompIDLink<ProjectType.numBusinessRole)
							//  un bus role, + 1 per saltare Any Business Role
							TMPbusModCompIDLink=(short)(busModCompIDLink+1);
						else
							//  una bus rela, + 2 per saltare Any Business Role e Any Business Relationship
							TMPbusModCompIDLink=(short)(busModCompIDLink+2);

					    finalPathBusModCompElementFile = new File(this.GetProjectFile(0,false,true)+System.getProperty("file.separator")+
																    prjID+System.getProperty("file.separator")+
																	ProjectType.prjTypeDescription[2][TMPbusModCompID]+System.getProperty("file.separator")+
																	"RESOURCES"+System.getProperty("file.separator")+
																	ProjectType.prjTypeDescription[2][TMPbusModCompIDLink]+System.getProperty("file.separator")+
																	busModCompElemType);
					}
					else    {
			    		//il file in questione fa parte delle risorse associate all'intero bus comp
					    finalPathBusModCompElementFile = new File(this.GetProjectFile(0,false,true)+System.getProperty("file.separator")+
																    prjID+System.getProperty("file.separator")+
																	ProjectType.prjTypeDescription[2][TMPbusModCompID]+System.getProperty("file.separator")+
																	busModCompElemType);
					}

					finalPathBusModCompElementFile.mkdirs();

				    //sposto ..\ID..\..
					//in
					// ..\nome_componente..\..
					File finalBusModCompElementFile = new File(finalPathBusModCompElementFile,busModCompElementFile.getName());
					busModCompElementFile.renameTo(finalBusModCompElementFile);
					busModCompElementFile=finalBusModCompElementFile;

					//cancello le dir dove si trovava la risorsa appena spostata
					File tmp = new File(this.GetProjectFile(0,false,true)+System.getProperty("file.separator")+
										numRandom);
					this.EmptyDir(tmp);
					tmp.delete();

					ProjectDown currPrj = (ProjectDown) this.dbPrjDown.get(prjID);
					busModComponents = currPrj.GetProjectBusModelComponents();
					currBusModComp = (BusinessModelComponent)busModComponents.get(busModCompID/*-1*/);
				}
				else    {
				  	//sta trasformando un progetto down in uno share, devo aggioranre le locazioni
					ProjectShare currPrj = (ProjectShare) this.dbPrjShare.get(prjID);
					busModComponents = currPrj.GetProjectBusModelComponents();
					currBusModComp = (BusinessModelComponent)busModComponents.get(busModCompID/*-1*/);
				}

				arrayOfInfo[y]=busModCompElementFile.getAbsolutePath();
			}//chiudo l'if sull'array di file ricevuto
		}//chiudo il for


	    //questo componente pu far parte o del bus comp intero, oppure di uno dei gruppi di risorse associate
		if(busModCompElemType.compareTo("SOURCE")==0)   {
			if(busModCompIDLink==-1)
			//questi sono i sorgenti dell'intero bus comp
				currBusModComp.SetBusModCompSourceLoc(arrayOfInfo);
//				currBusModComp.SetBusModCompSourceLoc(busModCompElementFile.getAbsolutePath());
			else
			//questa sono i sorgenti associati ad uno dei gruppi di risorse del bus comp
				currBusModComp.SetBusModComp_1_ResSourceLoc(arrayOfInfo,busModCompIDLink);
//				currBusModComp.SetBusModComp_1_ResSourceLoc(busModCompElementFile.getAbsolutePath(),busModCompIDLink);
		}

	    //questo componente pu far parte o del bus comp intero, oppure di uno dei gruppi di risorse associate
		if(busModCompElemType.compareTo("BINARY")==0)   {
			if(busModCompIDLink==-1)
			//questi sono i binari dell'intero bus comp
				currBusModComp.SetBusModCompBinaryLoc(arrayOfInfo);
//				currBusModComp.SetBusModCompBinaryLoc(busModCompElementFile.getAbsolutePath());
			else
			//questa sono i binari associati ad uno dei gruppi di risorse del bus comp
				currBusModComp.SetBusModComp_1_ResBinaryLoc(arrayOfInfo,busModCompIDLink);
//				currBusModComp.SetBusModComp_1_ResBinaryLoc(busModCompElementFile.getAbsolutePath(),busModCompIDLink);
		}

	    //questo componente pu far parte o del bus comp intero, oppure di uno dei gruppi di risorse associate
		if(busModCompElemType.compareTo("DOCUMENTATION")==0)    {
			if(busModCompIDLink==-1)
			//questa  la documentazione dell'intero bus comp
				currBusModComp.SetBusModCompDocLoc(arrayOfInfo);
//				currBusModComp.SetBusModCompDocLoc(busModCompElementFile.getAbsolutePath());
			else
			//questa  la documentazione associata ad uno dei gruppi di risorse del bus comp
				currBusModComp.SetBusModComp_1_ResDocLoc(arrayOfInfo,busModCompIDLink);
//				currBusModComp.SetBusModComp_1_ResDocLoc(busModCompElementFile.getAbsolutePath(),busModCompIDLink);
		}

		if(isDown)
			currBusModComp.CheckBusModCompLinksAllResourcesLocal(busModComponents);
	 }
	 /**
	  * Treasforma un progetto di tipo down in uno di share, trasferendo le info da un db all'altro e tutte le risorse da dir all'altra.
	  * Questo accade quando di un prj down ho recuperato tutte le risorse
	  */
	  public void MovePrjDownToPrjShare(Long prjID) throws PolitellaException{
		ProjectDown currPrj = (ProjectDown) this.dbPrjDown.get(prjID);

		ProjectShare newPrj = new ProjectShare(currPrj,this.GetProjectFile(0,true,false));
		//quando ritorna il costruttore sono sicuro che il prjFile associato non esiste ancora

		//creo la root per il prj
		newPrj.GetProjectFile().getParentFile().mkdir();

		//sposto il file del prj
		currPrj.GetProjectFile().renameTo(newPrj.GetProjectFile());
		//sposto le risorse associate al prj
		this.MoveDir(currPrj.GetProjectFile().getParentFile(),newPrj.GetProjectFile().getParentFile());
		//elimino il prj down
		this.dbPrjDown.remove(prjID);
		//aggiungo un prj share
		this.AddPrjToDbPrjShare(newPrj);

		//devo aggiornare tutte le locazioni delle risorse associate al prj
		File[] list = newPrj.GetProjectFile().getParentFile().listFiles();
		for(int i=0;i<list.length;i++)  {
			if(list[i].isDirectory())
				this.UpdateAllPrjResourcesLocation(list[i]);
		}

		//aggiorno con le nuove locazioni il file xml associato al nuovo prjShare
		this.SaveCurrPrj(new Long(newPrj.GetProjectID()),true);
	  }

	  /**
	   * Aggiorna tutte le locazioni delle risorse associate al prj
	   * chiamato nella trasformazione da down a share
	   */
	  private void UpdateAllPrjResourcesLocation(File resourceLocation)  {
		File[] list = resourceLocation.listFiles();
		//recupero i riferimenti alle dir in cui sono salvati i componenti
		for(int i=0;i<list.length;i++)  {
			if(list[i].isDirectory())
				//devo entrare dentro la dir e ctrl il contenuto
				this.UpdateAllPrjResourcesLocation(list[i]);
			else    {
				//per come vengono create le dir di un prj down, in una dir ci sono solo o file o altre dir.
				//l'unica mista  la home dove si trova il file xml
				this.UpdateOneBusModCompElementLocation(list,false);
				break;

				// una risorsa associata ad un determinato bus comp, ne aggiorno la locazione
//				this.UpdateOneBusModCompElementLocation(list[i],false);
			}
		}
	  }
}

