/*
*                            Copyright (c) 2001
*                        <B>Politecnico di Torino</B>
*                            All Rights Reserved
* <p>
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that: (1) source code distributions
* retain the above copyright notice and this paragraph in its entirety,(2)
* distributions including binary code include the above copyright notice and
* this paragraph in its entirety in the documentation or other materials
* provided with the distribution, and (3) all advertising materials mentioning
* features or use of this software display the following acknowledgement:
* ``This product includes software developed by Politecnico
* di Torino, and its contributors.'' (4) Neither the name of
* the University nor the names of its contributors may be used to endorse
* or promote products derived from this software without specific prior
* written permission.
* THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
* This product includes code licensed from RSA Data Security (Java Secure Socket Extension)
*
* Sun grants you ("Licensee") a non-exclusive, royalty free, license to use, and redistribute
* this software graphics artwork, as individual graphics or as a collection, as part of
* software code or programs that you develop, provided that
* i) this copyright notice and license accompany the software graphics artwork; and
* ii) you do not utilize the software graphics artwork in a manner which is disparaging to Sun.
* Unless enforcement is prohibited by applicable law, you may not modify the graphics,
* and must use them true to color and unmodified in every way.
*
*<p>
*/
package Project;

import Utility.BusinessModelComponent;
import Utility.ProjectType;

import java.io.File;
import java.io.Serializable;

import java.util.Vector;

/**
   Elemento del db contenente tutti i propri progetti condivisi.
 */
final class ProjectShare implements Serializable {
    /**
     * Contiene tutti i componenti del Business Model
     */
     private transient Vector projectBusModelComponents=null;
    /**
       Identificatore del file su cui  salvato il progetto.
       Questo identificatore  dato nella forma PATH + FILENAME
     */
    private File projectFile = null;
    /**
       Identificato del progetto, univoco solamente all'interno del peer su cui risiede
     */
    private long projectID;
    /**
       Tipologia del progetto
     */
    private short[] projectType;
    /**
       Descrizione completa del progetto.
     */
    private String projectDescr = null;
	/**
	 * Indica se il prj  stato creato localmente oppure  stato
	 * scaricato da qualche peer
	 */
	private boolean wasDown = false;
	/**
	 * Se il prj  stato down, indica il peer sorgente
	 */
	private String prjOrigPeerLocation = null;
	/**
	 * Se il prj  stato down, indica l'ID che aveva sul peer sorgente
	 */
	private long prjOrigID;

    /**
       Costruttore
       @roseuid 3AF80B220202
     */
    public ProjectShare(short[] projectType, String projectDescr, File projectFile) {
		this.projectID=new Long(projectFile.getName().substring(0,projectFile.getName().length()-4)).longValue();
		this.projectType=projectType;
		this.projectDescr=projectDescr;
		this.projectFile=projectFile;
		this.projectBusModelComponents=new Vector();
    }

	/**
	 * Costruttore (usato solo nel passaggio da prjDown a prjShare)
	 */
	 ProjectShare(ProjectDown currPrj,File rootSavingPath)  {
		this.projectDescr=currPrj.GetProjectDescr();
		this.projectBusModelComponents=(Vector)currPrj.GetProjectBusModelComponents().clone();
		this.projectType=currPrj.GetProjectType();

		this.projectID=currPrj.GetProjectID();

		this.wasDown=true;
		this.prjOrigPeerLocation = currPrj.GetProjectPeerLocation();
		this.prjOrigID = currPrj.GetPrjOrigID();

		this.projectFile=new File(new File(rootSavingPath,""+this.projectID+""),""+this.projectID+".xml");

		while(this.projectFile.exists())    {
			this.SetProjectID();
			this.projectFile=new File(new File(rootSavingPath,""+this.projectID+""),""+this.projectID+".xml");
		}
	 }

	/**
       Restituisce il valore del campo projectFile
       @roseuid 3AF80AD6016D
     */
    public File GetProjectFile() {
		return this.projectFile;
    }

    /**
       Restituisce il valore del campo projectType
       @roseuid 3B01098D038B
     */
    public short[] GetProjectType() {
		return this.projectType;
    }

	/**
	 * Restituisce il valore dell'attributo wasDown
	 */
	public boolean WasDown()    {
		return this.wasDown;
	}
	/**
	 * Restituisce il valore dell'attributo prjOrigPeerLocation
	 */
	public String GetPrjOrigPeerLocation()  {
		return this.prjOrigPeerLocation;
	}
	/**
	 * Restituisce il valore dell'attributo prjOrigID
	 */
	public long GetPrjOrigID()  {
		return this.prjOrigID;
	}

    /**
       Restituisce il valore del campo projectID
       @roseuid 3B01098D039F
     */
    public long GetProjectID() {
		return this.projectID;
    }

    /**
       Restituisce il valore del campo  projectDescr
       @roseuid 3B01098D03A9
     */
    public String GetProjectDescr() {
		return this.projectDescr;
    }

	/**
	 * Restituisce il valore dell'attributo  businessModelComponents
	 */
	 public Vector GetProjectBusModelComponents() {
		return this.projectBusModelComponents;
	 }

	/**
	 * Imposta il valore del projectID
	 */
	protected void SetProjectID() {
		this.projectID=(new java.util.Random()).nextLong();
	}

	/**
	 * Resetta il valore dell'attributo projectBusModelComponents
	 * Viene chiamato quando carico il db dei prj condivisi
	 */
	 protected void ResetProjectBusModelComponents()    {
		this.projectBusModelComponents=new Vector();
		//i business component sono numerati da 0 e nel vettore sono memorizzati da 0
		for(short busModCompID=0;busModCompID<ProjectType.numBusinessRole+ProjectType.numBusinessRelationship;busModCompID++)   {
			this.projectBusModelComponents.add(new BusinessModelComponent(busModCompID));
		}
	 }

}
