/*
*                            Copyright (c) 2001
*                        <B>Politecnico di Torino</B>
*                            All Rights Reserved
* <p>
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that: (1) source code distributions
* retain the above copyright notice and this paragraph in its entirety,(2)
* distributions including binary code include the above copyright notice and
* this paragraph in its entirety in the documentation or other materials
* provided with the distribution, and (3) all advertising materials mentioning
* features or use of this software display the following acknowledgement:
* ``This product includes software developed by Politecnico
* di Torino, and its contributors.'' (4) Neither the name of
* the University nor the names of its contributors may be used to endorse
* or promote products derived from this software without specific prior
* written permission.
* THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
* This product includes code licensed from RSA Data Security (Java Secure Socket Extension)
*
* Sun grants you ("Licensee") a non-exclusive, royalty free, license to use, and redistribute
* this software graphics artwork, as individual graphics or as a collection, as part of
* software code or programs that you develop, provided that
* i) this copyright notice and license accompany the software graphics artwork; and
* ii) you do not utilize the software graphics artwork in a manner which is disparaging to Sun.
* Unless enforcement is prohibited by applicable law, you may not modify the graphics,
* and must use them true to color and unmodified in every way.
*
*<p>
*/
package Utility;

import java.awt.Graphics2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.awt.Font;
import java.awt.geom.GeneralPath;
import java.awt.Point;
import java.awt.Shape;
import java.awt.RenderingHints;
import java.awt.Color;

import java.awt.event.MouseEvent;

import java.util.Vector;
import java.util.Enumeration;

import java.io.Serializable;

public class BusinessModelComponent implements Serializable{
    /**
	 * Identificativo del business component, coincide con la posizione, in cui si trova nel vettore di tutti i business component, +1
	 */
	private short busModCompID = -1;
	/**
	 * Identifica se il componente  un Business Role (se true)
	 * o una Business Relationship (se false).
     */
    private boolean isBusinessRole;
    /**
	 * Specifica se il componente prende parte o no al Business Model
     */
    private boolean isActive = false;
	/**
	 * Specifica se il componente  stato selezionato per visualizzare le informazioni ad esso associate
	 */
	 private transient boolean isSelected = false;
	/**
	 * Specifica se tutte le risorse associate a questo business component sono locali
	 */
	 private transient boolean  isAllResourcesLocal = true;
	 /**
	  * Specifica se il componente fa parte di una selezione multipla per il download
	  */
	  private transient boolean  isMultiSelected = false;
	/**
	 * Colore per lo stato di selezione-multipla per il download
	 */
	private final static Color BUS_MOD_COMP_MULTI_SELECTION_COLOR =Color.orange;
	/**
	 * Colore per lo stato di selezionato e selezione-multipla per il download
	 */
	private final static Color BUS_MOD_COMP_SELECTED_MULTI_SELECTION_COLOR =Color.magenta;
	/**
	 * Colore per lo stato di selezionato
	 */
	private final static Color BUS_MOD_COMP_SELECTED_COLOR =Color.blue;
	/**
	 * Colore per lo stato di attivo (incluso nel business model) e con risorse tutte locali
	 */
	private final static Color BUS_MOD_COMP_ACTIVE_LOCAL_RES_COLOR =Color.black;
	/**
	 * Colore per lo stato di attivo (incluso nel business model) e con tutte o alcune risorse remote
	 */
	 private final static Color BUS_MOD_COMP_ACTIVE_REMOTE_RES_COLOR = Color.red;
	/**
	 * Colore per lo stato di non attivo (non incluso nel business model)
	 */
	private final static Color BUS_MOD_COMP_NOTACTIVE_COLOR =Color.lightGray;
    /**
	 * Rappresenta il colore con cui viene disegnato il componente
     */
    private transient Color busModCompCurrentColor = BusinessModelComponent.BUS_MOD_COMP_NOTACTIVE_COLOR;
    /**
	 * Contiene gli oggetti grafici e le stringhe ad esso associate con relative
	 * coordinate, relative al business component in questione.
	 * Al suo interno ci sar o un Oval (per il business role) o un GeneralPath
	 * (per la business relationship) e un Vector contenente le stringhe associate
	 * al componente con relativa coordinate espresse in Point
     */
    private transient Vector busModCompGraphicalElements = null;
    /**
	 * Specifica il path dove si trova la documentazione dell'implementazione di questo componente
     */
    private String busModCompDocLocation[] = null;
   /**
	* Specifica il path dove si trovano i sorgenti dell'implementazione di questo componente
	*/
	private String busModCompSourceLocation[] = null;
	/**
	 * Specifica il path dove si trovano i binari dell'implementazione di questo componente
	*/
	private String busModCompBinaryLocation[] = null;
	/**
	 * Contiene una descrizione del componente
	 */
	private String busModCompDescription = null;
	/**
	 * Contiene tutte le risorse associate a questo componente
	*/
	private Vector busModComp_All_Resources = null;
	/**
	 * Definisce tutti con quali altri bus comp il presente ha dei collegamenti
	 */
	private short[] busModCompIDLinks = null;
	/**
	* Definisce i due reference point per una bus. relationship
	*/
	private ReferencePoint[] refPoints = null;

	/**
	 * Costruttore
	 */
	 public BusinessModelComponent(short busModCompID)    {
		this.busModCompID=busModCompID;
		//i business role sono numerati da 0 e nel vettore sono memorizzati da 0
		if(this.busModCompID<ProjectType.numBusinessRole)
			this.isBusinessRole=true;
		else
			this.isBusinessRole=false;

		this.busModComp_All_Resources= new Vector();

		if(this.busModCompID==ProjectType.BUSINESS_MODEL_RELATIONSHIP_USAGE_NET_SERVICES_CONNECTIVITY_PROVIDER   ||
			this.busModCompID == ProjectType.BUSINESS_MODEL_RELATIONSHIP_BROKERAGE_BROKER   ||
		    this.busModCompID==ProjectType.BUSINESS_MODEL_RELATIONSHIP_USAGE_NET_SERVICES_CONNECTIVITY_PROVIDER_BIS   ||
			this.busModCompID == ProjectType.BUSINESS_MODEL_RELATIONSHIP_BROKERAGE_BROKER_BIS)
		// un laccio, 1 solo ref point
		    this.refPoints = new ReferencePoint[1];
		else    {
			this.refPoints = new ReferencePoint[2];

			this.refPoints[1] = new ReferencePoint(null);
		}
		this.refPoints[0] = new ReferencePoint(null);
	 }

    /**
	 * Costruttore
     */
    public BusinessModelComponent(short busModCompID, boolean isActive, boolean isBusinessRole, short[] busModCompIDLinks, Vector busModCompGraphicalElements,Vector businessModelComponents) {
		this.busModCompID=busModCompID;

		this.isActive=isActive;
		this.isBusinessRole=isBusinessRole;
		this.SetBusModCompCurrentColor();
		this.busModCompGraphicalElements=(Vector)busModCompGraphicalElements.clone();

		if(!this.isBusinessRole)    {
			//bisogna gestire i reference point
			if(this.busModCompID == ProjectType.BUSINESS_MODEL_RELATIONSHIP_USAGE_NET_SERVICES_CONNECTIVITY_PROVIDER ||
				this.busModCompID == ProjectType.BUSINESS_MODEL_RELATIONSHIP_BROKERAGE_BROKER   ||
				this.busModCompID == ProjectType.BUSINESS_MODEL_RELATIONSHIP_USAGE_NET_SERVICES_CONNECTIVITY_PROVIDER_BIS ||
				this.busModCompID == ProjectType.BUSINESS_MODEL_RELATIONSHIP_BROKERAGE_BROKER_BIS)
			//ha 1 reference point solo (laccio)
				this.refPoints = new ReferencePoint[1];
			else    {
			//ha 2 reference point
				this.refPoints = new ReferencePoint[2];
				this.refPoints[1] = new ReferencePoint((Ellipse2D.Float)this.busModCompGraphicalElements.lastElement());
				this.busModCompGraphicalElements.remove(this.busModCompGraphicalElements.lastElement());
			}

			this.refPoints[0] = new ReferencePoint((Ellipse2D.Float)this.busModCompGraphicalElements.lastElement());
			this.busModCompGraphicalElements.remove(this.busModCompGraphicalElements.lastElement());

		}

		this.busModComp_All_Resources= new Vector();

		this.busModCompIDLinks=busModCompIDLinks;


		//questa distinzione  dovuto al fatto che le bus rela non posseggono dei propri gruppi di risorse
		//ma solo dei link ai corrispondenti gruppi presenti nei bus role.
		//Il busModCompIDLink in questo gruppo di risorse  quello specificato nella creazione del bus role
		//e quindi coincide sempre con il busModCompID dellla bus rela.
		//Il problema si risolve nelle bus rela salvando nel proprio vettore il link al gruppo di risorse
		//nella posizione corrispondente al busModCompID del bus role a cui le risorse appartengono.
		//Di default nel vettore per le bus rela vengono salvati ProjectType.numBusRole BusinessModelComponentResources
		//con busModCompIDLink = -1;.
		//Infatti sono ProjectType.numBusRole i bus role del mio bus model
		if(this.isBusinessRole) {
			//  un bus role, possiede dei gruppi di risorse propri
			for(int i=0;i<this.busModCompIDLinks.length;i++)
	    		this.SetBusModComp_1_Resource(busModCompIDLinks[i]);
		}
		else    {
		    //Inserisco il contenuto standard per il vettore delle risorse
			for(int i=0;i<ProjectType.numBusinessRole;i++)
				this.SetBusModComp_1_Resource((short)-1);

			this.SetBusModComp_All_ResourcesForBusRelationship(businessModelComponents);

		}
    }
	/**
	 * Imposta il contenuto delle busModComp_All_Resources di una bus rela, recuperando le info
	 * dal bus role col quale la bus rela  relazionato
	 */
	 public void SetBusModComp_All_ResourcesForBusRelationship(Vector businessModelComponents)   {
		//  una bus rela, non possiede dei gruppi di risorse propri, ma li recupera dai bus role con cui  relazionata
		for(int i=0;i<this.busModCompIDLinks.length;i++)    {
			//la length  sempre pari a 2 (o 1 nel caso del laccio)

			//rimuovo il contenuto di default presente nel vettore delle risorse
			this.busModComp_All_Resources.removeElementAt(busModCompIDLinks[i]);
			//recupero l'intero oggetto BusinessModelCOmponet con cui questa bus rela  legata, e da questo estraggo
			//l'oggetto BusinessModelComponentResources associato alla bus rela che sto creando

		    //gli aggiungo il reference point
			BusinessModelComponentResources tmp = ((BusinessModelComponent)businessModelComponents.get(busModCompIDLinks[i])).GetBusModComp_1_Resource(this.busModCompID);
			this.refPoints[i].SetIsAdded(tmp.refPoint.IsAdded());
			tmp.refPoint=this.refPoints[i];
			this.busModComp_All_Resources.add(busModCompIDLinks[i],((BusinessModelComponent)businessModelComponents.get(busModCompIDLinks[i])).GetBusModComp_1_Resource(this.busModCompID));
		}
		this.CheckAllResourcesLocal();
	 }

	/**
	 * Imposta l'attributo busModCompIDLinks in base al contenuto del vettore busModComp_All_Resources,
	 * Questo metodo viene invocato solo in fase di caricamento delle info dal file XML
	 */
	 void SetBusModCompIDLinks()    {
		if(!this.busModComp_All_Resources.isEmpty())    {
			//ho delle risorse associate al bus comp
			this.busModCompIDLinks = new short[this.busModComp_All_Resources.size()];
			for(int i=0;i<this.busModCompIDLinks.length;i++)
				this.busModCompIDLinks[i]=((BusinessModelComponentResources)this.busModComp_All_Resources.get(i)).GetBusModCompIDLink();
		}
	 }
    /**
	 * Restituisce il valore dell'attributo busModCompIDLinks
	 */
	 public short[] GetBusModCompIDLinks()  {
		return this.busModCompIDLinks;
	 }
	/**
	 * Restituisce il valore dell'attributo isBusinessRole
     */
    public boolean IsBusinessRole() {
		return this.isBusinessRole;
    }
	 /**
	 * Restituisce il valore dell'attributo isSelected
     */
    public boolean IsSelected() {
		return this.isSelected;
    }
    /**
	 * Imposta il valore dell'attributo isSelected
     */
    public void SetIsSelected(boolean newState) {
		this.isSelected=newState;
		this.SetBusModCompCurrentColor();
    }
    /**
	 * Restituisce il valore dell'attributo isActive
     */
    public boolean IsActive() {
		return this.isActive;
    }
	/**
	 * Restituisce il valore dell'attributo isMultiSelected
	 */
	 public boolean IsMultiSelected()   {
		return this.isMultiSelected;
	 }
	/**
	 * Imposta il valore dell'attributo isMultiSelected
	 */
	 public void SetIsMultiSelected(boolean newState)   {
		this.isMultiSelected=newState;
		this.SetBusModCompCurrentColor();
	 }
    /**
	 * Imposta il valore dell'attributo isActive
     */
    public void SetIsActive(boolean newState) {
		this.isActive=newState;
		this.SetBusModCompCurrentColor();
    }
    /**
	 * Disegna il componente specifico
     */
    public void paint(Graphics2D g) {
		//ctrl se il bus. comp. non sia la bus. rela fittizia per terminare i lacci
		if(this.busModCompID == ProjectType.BUSINESS_MODEL_RELATIONSHIP_BROKERAGE_BROKER_BIS    ||
		    this.busModCompID == ProjectType.BUSINESS_MODEL_RELATIONSHIP_USAGE_NET_SERVICES_CONNECTIVITY_PROVIDER_BIS)
			return;

		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
	    g.setRenderingHint(RenderingHints.KEY_RENDERING,RenderingHints.VALUE_RENDER_QUALITY);
		Enumeration componentList= this.busModCompGraphicalElements.elements();
	    if(this.isBusinessRole)    {
			// il componente in questione  un Business Role
			Ellipse2D.Float businessRoleOval =(Ellipse2D.Float)componentList.nextElement();
			g.setColor(Color.white);
//	    	g.fillOval((int)businessRoleOval.getX()+1, (int)businessRoleOval.getY()+1, (int)businessRoleOval.getWidth()-2, (int)businessRoleOval.getHeight()-2);
		    // lo disegno prima in bianco e poi nel colore giusto per evitare
			// che il disegno rimanga leggermente sbavato
			g.draw(businessRoleOval);
			g.drawOval((int)businessRoleOval.getX()+1, (int)businessRoleOval.getY()+1, (int)businessRoleOval.getWidth()-2, (int)businessRoleOval.getHeight()-2);
			g.setColor(this.busModCompCurrentColor);
			g.draw(businessRoleOval);
			g.drawOval((int)businessRoleOval.getX()+1, (int)businessRoleOval.getY()+1, (int)businessRoleOval.getWidth()-2, (int)businessRoleOval.getHeight()-2);

			g.setFont(new Font("Helvetica", Font.BOLD, 18));

			Enumeration list=this.busModComp_All_Resources.elements();
			while(list.hasMoreElements())   {
				BusinessModelComponentResources curr = (BusinessModelComponentResources)list.nextElement();
				if(!curr.refPoint.IsAdded())
					g.setColor(this.BUS_MOD_COMP_NOTACTIVE_COLOR);
				else
					g.setColor(this.busModCompCurrentColor);
				g.fillOval((int)curr.refPoint.graphicsPoint.getX(),(int)curr.refPoint.graphicsPoint.getY(),(int)curr.refPoint.graphicsPoint.getWidth(),(int)curr.refPoint.graphicsPoint.getHeight());
			}
			g.setColor(this.busModCompCurrentColor);
	    }
		else    {
	    	// il componente in questione  una Business Relationship
			GeneralPath businessRelationship= (GeneralPath)componentList.nextElement();
		    // lo disegno prima in bianco e poi nel colore giusto per evitare
			// che il disegno rimanga leggermente sbavato
			g.setColor(Color.white);
			g.draw(businessRelationship);
			g.setColor(this.busModCompCurrentColor);
			g.draw(businessRelationship);
			g.setFont(new Font("Helvetica", Font.ITALIC, 18));

			// mi serve per saltare il rettangolo attorno alla scritta, necessaria
			// per rendere sensibile al click la business relationship
			componentList.nextElement();
//		    g.draw((Rectangle2D)componentList.nextElement());
		}
	    while(componentList.hasMoreElements())  {
			String tmp=(String)componentList.nextElement();
			Point tmp2=(Point)componentList.nextElement();
		    // lo disegno prima in bianco e poi nel colore giusto per evitare
			// che il disegno rimanga leggermente sbavato
			g.setColor(Color.white);
			g.drawString(tmp,(int)tmp2.getX(),(int)tmp2.getY());
			g.setColor(this.busModCompCurrentColor);
			g.drawString(tmp,(int)tmp2.getX(),(int)tmp2.getY());
	    }
    }
	/**
	 * Controlla se il click del mouse si trovi all'interno del componente in
	 * questione del business modell
	 */
	public boolean CheckClickMouse(Point p)    {
		if(this.isBusinessRole) {
			Ellipse2D.Float businessRoleOval =(Ellipse2D.Float)this.busModCompGraphicalElements.firstElement();
			return businessRoleOval.contains(p);
		}
		else    {
			GeneralPath businessRelationship= (GeneralPath)this.busModCompGraphicalElements.firstElement();
			Rectangle2D.Float businessRelationshipTitle = (Rectangle2D.Float)this.busModCompGraphicalElements.elementAt(1);
			if(businessRelationship.contains(p) ||
				businessRelationshipTitle.contains(p))
				return true;
			return false;
		}
	}
	/**
	 * Aggiunge un nuovo gruppo di risorse al vettore busModComp_All_Resources
	 */
	 void SetBusModComp_1_Resource(short busModCompIDLink)  {
		this.busModComp_All_Resources.add(new BusinessModelComponentResources(busModCompIDLink));
	 }

	/**
	 * Restituisce il corretto gruppo di risorse richiesto
	 */
	 private BusinessModelComponentResources GetBusModComp_1_Resource(short busModCompIDLink)  {
		//questa distinzione  dovuto al fatto che le bus rela non posseggono dei propri gruppi di risorse
		//ma solo dei link ai corrispondenti gruppi presenti nei bus role.
		//Il busModCompIDLink in questo gruppo di risorse  quello specificato nella creazione del bus role
		//e quindi coincide sempre con il busModCompID dellla bus rela.
		//Il problema si risolve nelle bus rela salvando nel proprio vettore il link al gruppo di risorse
		//nella posizione corrispondente al busModCompID del bus role a cui le risorse appartengono.
		//Di default nel vettore per le bus rela vengono salvati ProjectType.numBusRole BusinessModelComponentResources
		//con busModCompIDLink = -1;.
		//Infatti sono ProjectType.numBusRole i bus role del mio bus model
		if(this.isBusinessRole) {
			// 1 bus role
			Enumeration list=this.busModComp_All_Resources.elements();
			while(list.hasMoreElements())   {
				BusinessModelComponentResources curr = (BusinessModelComponentResources)list.nextElement();
				if(curr.GetBusModCompIDLink()==busModCompIDLink)
					return curr;
			}
		}
		else    {
			// 1 bus relationship
			return (BusinessModelComponentResources) this.busModComp_All_Resources.get(busModCompIDLink);
		}
		//non verr mai chiamato
		return null;
	 }

    /**
       Imposta il valore di uno degli attributi busModCompResSourceLocation contenuti in busModCompResources
     */
    public void SetBusModComp_1_ResSourceLoc(String[] location, short busModCompIDLink) {
		this.GetBusModComp_1_Resource(busModCompIDLink).SetBusModCompResSourceLoc(location);
		this.CheckAllResourcesLocal();
    }
    /**
       Restituisce il valore di uno degli attributi busModCompResSourceLocation contenuti in busModCompResources
     */
    public String[] GetBusModComp_1_ResSourceLoc(short busModCompIDLink) {
		return this.GetBusModComp_1_Resource(busModCompIDLink).GetBusModCompResSourceLoc();
    }
    /**
       Imposta il valore di uno degli attributi busModCompResBinaryLocation contenuti in busModCompResources
     */
    public void SetBusModComp_1_ResBinaryLoc(String[] location, short busModCompIDLink) {
		this.GetBusModComp_1_Resource(busModCompIDLink).SetBusModCompResBinaryLoc(location);
		this.CheckAllResourcesLocal();
    }
    /**
       Restituisce il valore di uno degli attributi busModCompResBinaryLocation contenuti in busModCompResources
     */
    public String[] GetBusModComp_1_ResBinaryLoc(short busModCompIDLink) {
		return this.GetBusModComp_1_Resource(busModCompIDLink).GetBusModCompResBinaryLoc();
    }
    /**
       Imposta il valore di uno degli attributi busModCompResDocLocation contenuti in busModCompResources
     */
    public void SetBusModComp_1_ResDocLoc(String[] location, short busModCompIDLink) {
		this.GetBusModComp_1_Resource(busModCompIDLink).SetBusModCompResDocLoc(location);
		this.CheckAllResourcesLocal();
    }
    /**
       Restituisce il valore di uno degli attributi busModCompResDocLocation contenuti in busModCompResources
     */
    public String[] GetBusModComp_1_ResDocLoc(short busModCompIDLink) {
		return this.GetBusModComp_1_Resource(busModCompIDLink).GetBusModCompResDocLoc();
    }
    /**
       Imposta il valore di uno degli attributi busModCompResDescription contenuti in busModCompResources
     */
    public void SetBusModComp_1_ResDecription(String description,short busModCompIDLink) {
		this.GetBusModComp_1_Resource(busModCompIDLink).SetBusModCompResDecription(description);
		this.CheckAllResourcesLocal();
    }
    /**
       Restituisce il valore di uno degli attributi busModCompResDescription contenuti in busModCompResources
     */
    public String GetBusModComp_1_ResDescription(short busModCompIDLink) {
		return this.GetBusModComp_1_Resource(busModCompIDLink).GetBusModCompResDescription();
    }
    /**
       Imposta il valore dell'attributo busModCompBinaryLocation
     */
    public void SetBusModCompBinaryLoc(String[] location) {
		if(location==null)  {
			//ho tolto tutte le risorse
			this.busModCompBinaryLocation=location;
		}
		else    {
			if(this.busModCompBinaryLocation==null ||
				this.busModCompBinaryLocation.length!=location.length)
			// la prima volta che imposto queste risorse
			//o sto aggiornando le risorse togliendole o aggiungendole
				this.busModCompBinaryLocation=location;
			else    {
				//sto aggiornando i dati
				for(int i=0;i<location.length;i++)  {
					if(location[i]!=null)
						this.busModCompBinaryLocation[i]=location[i];
				}
			}
		}
		this.CheckAllResourcesLocal();
    }
    /**
       Imposta il valore dell'attributo busModCompSourceLocation
     */
    public void SetBusModCompSourceLoc(String[] location) {
		if(location==null)  {
			//ho tolto tutte le risorse
			this.busModCompSourceLocation=location;
		}
		else    {
			if(this.busModCompSourceLocation==null ||
				this.busModCompSourceLocation.length!=location.length)
			// la prima volta che imposto queste risorse
			//o sto aggiornando le risorse togliendole o aggiungendole
				this.busModCompSourceLocation=location;
			else    {
				//sto aggiornando i dati
				for(int i=0;i<location.length;i++)  {
					if(location[i]!=null)
						this.busModCompSourceLocation[i]=location[i];
				}
			}
		}
		this.CheckAllResourcesLocal();
    }
    /**
       Imposta il valore dell'attributo busModCompDocLocation
     */
    public void SetBusModCompDocLoc(String[] location) {
		if(location==null)  {
			//ho tolto tutte le risorse
			this.busModCompDocLocation=location;
		}
		else    {
			if(this.busModCompDocLocation==null ||
				this.busModCompDocLocation.length!=location.length)
			// la prima volta che imposto queste risorse
			//o sto aggiornando le risorse togliendole o aggiungendole
				this.busModCompDocLocation=location;
			else    {
				//sto aggiornando i dati
				for(int i=0;i<location.length;i++)  {
					if(location[i]!=null)
						this.busModCompDocLocation[i]=location[i];
				}
			}
		}
		this.CheckAllResourcesLocal();
    }
    /**
       Restituisce il valore dell'attributo busModCompSourceLocation
     */
    public String[] GetBusModCompSourceLoc() {
		return this.busModCompSourceLocation;
    }
    /**
       Restituisce il valore dell'attributo busModCompBinaryLocation
     */
    public String[] GetBusModCompBinaryLoc() {
		return this.busModCompBinaryLocation;
    }
    /**
       Restituisce il valore dell'attributo busModCompDocLocation
     */
    public String[] GetBusModCompDocLoc() {
		return this.busModCompDocLocation;
    }
    /**
       Imposta il valore dell'attributo busModCompDescription
       @roseuid 3BA9A9B801C0
     */
    public void SetBusModCompDecription(String description) {
		this.busModCompDescription=description;
    }
    /**
       Restituisce il valore dell'attributo busModCompDecription
       @roseuid 3BA9AA050261
     */
    public String GetBusModCompDescription() {
		return this.busModCompDescription;
    }

	/**
	 * Restituisce il valore dell'attributo busModCompID
	 */
	 public short GetBusModCompID()    {
		return this.busModCompID;
	 }

	/**
	 * Resetta ai valori di default tutti i componenti del Business Model
	 */
	 public void ResetAll()  {
		Enumeration list = this.busModComp_All_Resources.elements();
		if(this.isBusinessRole) {
			//se  un bus. role ha potere sulle implementazioni delle relationship
			//altrimenti no.
			//se non faccio questo ctrl. se tolgo una relationship, perdo anche le info
			//relative alla sua implementazione sui rispettivi bus. role
			while(list.hasMoreElements())   {
				((BusinessModelComponentResources)list.nextElement()).ResetAll();
			}
		}


		this.busModCompDescription=null;
		this.busModCompDocLocation=null;
		this.busModCompBinaryLocation=null;
		this.busModCompSourceLocation=null;
		this.isActive=false;
		this.isSelected=false;
		this.isAllResourcesLocal=true;
		this.isMultiSelected=false;
		this.SetBusModCompCurrentColor();
	 }

	 /**
	  * Cambia il colore associato del componente in base allo stato di selezionato e incluso
	  */
	private void SetBusModCompCurrentColor()    {
		if(this.busModCompCurrentColor==null)
			this.busModCompCurrentColor=BusinessModelComponent.BUS_MOD_COMP_NOTACTIVE_COLOR;

		if(this.isMultiSelected && this.isSelected)
			this.busModCompCurrentColor=BusinessModelComponent.BUS_MOD_COMP_SELECTED_MULTI_SELECTION_COLOR;
		else    {
			if(this.isMultiSelected)
				this.busModCompCurrentColor=BusinessModelComponent.BUS_MOD_COMP_MULTI_SELECTION_COLOR;
			else    {
				if(this.isSelected)
					this.busModCompCurrentColor=BusinessModelComponent.BUS_MOD_COMP_SELECTED_COLOR;
				else    {
					if(this.isActive)   {
						if(this.isAllResourcesLocal)
							this.busModCompCurrentColor=BusinessModelComponent.BUS_MOD_COMP_ACTIVE_LOCAL_RES_COLOR;
						else
							this.busModCompCurrentColor=BusinessModelComponent.BUS_MOD_COMP_ACTIVE_REMOTE_RES_COLOR;
					}
					else
						this.busModCompCurrentColor=BusinessModelComponent.BUS_MOD_COMP_NOTACTIVE_COLOR;
				}
			}
		}
	}

	/**
	 * Imposta le info associate al bus comp in base a quelle gi contenute e quelle associate al bus comp corrispondente di defualt
	 */
//	public void SetReplaceBusModCompGraphicalElements(Vector busModCompGraphicalElements) {
	public void SetReplaceBusModCompInfos(BusinessModelComponent defaultBusModComp) {
		// imposto i link con altri bus comp
		this.busModCompIDLinks=defaultBusModComp.busModCompIDLinks;
		if(this.busModComp_All_Resources.isEmpty())
		//sto creando un prj nuovo
		//oppure quello caricato non risorse
			this.busModComp_All_Resources.addAll(defaultBusModComp.busModComp_All_Resources);

		//imposto gli oggetti grafici
		if(this.busModCompGraphicalElements==null)
			this.busModCompGraphicalElements= new Vector();
		else
			this.busModCompGraphicalElements.removeAllElements();
		this.isSelected=false;

		this.SetBusModCompCurrentColor();

		this.busModCompGraphicalElements.addAll(defaultBusModComp.busModCompGraphicalElements);

		//imposto i reference point
		this.refPoints = defaultBusModComp.refPoints;
	}

	/**
	 * Restituisce il vettore contenente gli oggetti grafici specifici del business component
	 */
	public Vector GetBusModCompGraphicalElements()   {
		return this.busModCompGraphicalElements;
	}

	/**
	 * Imposta il valore identificativo del tipo di business component
	 * Viene chiamato solo in fase di caricamente delle info dal file XML
	 */
	void SetIsBusinessRole(boolean isBusinessRole)  {
		this.isBusinessRole=isBusinessRole;
	}

	 /**
	  * Restituisce il valore dell'attributo allResourcesLocal
	  */
	  public boolean IsAllResourcesLocal() {
		return this.isAllResourcesLocal;
	  }
	/**
	 * Al termine di ogni download, ctrl se i bus comp a cui quello corrente  legato hanno tutte le risorse locali
	 */
	 public void CheckBusModCompLinksAllResourcesLocal(Vector busModComponents) {
		for(int i=0;i<this.busModCompIDLinks.length;i++)    {
			BusinessModelComponent curr = (BusinessModelComponent)busModComponents.get(busModCompIDLinks[i]);
			if(curr.isActive)
				curr.CheckAllResourcesLocal();
		}
	 }

	  /**
	   * Ctrl se tutte le risorse siano locali o no
	   */
	private void CheckAllResourcesLocal() {
		Enumeration list = this.busModComp_All_Resources.elements();
		while(list.hasMoreElements())   {
			if(!((BusinessModelComponentResources)list.nextElement()).IsAllResourcesLocal())    {
			//se anche uno solo dei gruppi di risorse non sono tutte locali, allora tutto
			//il componente  in stato di non locale, e posso uscire dal while subito
				this.isAllResourcesLocal=false;
				break;
			}
			else
				this.isAllResourcesLocal=true;
		}
		if(this.isAllResourcesLocal)    {
			//tutti i gruppi di risorse sono locali, ctrl quelle associate direttamente
			//al bus comp
			int numResourcesLocal=0;
			if(this.busModCompBinaryLocation!=null) {
			    int i;
				for(i=0;i<this.busModCompBinaryLocation.length;i++) {
					if(this.busModCompBinaryLocation[i].lastIndexOf('@')!=-1)
					//la risorsa non  locale
						break;
				}
				if(i==this.busModCompBinaryLocation.length)
				// uscito dal for non con il break
				    numResourcesLocal++;
			}
			else
				numResourcesLocal++;

			if(this.busModCompSourceLocation!=null) {
			    int i;
				for(i=0;i<this.busModCompSourceLocation.length;i++) {
					if(this.busModCompSourceLocation[i].lastIndexOf('@')!=-1)
					//la risorsa non  locale
						break;
				}
				if(i==this.busModCompSourceLocation.length)
				// uscito dal for non con il break
				    numResourcesLocal++;
			}
			else
				numResourcesLocal++;

			if(this.busModCompDocLocation!=null)    {
			    int i;
			    for(i=0;i<this.busModCompDocLocation.length;i++) {
					if(this.busModCompDocLocation[i].lastIndexOf('@')!=-1)
					//la risorsa non  locale
						break;
				}
				if(i==this.busModCompDocLocation.length)
				// uscito dal for non con il break
				    numResourcesLocal++;
			}
			else
				numResourcesLocal++;

			//tutti le risorse sono locali
			if(numResourcesLocal==3)
				this.isAllResourcesLocal=true;
			else
				this.isAllResourcesLocal=false;
		}

		this.SetBusModCompCurrentColor();
	}

	/**
	 * Definisce le risorse associate ad ogni relazione tra bus comp
	 */
	private class BusinessModelComponentResources  {
	    /**
		 * Specifica il path dove si trovano i sorgenti dell'implementazione di questo componente
	     */
	    private String[] busModCompResSourceLocation = null;
	    /**
		 * Specifica il path dove si trovano i binari dell'implementazione di questo componente
	     */
	    private String[] busModCompResBinaryLocation = null;
	    /**
		 * Specifica il path dove si trova la documentazione dell'implementazione di questo componente
	     */
	    private String[] busModCompResDocLocation = null;
		/**
		 * Contiene una descrizione del componente
		 */
		 private String busModCompResDescription = null;
		 /**
		  * Specifica a quale bus comp si riferiscono le risorse associate
		  */
		  private short busModCompIDLink=-1;
		/**
		 * Specifica se tutte le risorse associate sono locali
		 */
		 private transient boolean  isAllResourcesLocal = true;
		/**
		 * Specifica il reference point associato alla risorsa
		 */
		private transient ReferencePoint refPoint = null;

		/**
		 * Costruttore
		 */
		public BusinessModelComponentResources(short busModCompIDLink)    {
			this.busModCompIDLink=busModCompIDLink;

			this.refPoint = new ReferencePoint(null);
		}

	    /**
	       Imposta il valore dell'attributo busModCompResSourceLocation
	       @roseuid 3BA9A970016C
	     */
	    protected void SetBusModCompResSourceLoc(String[] location) {
			if(location==null)  {
				//ho tolto tutte le risorse
				this.busModCompResSourceLocation=location;
			}
			else    {
				if(this.busModCompResSourceLocation==null ||
					this.busModCompResSourceLocation.length!=location.length)
		    		// la prima volta che imposto queste risorse
					//o sto aggiornando le risorse togliendole o aggiungendole
					this.busModCompResSourceLocation=location;
				else    {
					//sto aggiornando i dati
					for(int i=0;i<location.length;i++)  {
						if(location[i]!=null)
							this.busModCompResSourceLocation[i]=location[i];
					}
				}
			}
	    	this.CheckAllResourcesLocal();
			this.CheckResourcesForRefPoint();
	    }
	    /**
	       Imposta il valore dell'attributo busModCompResBinaryLocation
	       @roseuid 3BA9A9B80043
	     */
	    protected void SetBusModCompResBinaryLoc(String[] location) {
			if(location==null)  {
				//ho tolto tutte le risorse
				this.busModCompResBinaryLocation=location;
			}
			else    {
				if(this.busModCompResBinaryLocation==null ||
					this.busModCompResBinaryLocation.length!=location.length)
		    		// la prima volta che imposto queste risorse
					//o sto aggiornando le risorse togliendole o aggiungendole
					this.busModCompResBinaryLocation=location;
				else    {
					//sto aggiornando i dati
					for(int i=0;i<location.length;i++)  {
						if(location[i]!=null)
							this.busModCompResBinaryLocation[i]=location[i];
					}
				}
			}
	  		this.CheckAllResourcesLocal();
			this.CheckResourcesForRefPoint();
	    }
	    /**
	       Imposta il valore dell'attributo busModCompResDocLocation
	       @roseuid 3BA9A9B801C0
	     */
	    protected void SetBusModCompResDocLoc(String[] location) {
			if(location==null)  {
				//ho tolto tutte le risorse
				this.busModCompResDocLocation=location;
			}
			else    {
				if(this.busModCompResDocLocation==null ||
					this.busModCompResDocLocation.length!=location.length)
		    		// la prima volta che imposto queste risorse
					//o sto aggiornando le risorse togliendole o aggiungendole
					this.busModCompResDocLocation=location;
	   			else    {
					//sto aggiornando i dati
					for(int i=0;i<location.length;i++)  {
						if(location[i]!=null)
							this.busModCompResDocLocation[i]=location[i];
					}
				}
			}
	  		this.CheckAllResourcesLocal();
			this.CheckResourcesForRefPoint();
	    }
	    /**
	       Imposta il valore dell'attributo busModCompResDescription
	       @roseuid 3BA9A9B801C0
	     */
	    protected void SetBusModCompResDecription(String description) {
			this.busModCompResDescription=description;
			this.CheckResourcesForRefPoint();
        }
	    /**
	       Restituisce il valore dell'attributo busModCompResDocLocation
	       @roseuid 3BA9A9E00326
	     */
	    public String[] GetBusModCompResDocLoc() {
			return this.busModCompResDocLocation;
	    }
	    /**
	       Restituisce il valore dell'attributo busModCompResSourceLocation
	       @roseuid 3BA9AA05018E
	     */
	    public String[] GetBusModCompResSourceLoc() {
			return this.busModCompResSourceLocation;
	    }
	    /**
	       Restituisce il valore dell'attributo busModCompResBinaryLocation
	       @roseuid 3BA9AA050261
	     */
	    public String[] GetBusModCompResBinaryLoc() {
			return this.busModCompResBinaryLocation;
	    }
	    /**
	       Restituisce il valore dell'attributo busModCompResDescription
	       @roseuid 3BA9AA050261
	     */
	    public String GetBusModCompResDescription() {
			return this.busModCompResDescription;
	    }
		/**
		 *  Restituisce il valore dell'attributo busModCompIDLink
		 */
		 public short GetBusModCompIDLink() {
			return this.busModCompIDLink;
		 }

	    /**
		 * Ctrl se sono state specificate delle risorse, e di conseguenza muta lo stato del refPoint
		 */
		 private void CheckResourcesForRefPoint()    {
			if(this.busModCompResDescription==null &&
				this.busModCompResSourceLocation==null &&
				this.busModCompResBinaryLocation==null &&
				this.busModCompResDocLocation==null)
				//non ho risorse
				this.refPoint.SetIsAdded(false);
			else
				//ho qualche risorsa associata
				this.refPoint.SetIsAdded(true);
		 }


		/**
		* Ctrl se tutte le risorse siano locali o no
	    */
		private void CheckAllResourcesLocal() {
			int numResourcesLocal=0;
			if(this.busModCompResSourceLocation!=null) {
			    int i;
				for(i=0; i<this.busModCompResSourceLocation.length;i++) {
				    if(this.busModCompResSourceLocation[i].lastIndexOf('@')!=-1)
					//la risorsa non  locale
						break;
				}
				if(i==this.busModCompResSourceLocation.length)
				// uscito dal for non con il break
				    numResourcesLocal++;
			}
			else
				numResourcesLocal++;

			if(this.busModCompResBinaryLocation!=null) {
			    int i;
				for(i=0; i<this.busModCompResBinaryLocation.length;i++) {
				    if(this.busModCompResBinaryLocation[i].lastIndexOf('@')!=-1)
					//la risorsa non  locale
						break;
				}
				if(i==this.busModCompResBinaryLocation.length)
				// uscito dal for non con il break
				    numResourcesLocal++;
			}
			else
				numResourcesLocal++;

			if(this.busModCompResDocLocation!=null)    {
			    int i;
				for(i=0; i<this.busModCompResDocLocation.length;i++) {
				    if(this.busModCompResDocLocation[i].lastIndexOf('@')!=-1)
					//la risorsa non  locale
						break;
				}
				if(i==this.busModCompResDocLocation.length)
				// uscito dal for non con il break
				    numResourcesLocal++;
			}
			else
				numResourcesLocal++;

			//tutti le risorse sono locali
			if(numResourcesLocal==3)
				this.isAllResourcesLocal=true;
			else
				this.isAllResourcesLocal=false;
		}

		/**
		 * Resetta ai valori di default tutti i componenti del Business Model
		 */
		 public void ResetAll()  {
			this.busModCompResSourceLocation=null;
			this.busModCompResBinaryLocation=null;
			this.busModCompResDocLocation=null;
			this.busModCompResDescription=null;
			this.isAllResourcesLocal=true;
			this.refPoint.SetIsAdded(false);
	     }

	 /**
	  * Restituisce il valore dell'attributo allResourcesLocal
	  */
	  public boolean IsAllResourcesLocal() {
		return this.isAllResourcesLocal;
	  }
	}//finisce classe BusinessModelComponentResources

	/**
	 * Classe a gestione dei reference point (in questo caso i pallini grafici terminatori delle bus. rela.)
	 */
	 private class ReferencePoint {
		/**
		 * E' il Pallino grafico
		 */
		private Ellipse2D.Float graphicsPoint = null;
		/**
		 * Indica se per il ref. point corrente sono state associate delle risorse
		 */
		private boolean isAdded = false;

		/**
		 * Costruttore
		 */
		ReferencePoint (Ellipse2D.Float graphicsPoint)   {
			this.graphicsPoint = graphicsPoint;
		}
		/**
		 * Imposta il valore di isAdded
		 */
		 protected void SetIsAdded(boolean newState) {
			this.isAdded=newState;
		 }
		 /**
		  * Restituisce il valore di isAdded
		  */
		  public boolean IsAdded()  {
			return this.isAdded;
		  }


	 }
}

