/*
*                            Copyright (c) 2001
*                        <B>Politecnico di Torino</B>
*                            All Rights Reserved
* <p>
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that: (1) source code distributions
* retain the above copyright notice and this paragraph in its entirety,(2)
* distributions including binary code include the above copyright notice and
* this paragraph in its entirety in the documentation or other materials
* provided with the distribution, and (3) all advertising materials mentioning
* features or use of this software display the following acknowledgement:
* ``This product includes software developed by Politecnico
* di Torino, and its contributors.'' (4) Neither the name of
* the University nor the names of its contributors may be used to endorse
* or promote products derived from this software without specific prior
* written permission.
* THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
* This product includes code licensed from RSA Data Security (Java Secure Socket Extension)
*
* Sun grants you ("Licensee") a non-exclusive, royalty free, license to use, and redistribute
* this software graphics artwork, as individual graphics or as a collection, as part of
* software code or programs that you develop, provided that
* i) this copyright notice and license accompany the software graphics artwork; and
* ii) you do not utilize the software graphics artwork in a manner which is disparaging to Sun.
* Unless enforcement is prohibited by applicable law, you may not modify the graphics,
* and must use them true to color and unmodified in every way.
*
*<p>
*/
package Utility;

import javax.swing.JOptionPane;

import java.awt.Component;

/**
 * Classe delle eccezioni in Politella
 */
public class PolitellaException extends Exception {
	/**
	 * Impossibile connettersi al peer per download
	 */
	public static final short NO_DOWNLOAD_CONNECTION = 1;
	/**
	 * Impossibile temrinare il processo di download
	 */
	public static final short NO_DOWNLOAD_TRANSFER = 2;
	/**
	 * Impossibile connettersi al peer per l'upload
	 */
	public static final short NO_UPLOAD_CONNECTION = 3;
	/**
	 * Impossiible terminare il processo di upload
	 */
	public static final short NO_UPLOAD_TRANSFER = 4;
	/**
	 * Impossibile connettersi al peer per scambio messaggi
	 */
	public static final short NO_MESSAGE_CONNECTION = 5;
	/**
	 * Impossibile terminare lo scambio di messaggi con il peer
	 */
	public static final short NO_MESSAGE_TRANSFER = 6;
	/**
	 * Impossbile crare un socket di ascolto
	 */
	public static final short NO_LISTEN_SOCKET = 7;
	/**
	 * Impossibile caricare i db dei progetti
	 */
	public static final short NO_DB_LOAD = 8;
	/**
	 * Impossibile salvare i db dei progetti
	 */
	public static final short NO_DB_SAVE = 9;
	/**
	 * Impossibile caricare le impostazioni di setup
	 */
	public static final short NO_SETUP_LOAD = 10;
	/**
	 * Impossibile salvare le impostazioni di setup
	 */
	public static final short NO_SETUP_SAVE = 11;
	/**
	 * Impossibile visualizzare il progetto ricercato
	 */
	public static final short NO_PROJECT_LOAD = 12;
	/**
	 * Impossibile salvare il progetto visualizzato e/o editato
	 */
	public static final short NO_PROJECT_SAVE = 13;
	/**
	 * Impossibile spostare la dtd nella nuova dir dei db, oppure cancellarla dalla dir user.dir
	 */
	public static final short NO_POLITELLA_DTD_MANAGE = 14;
	/**
	 * Impossibile inizializzare l'ambiente SSL
	 */
	public static final short NO_SSL_INITIALIZATION = 15;
	/**
	 * Informazioni aggiuntive
	 */
	private String data = null;
	/**
	 * Specifica il tipo di eccezzione generata
	 */
	private short type;

	/**
	 * Restituisce il valore del campo data
	 */
	public String GetData() {
		return this.data;
	}

	/**
	 * Restituisce il valore del campo type
	 */
	public short GetType() {
		return this.type;
	}

	 /**
	  * Costruttore
	  */
	public PolitellaException(short type,String data)	{
		this.type=type;
		this.data=data;
	}

	/**
	 * Elabora un eccezione scaturita all'interno del programma
	 */
	public static synchronized void ProcessException(Component parent,PolitellaException e) {
		switch(e.type){
			case PolitellaException.NO_DB_LOAD: {
				JOptionPane.showMessageDialog(parent,"Impossible loading share, downloaded and found projects list.","Error",JOptionPane.ERROR_MESSAGE);
				System.exit(-1);
				break;
			}
			case PolitellaException.NO_DB_SAVE: {
				JOptionPane.showMessageDialog(parent,"Impossible saving share and found projects list.\nThere will be set up to default value","Error",JOptionPane.ERROR_MESSAGE);
				break;
			}
			case PolitellaException.NO_POLITELLA_DTD_MANAGE: {
				JOptionPane.showMessageDialog(parent,"Impossible moving the Politella DTD, saved in file politella.dtd, from\n" +System.getProperty("user.dir")+ "\nto\n" + e.GetData() + "\nMake it manually and check if the disc is full or write-protected","Error",JOptionPane.ERROR_MESSAGE);
				break;
			}
			case PolitellaException.NO_SETUP_LOAD: {
				JOptionPane.showMessageDialog(parent,"Impossible loading setup parameters.\nThere will be set up to default value","Error",JOptionPane.ERROR_MESSAGE);
				break;
			}
			case PolitellaException.NO_SETUP_SAVE: {
				JOptionPane.showMessageDialog(parent,"Impossible saving setup parameters.\nCheck if the disc is full or write-protected","Error",JOptionPane.ERROR_MESSAGE);
				break;
			}
			case PolitellaException.NO_PROJECT_LOAD: {
				JOptionPane.showMessageDialog(parent,"Impossible loading the project included in the file:\n"+ e.GetData()+"\nCheck it manually","Error",JOptionPane.ERROR_MESSAGE);
				break;
			}
			case PolitellaException.NO_PROJECT_SAVE: {
				JOptionPane.showMessageDialog(parent,"Impossible saving current project in the file:\n"+ e.GetData()+".\nCheck if the disc is full or write-protected","Error",JOptionPane.ERROR_MESSAGE);
				break;
			}
			case PolitellaException.NO_DOWNLOAD_CONNECTION: {
                JOptionPane.showMessageDialog(parent,"Impossible connecting to "+ e.GetData() +" for downloading.\nTry again later","Error",JOptionPane.ERROR_MESSAGE);
				break;
			}
			case PolitellaException.NO_DOWNLOAD_TRANSFER: {
                JOptionPane.showMessageDialog(parent,"Impossible downloading the file "+ e.GetData() +".\nMaybe it was deleted.","Error",JOptionPane.ERROR_MESSAGE);
				break;
			}
			case PolitellaException.NO_LISTEN_SOCKET: {
				JOptionPane.showMessageDialog(parent,"No TCP port available.\nTry again later","Error",JOptionPane.ERROR_MESSAGE);
				System.exit(-1);
				break;
			}
			case PolitellaException.NO_SSL_INITIALIZATION:  {
				JOptionPane.showMessageDialog(parent,"SSL Initialization failed.\nTry again later","Error",JOptionPane.ERROR_MESSAGE);
				System.exit(-1);
				break;
			}
			case PolitellaException.NO_MESSAGE_CONNECTION: {
				JOptionPane.showMessageDialog(parent,"The access peer '"+e.data+"' is unreachable or unknown.\nPlease check it.","Error",JOptionPane.ERROR_MESSAGE);
				break;
			}
			case PolitellaException.NO_MESSAGE_TRANSFER: {
//                System.out.println("Errore nel invio di un messaggio");
				break;
			}
			case PolitellaException.NO_UPLOAD_CONNECTION: {
				break;
			}
			case PolitellaException.NO_UPLOAD_TRANSFER: {
				break;
			}
		}

	}
}
