/*
*                            Copyright (c) 2001
*                        <B>Politecnico di Torino</B>
*                            All Rights Reserved
* <p>
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that: (1) source code distributions
* retain the above copyright notice and this paragraph in its entirety,(2)
* distributions including binary code include the above copyright notice and
* this paragraph in its entirety in the documentation or other materials
* provided with the distribution, and (3) all advertising materials mentioning
* features or use of this software display the following acknowledgement:
* ``This product includes software developed by Politecnico
* di Torino, and its contributors.'' (4) Neither the name of
* the University nor the names of its contributors may be used to endorse
* or promote products derived from this software without specific prior
* written permission.
* THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
* This product includes code licensed from RSA Data Security (Java Secure Socket Extension)
*
* Sun grants you ("Licensee") a non-exclusive, royalty free, license to use, and redistribute
* this software graphics artwork, as individual graphics or as a collection, as part of
* software code or programs that you develop, provided that
* i) this copyright notice and license accompany the software graphics artwork; and
* ii) you do not utilize the software graphics artwork in a manner which is disparaging to Sun.
* Unless enforcement is prohibited by applicable law, you may not modify the graphics,
* and must use them true to color and unmodified in every way.
*
*<p>
*/
package Utility;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Vector;
import java.util.Enumeration;

import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.AttributesImpl;

import org.apache.xalan.serialize.SerializerToXML;

public class XMLProcessor extends DefaultHandler {
	/**
	 * Contiene i componenti del bus model associato al progetto che si sta elaborando
	 */
	static private Vector businessModelComponents=null;
	/**
	 * Identifica un particolare bus comp di cui che si stanno elaborando le informazioni associate
	 */
	static private short busModCompID=-1;
	/**
	 * Utilizzato per memorizzare le informazioni in fase di lettura
	 */
	static private Vector data=null;
	/**
	 * Di volta in volta contiene le informazioni che si recuperano da un file XML
	 */
	static private String[] arrayOfData=null;
	/**
	 * Indica che la lettura del file si trova all'interno delle risorse associate
	 */
	 static private boolean isInsideRESOURCES=false;
	 /**
	  * Indica il gruppo di risorse che si sta considerando
	  */
	  static private short busModCompIDLink = -1;

    private XMLProcessor(Vector businessModelComponents) {
		super();
		this.businessModelComponents=businessModelComponents;
    }

	/**
	 * Carica le informazioni contenute in file XML all'interno del corrispondente progetto
	 */
	static public void LoadProject(File prjFile, Vector businessModelComponents)  throws PolitellaException{

		data = new Vector();
		// Use an instance of ourselves as the SAX event handler
		DefaultHandler handler = new XMLProcessor(businessModelComponents);
		// Use the default (non-validating) parser
		SAXParserFactory factory = SAXParserFactory.newInstance();
		try {
		    SAXParser saxParser = factory.newSAXParser();
			saxParser.parse( prjFile, handler );
		}catch (Throwable t) {
			t.printStackTrace();
			throw new PolitellaException(PolitellaException.NO_PROJECT_LOAD,prjFile.getAbsolutePath());
		}
	}

	/**
	 * Salva le informazioni contenute in un progetto nel corrispondente file XML
	 */
	static public void SaveProject(String prjDecr, String[] prjType,File prjFile, Vector businessModelComponents) throws PolitellaException{
	    SerializerToXML serToXML = new SerializerToXML();
	    AttributesImpl attrList = new AttributesImpl();
		try {
			serToXML.setOutputStream(new FileOutputStream(prjFile));
			//scrivo l'intestazione
			serToXML.startDocument();
			//scrivo il riferimento alla DTD
			serToXML.startDTD("POLITELLA_PROJECT",null,"../../politella.dtd");
			serToXML.endDTD();
			//scrivo la root POLITELLA_PROJECT
			serToXML.startElement("","","POLITELLA_PROJECT",attrList);
			//scrivo l'elemento DESCRIPTION
			serToXML.startElement("","","DESCRIPTION",attrList);
			serToXML.characters(prjDecr.toCharArray(),0,prjDecr.length());
			//termino l'elemento DESCRIPTION
			serToXML.endElement("","","DESCRIPTION");
			//scrivo l'elemento COMMERCIAL_MODEL
			serToXML.startElement("","","COMMERCIAL_MODEL",attrList);
			serToXML.characters(prjType[0].toCharArray(),0,prjType[0].length());
			//termino l'elemento COMMERCIAL_MODEL
			serToXML.endElement("","","COMMERCIAL_MODEL");
			//scrivo l'elemento ARCHITECTURAL_MODEL
			serToXML.startElement("","","ARCHITECTURAL_MODEL",attrList);
			serToXML.characters(prjType[1].toCharArray(),0,prjType[1].length());
			//termino l'elemento ARCHITECTURAL_MODEL
			serToXML.endElement("","","ARCHITECTURAL_MODEL");
			//scrivo l'elemento BUSINESS_MODEL
			serToXML.startElement("","","BUSINESS_MODEL",attrList);
			//inizio a gestire i singoli compoenti del business model
			Enumeration busCompList = businessModelComponents.elements();
			busModCompID=-1;
			while(busCompList.hasMoreElements())    {
				BusinessModelComponent currBusComp = (BusinessModelComponent)busCompList.nextElement();
				busModCompID++;
				if(currBusComp.IsActive())  {
					//fa parte del business model devo scriverlo
					//mi occupo dell'elemento BUSINESS_ROLE o BUSINESS_RELATIONSHIP
					String elementName=null;
					if(currBusComp.IsBusinessRole())
						elementName=new String("BUSINESS_ROLE");
					else
						elementName=new String("BUSINESS_RELATIONSHIP");
					attrList.addAttribute("","","IS_ACTIVE","(TRUE|FALSE)","TRUE");
					attrList.addAttribute("","","ID_BUS_COMP","ID","ID"+busModCompID);
					serToXML.startElement("","",elementName,attrList);
					attrList.clear();
					arrayOfData= new String[1];
					if((arrayOfData[0]=currBusComp.GetBusModCompDescription())!=null)    {
						//c' una descrizione associata, devo scriverla
						serToXML.startElement("","","DESCRIPTION",attrList);
						serToXML.characters(arrayOfData[0].toCharArray(),0,arrayOfData[0].length());
						serToXML.endElement("","","DESCRIPTION");
						arrayOfData=null;
					}
					if((arrayOfData=currBusComp.GetBusModCompSourceLoc())!=null)    {
						//ci sono dei sorgenti, devo scriverlo
						serToXML.startElement("","","SOURCE_LOCATION",attrList);
						for(int y=0;y<arrayOfData.length;y++)   {
							serToXML.characters(arrayOfData[y].toCharArray(),0,arrayOfData[y].length());
						    serToXML.characters("\n".toCharArray(),0,"\n".length());
						}
						serToXML.endElement("","","SOURCE_LOCATION");
					}
					if((arrayOfData=currBusComp.GetBusModCompBinaryLoc())!=null)    {
						//ci sono dei binari, devo scriverlo
						serToXML.startElement("","","BINARY_LOCATION",attrList);
						for(int y=0;y<arrayOfData.length;y++)   {
							serToXML.characters(arrayOfData[y].toCharArray(),0,arrayOfData[y].length());
						    serToXML.characters("\n".toCharArray(),0,"\n".length());
						}
						serToXML.endElement("","","BINARY_LOCATION");
					}
					if((arrayOfData=currBusComp.GetBusModCompDocLoc())!=null)    {
						//c' della documentazione, devo scriverlo
						serToXML.startElement("","","DOCUMENTATION_LOCATION",attrList);
						for(int y=0;y<arrayOfData.length;y++)   {
							serToXML.characters(arrayOfData[y].toCharArray(),0,arrayOfData[y].length());
						    serToXML.characters("\n".toCharArray(),0,"\n".length());
						}
						serToXML.endElement("","","DOCUMENTATION_LOCATION");
					}

					if(currBusComp.IsBusinessRole())    {
						// 1 bus role, ci possono essere delle risorse associate
						serToXML.startElement("","","RESOURCES",attrList);

						short[] busModCompIDLinks = currBusComp.GetBusModCompIDLinks();
						for(int i=0;i<busModCompIDLinks.length;i++) {
							attrList.addAttribute("","","ID_BUS_COMP","ID","ID"+busModCompIDLinks[i]);
							serToXML.startElement("","","ID_BUS_COMP_LINK",attrList);
						    attrList.clear();

							arrayOfData= new String[1];
							if((arrayOfData[0]=currBusComp.GetBusModComp_1_ResDescription(busModCompIDLinks[i]))!=null)    {
								//c' una descrizione associata
								serToXML.startElement("","","DESCRIPTION",attrList);
								serToXML.characters(arrayOfData[0].toCharArray(),0,arrayOfData[0].length());
								serToXML.endElement("","","DESCRIPTION");
								arrayOfData=null;
							}

							if((arrayOfData=currBusComp.GetBusModComp_1_ResSourceLoc(busModCompIDLinks[i]))!=null)    {
								//ci sono dei sorgenti, devo scriverlo
								serToXML.startElement("","","SOURCE_LOCATION",attrList);
								for(int y=0;y<arrayOfData.length;y++)   {
									serToXML.characters(arrayOfData[y].toCharArray(),0,arrayOfData[y].length());
								    serToXML.characters("\n".toCharArray(),0,"\n".length());
								}
								serToXML.endElement("","","SOURCE_LOCATION");
							}
							if((arrayOfData=currBusComp.GetBusModComp_1_ResBinaryLoc(busModCompIDLinks[i]))!=null)    {
								//ci sono dei binari, devo scriverlo
								serToXML.startElement("","","BINARY_LOCATION",attrList);
								for(int y=0;y<arrayOfData.length;y++)   {
									serToXML.characters(arrayOfData[y].toCharArray(),0,arrayOfData[y].length());
								    serToXML.characters("\n".toCharArray(),0,"\n".length());
								}
								serToXML.endElement("","","BINARY_LOCATION");
							}
							if((arrayOfData=currBusComp.GetBusModComp_1_ResDocLoc(busModCompIDLinks[i]))!=null)    {
								//c' della documentazione, devo scriverlo
								serToXML.startElement("","","DOCUMENTATION_LOCATION",attrList);
								for(int y=0;y<arrayOfData.length;y++)   {
									serToXML.characters(arrayOfData[y].toCharArray(),0,arrayOfData[y].length());
								    serToXML.characters("\n".toCharArray(),0,"\n".length());
								}
								serToXML.endElement("","","DOCUMENTATION_LOCATION");
							}

							serToXML.endElement("","","ID_BUS_COMP_LINK");
						}

						serToXML.endElement("","","RESOURCES");
					}
					//se  1 bus rela non ci possono essere delle risorse direttamente associate,
					//ci sono ma appartengono ai bus role con cui  relazionata la bus rela

					//termino l'elemento BUSINESS_ROLE o BUSINESS_RELATIONSHIP
					serToXML.endElement("","",elementName);
				}//chiudo l'if
				//se non faceva parte del business model passo subito all'elemento successivo
			}//chiudo il while sui componenti
				//termino l'elemento BUSINESS_MODEL
			serToXML.endElement("","","BUSINESS_MODEL");
			//termino la root
			serToXML.endElement("","","POLITELLA_PROJECT");
			//termino il documento
			serToXML.endDocument();
		}catch(IOException e){
			throw new PolitellaException(PolitellaException.NO_PROJECT_SAVE,prjFile.getAbsolutePath());
		}catch(SAXException e)    {
			throw new PolitellaException(PolitellaException.NO_PROJECT_SAVE,prjFile.getAbsolutePath());
		}
	}

	public void endElement(String uri, String localName,String qName) throws SAXException	{
//		System.out.println(qName);
//		System.out.println(this.data.toString());
		Object[] arrayOfData=this.data.toArray();
	    String[] infoToTx = new String[arrayOfData.length];
		for(int i=0;i<arrayOfData.length;i++)
			infoToTx[i]=(String)arrayOfData[i];

		if((qName.compareTo("BUSINESS_ROLE")==0) ||
			(qName.compareTo("BUSINESS_RELATIONSHIP")==0))  {
				//  terminato un Business Role o
				//  terminato una Business Relationship
			this.busModCompID=-1;
			this.data=new Vector();
			return;
		}
		if((qName.compareTo("DESCRIPTION")==0) && this.busModCompID!=-1)   {
			if(this.isInsideRESOURCES)
				// terminato il riferimento alla descrizione di uno dei gruppi di risorse del business component corrente
				((BusinessModelComponent)this.businessModelComponents.elementAt(this.busModCompID/*-1*/)).SetBusModComp_1_ResDecription((String)this.data.firstElement(),this.busModCompIDLink);
			else
				// terminato il riferimento alla descrizione del business component corrente
				((BusinessModelComponent)this.businessModelComponents.elementAt(this.busModCompID/*-1*/)).SetBusModCompDecription((String)this.data.firstElement());
			this.data=new Vector();
			return;
		}
		if(qName.compareTo("SOURCE_LOCATION")==0)   {
			if(this.isInsideRESOURCES)
				// terminato il riferimento ai sorgente di uno dei gruppi di risorse del business component corrente
				((BusinessModelComponent)this.businessModelComponents.elementAt(this.busModCompID/*-1*/)).SetBusModComp_1_ResSourceLoc(infoToTx,this.busModCompIDLink);
			else
				// terminato il riferimento ai sorgente del business component corrente
				((BusinessModelComponent)this.businessModelComponents.elementAt(this.busModCompID/*-1*/)).SetBusModCompSourceLoc(infoToTx);
			this.data=new Vector();
			return;
		}
		if(qName.compareTo("BINARY_LOCATION")==0)   {
			if(this.isInsideRESOURCES)
				// terminato il riferimento ai binari di uno dei gruppi di risorse del business component corrente
				((BusinessModelComponent)this.businessModelComponents.elementAt(this.busModCompID/*-1*/)).SetBusModComp_1_ResBinaryLoc(infoToTx,this.busModCompIDLink);
			else
				// terminato il riferimento ai binari del business component corrente
				((BusinessModelComponent)this.businessModelComponents.elementAt(this.busModCompID/*-1*/)).SetBusModCompBinaryLoc(infoToTx);
			this.data=new Vector();
			return;
		}
		if(qName.compareTo("DOCUMENTATION_LOCATION")==0)   {
//			System.out.println("ciao"+(String[])this.data.toArray());
			if(this.isInsideRESOURCES)
				// terminato il riferimento alla documentazione di uno dei gruppi di risorse del business component corrente
				((BusinessModelComponent)this.businessModelComponents.elementAt(this.busModCompID/*-1*/)).SetBusModComp_1_ResDocLoc(infoToTx,this.busModCompIDLink);
			else
				// terminato il riferimento alla documentazione del business component corrente
				((BusinessModelComponent)this.businessModelComponents.elementAt(this.busModCompID/*-1*/)).SetBusModCompDocLoc(infoToTx);
			this.data=new Vector();
			return;
		}
		if(qName.compareTo("RESOURCES")==0) {
			// terminata la sezione RESOURCES
			this.isInsideRESOURCES=false;
			//ho caricato i vari gruppi di risorse, ora devo recuperare i singoli busModCompIDLink e
			//metterli nel vettore busModCompIDLinks
			((BusinessModelComponent)this.businessModelComponents.get(this.busModCompID)).SetBusModCompIDLinks();
			return;
		}
		if(qName.compareTo("ID_BUS_COMP_LINK")==0)  {
			// terminato uno dei gruppi di risorse
			this.busModCompIDLink=-1;
			return;
		}
		this.data=new Vector();
	}

	public void startElement(String namespaceURI,String sName,String qName,Attributes attrs) throws SAXException	{
	// sName = simple name (localName) // qName = qualified name
		if((qName.compareTo("BUSINESS_ROLE")==0) ||
			(qName.compareTo("BUSINESS_RELATIONSHIP")==0))  {
			//ho individuato un nuovo Business Role
			//ho individuato una nuova Business Relationship
			String busModCompID = attrs.getValue("ID_BUS_COMP");
			this.busModCompID=(new Short(busModCompID.substring(2))).shortValue();
			if(attrs.getValue("IS_ACTIVE").compareTo("TRUE")==0)
				// i componenti sono numerati da 0 e il vettore parte da 0
				((BusinessModelComponent)this.businessModelComponents.elementAt(this.busModCompID/*-1*/)).SetIsActive(true);
			if(qName.compareTo("BUSINESS_ROLE")==0)
				((BusinessModelComponent)this.businessModelComponents.elementAt(this.busModCompID/*-1*/)).SetIsBusinessRole(true);
			else
				((BusinessModelComponent)this.businessModelComponents.elementAt(this.busModCompID/*-1*/)).SetIsBusinessRole(false);
		    return;
		}
		if(qName.compareTo("RESOURCES")==0) {
		// iniziato l'elenco delle risorse
			this.isInsideRESOURCES=true;
			return;
		}
		if(qName.compareTo("ID_BUS_COMP_LINK")==0) {
		//sono dentro ad uno dei gruppi di risorse
		    String busModCompIDLink= attrs.getValue("ID_BUS_COMP");
			this.busModCompIDLink= (new Short(busModCompIDLink.substring(2))).shortValue();
		    ((BusinessModelComponent)this.businessModelComponents.elementAt(this.busModCompID)).SetBusModComp_1_Resource(this.busModCompIDLink);
			return;
		}
	}

    public void characters(char buf[], int offset, int len) throws SAXException {
		String newData=(new String(buf, offset, len)).trim();
		if(newData.compareTo("")!=0)
			this.data.add(newData);
		//questo if pu essere true quando si associano pi file allo stesso tipo di risorsa,
		//in fase di scrittura questi vengono scritti separandoli con un \n
	}
}